@@ -20,9 +20,7 @@
 #ifndef _dblib_h_
 #define _dblib_h_
 
-#if defined(__GNUC__) && __GNUC__ >= 4 && !defined(__MINGW32__)
-#pragma GCC visibility push(hidden)
-#endif
+#include <freetds/pushvis.h>
 
 #ifdef __cplusplus
 extern "C"
@@ -32,8 +30,6 @@ extern "C"
 #endif
 #endif
 
-/* $Id: dblib.h,v 1.46 2010/01/25 23:05:58 freddy77 Exp $ */
-
 typedef enum tag_DB_RESULT_STATE {
 	  _DB_RES_INIT
 	, _DB_RES_RESULTSET_EMPTY
@@ -63,17 +59,16 @@ typedef struct tag_DBPROC_ROWBUF
 typedef struct
 {
 	int host_column;
-	int datatype;
+	TDS_SERVER_TYPE datatype;
 	int prefix_len;
 	DBINT column_len;
 	BYTE *terminator;
 	int term_len;
 	int tab_colnum;
 	int column_error;
-	BCPCOLDATA *bcp_column_data;
 } BCP_HOSTCOLINFO;
 
-typedef struct 
+typedef struct
 {
 	TDS_CHAR *hostfile;
 	TDS_CHAR *errorfile;
@@ -94,7 +89,7 @@ typedef struct _DBREMOTE_PROC_PARAM
 
 	char *name;
 	BYTE status;
-	int type;
+	TDS_SERVER_TYPE type;
 	DBINT maxlen;
 	DBINT datalen;
 	BYTE *value;
@@ -109,15 +104,12 @@ typedef struct _DBREMOTE_PROC
 	DBREMOTE_PROC_PARAM *param_list;
 } DBREMOTE_PROC;
 
-#define MAXOPTTEXT    32
-
-struct dboption
+typedef struct dboption
 {
-	char text[MAXOPTTEXT];
+	const char *text;
 	DBSTRING *param;
 	DBBOOL factive;
-};
-typedef struct dboption DBOPTION;
+} DBOPTION;
 
 typedef struct _null_representation
 {
@@ -129,7 +121,7 @@ struct tds_dblib_dbprocess
 {
 	TDSSOCKET *tds_socket;
 
-	TDS_INT row_type;
+	STATUS row_type;
 	DBPROC_ROWBUF row_buf;
 
 	int noautofree;
@@ -155,7 +147,7 @@ struct tds_dblib_dbprocess
 	FILE *ftos;
 	DB_DBCHKINTR_FUNC chkintr;
 	DB_DBHNDLINTR_FUNC hndlintr;
-	
+
 	/** boolean use ms behaviour */
 	int msdblib;
 
@@ -165,15 +157,29 @@ struct tds_dblib_dbprocess
 	NULLREP		nullreps[MAXBINDTYPES];
 };
 
+enum {
+#if MSDBLIB
+	dblib_msdblib = 1
+#else
+	dblib_msdblib = 0
+#endif
+};
+
 /*
  * internal prototypes
  */
+RETCODE dbgetnull(DBPROCESS *dbproc, int bindtype, int varlen, BYTE* varaddr);
+void copy_data_to_host_var(DBPROCESS * dbproc, TDS_SERVER_TYPE srctype, const BYTE * src, DBINT srclen,
+			   BYTE * dest, DBINT destlen,
+			   int bindtype, DBINT *indicator);
+
 int dbperror (DBPROCESS *dbproc, DBINT msgno, long errnum, ...);
 int _dblib_handle_info_message(const TDSCONTEXT * ctxptr, TDSSOCKET * tdsptr, TDSMESSAGE* msgptr);
 int _dblib_handle_err_message(const TDSCONTEXT * ctxptr, TDSSOCKET * tdsptr, TDSMESSAGE* msgptr);
 int _dblib_check_and_handle_interrupt(void * vdbproc);
 
 void _dblib_setTDS_version(TDSLOGIN * tds_login, DBINT version);
+void _dblib_convert_err(DBPROCESS * dbproc, TDS_INT len);
 
 DBINT _convert_char(int srctype, BYTE * src, int destype, BYTE * dest, DBINT destlen);
 DBINT _convert_intn(int srctype, BYTE * src, int destype, BYTE * dest, DBINT destlen);
@@ -185,11 +191,12 @@ extern MHANDLEFUNC _dblib_msg_handler;
 extern EHANDLEFUNC _dblib_err_handler;
 
 #define CHECK_PARAMETER(x, msg, ret)	if (!(x)) { dbperror(dbproc, (msg), 0); return ret; }
-#define CHECK_DBPROC() CHECK_PARAMETER(dbproc, SYBENULL, FAIL)
 #define CHECK_NULP(x, func, param_num, ret)	if (!(x)) { dbperror(dbproc, SYBENULP, 0, func, (int) param_num); return ret; }
 #define CHECK_PARAMETER_NOPROC(x, msg)	if (!(x)) { dbperror(NULL, (msg), 0); return FAIL; }
 #define DBPERROR_RETURN(x, msg)	if (x) { dbperror(dbproc, (msg), 0); return FAIL; }
 #define DBPERROR_RETURN3(x, msg, a, b, c)	if (x) { dbperror(dbproc, (msg), 0, a, b, c); return FAIL; }
+#define CHECK_CONN(ret) do { CHECK_PARAMETER(dbproc, SYBENULL, (ret)); \
+	if (IS_TDSDEAD(dbproc->tds_socket)) { dbperror(dbproc, SYBEDDNE, 0); return (ret); } } while(0)
 
 
 #ifdef __cplusplus
@@ -199,8 +206,6 @@ extern EHANDLEFUNC _dblib_err_handler;
 }
 #endif
 
-#if defined(__GNUC__) && __GNUC__ >= 4 && !defined(__MINGW32__)
-#pragma GCC visibility pop
-#endif
+#include <freetds/popvis.h>
 
 #endif
