@@ -582,6 +582,8 @@ typedef struct tds_login
 	DSTR dump_file;
 	int debug_flags;
 	int text_size;
+	DSTR routing_address;
+	TDS_USMALLINT routing_port;
 
 	unsigned char option_flag2;
 
@@ -1230,6 +1232,7 @@ struct tds_socket
 	TDSCURSOR *cur_cursor;		/**< cursor in use */
 	TDS_TINYINT bulk_query;		/**< true is query sent was a bulk query so we need to switch state to QUERYING */
 	TDS_TINYINT has_status; 	/**< true is ret_status is valid */
+	bool in_row;			/**< true if we are getting rows */
 	TDS_INT ret_status;     	/**< return status from store procedure */
 	TDS_STATE state;
 	volatile 
