@@ -17,9 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
+#undef NDEBUG
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <assert.h>
@@ -53,8 +52,7 @@
 #include <sybdb.h>
 #include "replacements.h"
 
-static char software_version[] = "$Id: bsqldb.c,v 1.49 2011/03/13 21:32:46 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/sysdep_private.h>
 
 #ifdef _WIN32
 #define NULL_DEVICE "NUL:"
@@ -72,8 +70,17 @@ static int get_printable_size(int type, int size);
 static void usage(const char invoked_as[]);
 
 struct METADATA { char *name, *format_string; const char *source; int type, size, width; };
+struct DATA { char *buffer; int status; };
+struct METACOMP { int numalts; struct METADATA *meta; struct DATA *data; };
 static int set_format_string(struct METADATA * meta, const char separator[]);
 
+
+typedef struct KEY_T
+{
+	size_t nkeys;
+	int *keys;
+} KEY_T;
+
 typedef struct _options 
 { 
 	int 	fverbose, 
@@ -88,8 +95,15 @@ typedef struct _options
 	char	*input_filename, 
 		*output_filename, 
 		*error_filename; 
+	struct pivot_t {
+		KEY_T row_key, col_key;
+		int val_col;
+		DBPIVOT_FUNC func;
+	} pivot;
 } OPTIONS;
 
+static void parse_pivot_description(OPTIONS *options, const char *optarg);
+
 LOGINREC* get_login(int argc, char *argv[], OPTIONS *poptions);
 
 /* global variables */
@@ -175,15 +189,15 @@ main(int argc, char *argv[])
 
 	fprintf(options.verbose, "%s:%d: Verbose operation enabled\n", options.appname, __LINE__);
 	
+	/* Select the specified database, if any */
+	if (options.database)
+		DBSETLDBNAME(login, options.database);
+
 	/* 
 	 * Connect to the server 
 	 */
 	if ((dbproc = dbopen(login, options.servername)) == NULL)
 		return 1;
-	
-	/* Switch to the specified database, if any */
-	if (options.database)
-		dbuse(dbproc, options.database);
 
 	/* 
 	 * Read the queries and write the results
@@ -238,6 +252,10 @@ next_query(DBPROCESS *dbproc)
 	while (fgets(query_line, sizeof(query_line), stdin)) {
 		/* 'go' or 'GO' separates command batches */
 		const char *p = query_line;
+
+		/* Skip past leading white spaces */
+		while (isspace((unsigned char) *p))
+			p++;
 		if (strncasecmp(p, "go", 2) == 0) {
 			for (p+=2; isspace((unsigned char) *p); p++) {
 				if (*p == '\n')
@@ -267,6 +285,54 @@ next_query(DBPROCESS *dbproc)
 	return 1;
 }
 
+static void
+free_metadata(struct METADATA **pmetadata, int ncols)
+{
+	int c;
+	struct METADATA *metadata = *pmetadata;
+
+	if (!metadata) return;
+	for (c=0; c < ncols; c++) {
+		free(metadata[c].format_string);
+	}
+	free(metadata);
+	*pmetadata = NULL;
+}
+
+static void
+free_data(struct DATA **pdata, int ncols)
+{
+	int c;
+	struct DATA *data = *pdata;
+
+	if (!data) return;
+	for (c=0; c < ncols; c++) {
+		free(data[c].buffer);
+	}
+	free(data);
+	*pdata = NULL;
+}
+
+static void
+free_metacomp(struct METACOMP ***pmetacompute, int ncomputeids)
+{
+	int i, c;
+	struct METACOMP **metacompute = *pmetacompute;
+
+	if (!metacompute) return;
+	for (i=0; i < ncomputeids; i++) {
+		for (c=0; c < metacompute[i]->numalts; c++) {
+			free(metacompute[i]->meta[c].name);
+			free(metacompute[i]->meta[c].format_string);
+		}
+		free(metacompute[i]->meta);
+		free(metacompute[i]->data);
+		free(metacompute[i]);
+	}
+	free(metacompute);
+	*pmetacompute = NULL;
+}
+
 static void
 print_results(DBPROCESS *dbproc) 
 {
@@ -278,15 +344,16 @@ print_results(DBPROCESS *dbproc)
 	
 	struct METADATA *metadata = NULL, return_status;
 	
-	struct DATA { char *buffer; int status; } *data = NULL;
+	struct DATA *data = NULL;
 	
-	struct METACOMP { int numalts; struct METADATA *meta; struct DATA *data; } **metacompute = NULL;
+	struct METACOMP **metacompute = NULL;
 	
 	RETCODE erc;
 	int row_code;
 	int i, c, ret;
 	int iresultset;
 	int ncomputeids = 0, ncols = 0;
+	
 
 	/* 
 	 * If using default column separator, we want columns to line up vertically, 
@@ -303,33 +370,25 @@ print_results(DBPROCESS *dbproc)
 	for (iresultset=1; (erc = dbresults(dbproc)) != NO_MORE_RESULTS; iresultset++) {
 		if (erc == FAIL) {
 			fprintf(stderr, "%s:%d: dbresults(), result set %d failed\n", options.appname, __LINE__, iresultset);
+			free_metadata(&metadata, ncols);
+			free_data(&data, ncols);
+			free_metacomp(&metacompute, ncomputeids);
 			return;
 		}
 		
+		if (options.pivot.func) {
+			const KEY_T *rk = &options.pivot.row_key, *ck = &options.pivot.col_key;
+			erc = dbpivot(dbproc, rk->nkeys, rk->keys, ck->nkeys, ck->keys, 
+					options.pivot.func, options.pivot.val_col);
+		}
+		
 		fprintf(options.verbose, "Result set %d\n", iresultset);
 		/* Free prior allocations, if any. */
-		fprintf(options.verbose, "Freeing prior allocations\n");
-		for (c=0; c < ncols; c++) {
-			free(metadata[c].format_string);
-			free(data[c].buffer);
-		}
-		free(metadata);
-		metadata = NULL;
-		free(data);
-		data = NULL;
+		free_metadata(&metadata, ncols);
+		free_data(&data, ncols);
 		ncols = 0;
-		
-		for (i=0; i < ncomputeids; i++) {
-			for (c=0; c < metacompute[i]->numalts; c++) {
-				free(metacompute[i]->meta[c].name);
-				free(metacompute[i]->meta[c].format_string);
-			}
-			free(metacompute[i]->meta);
-			free(metacompute[i]->data);
-			free(metacompute[i]);
-		}
-		free(metacompute);
-		metacompute = NULL;
+
+		free_metacomp(&metacompute, ncomputeids);
 		ncomputeids = 0;
 		
 		/* 
@@ -403,7 +462,7 @@ print_results(DBPROCESS *dbproc)
 			ret = set_format_string(&metadata[c], (c+1 < ncols)? options.colsep : "\n");
 			if (ret <= 0) {
 				fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
-				return;
+				exit(1);
 			}
 
 			/* 
@@ -418,19 +477,19 @@ print_results(DBPROCESS *dbproc)
 			 */
 
 			if (metadata[c].width < INT_MAX) {
-				data[c].buffer = calloc(1, 1 + metadata[c].width); /* allow for null terminator */
+				data[c].buffer = (char *) calloc(1, 1 + metadata[c].width); /* allow for null terminator */
 				assert(data[c].buffer);
 
 				erc = dbbind(dbproc, c+1, bindtype, 0, (BYTE *) data[c].buffer);
 				if (erc == FAIL) {
 					fprintf(stderr, "%s:%d: dbbind(), column %d failed\n", options.appname, __LINE__, c+1);
-					return;
+					exit(1);
 				}
 
 				erc = dbnullbind(dbproc, c+1, &data[c].status);
 				if (erc == FAIL) {
 					fprintf(stderr, "%s:%d: dbnullbind(), column %d failed\n", options.appname, __LINE__, c+1);
-					return;
+					exit(1);
 				}
 			} else {
 				/* We don't bind text buffers, but use dbreadtext instead. */
@@ -468,7 +527,7 @@ print_results(DBPROCESS *dbproc)
 										(iby+1 < nby)? ", " : ")");
 					if (ret < 0) {
 						fprintf(options.verbose, "Insufficient room to create name for column %d:\n", 1+c);
-						break;
+						exit(1);
 					}
 					free(bynames);
 					bynames = s;
@@ -481,9 +540,12 @@ print_results(DBPROCESS *dbproc)
 					colname = metadata[--altcolid].name;
 				}
 
-				asprintf(&metacompute[i]->meta[c].name, "%s(%s)", dbprtype(dbaltop(dbproc, i+1, c+1)), colname);
-				assert(metacompute[i]->meta[c].name);
-					
+				ret = asprintf(&metacompute[i]->meta[c].name, "%s(%s)", dbprtype(dbaltop(dbproc, i+1, c+1)), colname);
+				if (ret < 0) {
+					fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
+					exit(1);
+				}
+
 				metacompute[i]->meta[c].width = get_printable_size(metacompute[i]->meta[c].type, 
 										   metacompute[i]->meta[c].size);
 				if (metacompute[i]->meta[c].width < strlen(metacompute[i]->meta[c].name))
@@ -493,7 +555,7 @@ print_results(DBPROCESS *dbproc)
 				if (ret <= 0) {
 					free(bynames);
 					fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
-					return;
+					exit(1);
 				}
 				
 				fprintf(options.verbose, "\tcolumn %d is %s, type %s, size %d %s\n", 
@@ -503,14 +565,14 @@ print_results(DBPROCESS *dbproc)
 	
 				/* allocate buffer */
 				assert(metacompute[i]->data);
-				metacompute[i]->data[c].buffer = calloc(1, metacompute[i]->meta[c].width);
+				metacompute[i]->data[c].buffer = (char *) calloc(1, metacompute[i]->meta[c].width);
 				assert(metacompute[i]->data[c].buffer);
 				
 				/* bind */
 				erc = dbaltbind(dbproc, i+1, c+1, bindtype, -1, (BYTE*) metacompute[i]->data[c].buffer);
 				if (erc == FAIL) {
 					fprintf(stderr, "%s:%d: dbaltbind(), column %d failed\n", options.appname, __LINE__, c+1);
-					return;
+					exit(1);
 				}
 			}
 		}
@@ -541,11 +603,24 @@ print_results(DBPROCESS *dbproc)
 						size_t len = dbdatlen(dbproc, c+1);
 						if (len == 0) {
 							fputs("NULL", stdout);
-						} else if (fwrite(p, len, 1, stdout) != 1) {
-							perror("could not write to output file");
-							exit(EXIT_FAILURE);
+						} else {
+							BYTE *pend = p + len;
+							switch(dbcoltype(dbproc, c+1)) {
+							case SYBTEXT:
+								if (fwrite(p, len, 1, stdout) != 1) {
+									perror("could not write to output file");
+									exit(EXIT_FAILURE);
+								}
+								break;
+							default:	/* image, binary */
+								fprintf(stdout, "0x");
+								for (; p < pend; p++) {
+									printf("%02hx", (unsigned short int)*p);
+								}
+								break;
+							}
 						}
-						fprintf(stdout, metadata[c].format_string); /* col/row separator */
+						fprintf(stdout, metadata[c].format_string, ""); /* col/row separator */
 						continue;
 					}
 					switch (data[c].status) { /* handle nulls */
@@ -579,7 +654,7 @@ print_results(DBPROCESS *dbproc)
 					struct METADATA *meta = &metacompute[row_code-1]->meta[c];
 					
 					/* left justify the names */
-					strcat(fmt, &meta->format_string[1]);
+					strlcat(fmt, &meta->format_string[1], sizeof(fmt));
 					fprintf(options.headers, fmt, meta->name);
 				}
 
@@ -655,6 +730,9 @@ print_results(DBPROCESS *dbproc)
 		}
 	} /* wend dbresults */
 	fprintf(options.verbose, "%s:%d: dbresults() returned NO_MORE_RESULTS (%d):\n", options.appname, __LINE__, erc);
+	free_metadata(&metadata, ncols);
+	free_data(&data, ncols);
+	free_metacomp(&metacompute, ncomputeids);
 }
 
 static int
@@ -799,6 +877,86 @@ static void unescape(char arg[])
 	}
 }
 
+/* pivot syntax: down cols, across cols, function, value */
+
+static void
+parse_pivot_description(OPTIONS *options, const char *optarg)
+{
+/**
+	KEY_T { size_t nkeys; int *keys; };
+	struct pivot_t {
+		KEY_T row_key, col_key;
+		int val_col;
+		DBPIVOT_FUNC func;
+**/
+	char *p, *pend;
+	KEY_T *keys[2] = { &options->pivot.row_key, &options->pivot.col_key},**pk;
+	int nchars;
+	char ch, *input = strdup(optarg);
+	assert(input);
+	
+	keys[0]->keys = NULL;
+	keys[1]->keys = NULL;
+	keys[0]->nkeys = 0;
+	keys[1]->nkeys = 0;
+	
+	for (p=input, pk=keys; pk < keys + 2; p++, pk++) {
+		if ((pend = strchr(p, ' ')) != NULL) {
+			unsigned col;
+			int ncols;
+
+			*pend = '\0';
+			while((ncols = sscanf(p, "%u%c%n", &col, &ch, &nchars)) > 0) {
+				int *pi;
+				assert(ncols <= 2);
+				if ((pi = (int *) realloc((*pk)->keys, sizeof((*pk)->keys[0]) * ++((*pk)->nkeys))) == NULL) {
+					assert(pi);
+					return;
+				}
+				(*pk)->keys = pi;
+				(*pk)->keys[(*pk)->nkeys - 1] = col;
+				if (ncols == 2) {
+					assert(nchars <= pend - p);
+					if (ch != ',')
+						fprintf(stderr, "surprised by %c\n", ch);
+					p += nchars;
+				} else {
+					p = pend;
+					break;
+				}
+			}
+		}
+	}
+	if (options->fverbose)
+		printf("found %lu row and %lu col keys\n",
+		       (unsigned long) options->pivot.row_key.nkeys,
+		       (unsigned long) options->pivot.col_key.nkeys);
+
+	if ((pend = strchr(p, ' ')) == NULL) {
+		fprintf(stderr, "bsqldb(): no name in %s\n", p);
+		free(input);
+		return;
+	}
+	*pend = '\0';
+	
+	if ((options->pivot.func = dbpivot_lookup_name(p)) == NULL) {
+		fprintf(stderr, "bsqldb(): invalid name in %s\n", p);
+		free(input);
+		return;
+	}
+	
+	p = ++pend;
+	if( sscanf(p, "%d%c%n", &options->pivot.val_col, &ch, &nchars) != 1) {
+		fprintf(stderr, "bsqldb(): could not parse value column %s\n", p);
+		free(input);
+		return;
+	}
+
+
+	free(input);
+}
+
+
 LOGINREC *
 get_login(int argc, char *argv[], OPTIONS *options)
 {
@@ -810,7 +968,7 @@ get_login(int argc, char *argv[], OPTIONS *options)
 
 	assert(options && argv);
 	
-	options->appname = tds_basename(argv[0]);
+	options->appname = basename(argv[0]);
 	options->colsep = default_colsep; /* may be overridden by -t */
 	
 	login = dblogin();
@@ -824,14 +982,18 @@ get_login(int argc, char *argv[], OPTIONS *options)
 	
 	options->servername = getenv("DSQUERY");
 	
-	while ((ch = getopt(argc, argv, "U:P:S:dD:i:o:e:t:H:hqv")) != -1) {
+	while ((ch = getopt(argc, argv, "U:P:R:S:d:D:i:o:e:t:H:hqv")) != -1) {
 		switch (ch) {
 		case 'U':
+			free(username);
 			username = strdup(optarg);
 			break;
+		case 'R': 
+			parse_pivot_description(options, optarg);
+			break;
 		case 'P':
-			password = strdup(optarg);
-			memset(optarg, 0, strlen(optarg));
+			free(password);
+			password = tds_getpassarg(optarg);
 			break;
 		case 'S':
 			options->servername = strdup(optarg);
@@ -857,7 +1019,7 @@ get_login(int argc, char *argv[], OPTIONS *options)
 			options->headers = stdout;
 			break;
 		case 'H':
-			strcpy(options->hostname, optarg);
+			strlcpy(options->hostname, optarg, sizeof(options->hostname));
 			break;
 		case 'q':
 			options->fquiet = 1;
@@ -909,14 +1071,17 @@ int
 err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr)
 {
 	if (dberr) {
-		fprintf(stderr, "%s: Msg %d, Level %d\n", options.appname, dberr, severity);
-		fprintf(stderr, "%s\n\n", dberrstr);
+		fprintf(stderr, "%s: Msg %d, Level %d", options.appname, dberr, severity);
+	} else {
+		fprintf(stderr, "%s: DB-LIBRARY error", options.appname);
 	}
 
-	else {
-		fprintf(stderr, "%s: DB-LIBRARY error:\n\t", options.appname);
-		fprintf(stderr, "%s\n", dberrstr);
-	}
+	if (oserr && oserrstr)
+		fprintf(stderr, " (OS error %d: %s)", oserr, oserrstr);
+	
+	printf("\n");
+		
+	fprintf(stderr, "%s\n\n", dberrstr);
 
 	return INT_CANCEL;
 }
