@@ -191,6 +191,10 @@ main(int argc, char *argv[])
 		}
 	}
 	
+	/* Select the specified database, if any */
+	if (options.database)
+		DBSETLDBNAME(login, options.database);
+
 	/* 
 	 * Connect to the server 
 	 */
@@ -200,10 +204,6 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	/* Switch to the specified database, if any */
-	if (options.database)
-		dbuse(dbproc, options.database);
-
 	/* 
 	 * Read the procedure names and move their texts.  
 	 */
@@ -220,14 +220,15 @@ main(int argc, char *argv[])
 					 " and		o.name = '%s'"
 					 " and		o.uid = user_id('%s')"
 					 " and		o.type not in ('U', 'S')" /* no user or system tables */
-					 " order by 	c.number, c.colid"
+					 " order by 	c.number, %sc.colid"
 					;
 		static const char query_table[] = " execute sp_help '%s.%s' ";
 
 		parse_argument(argv[i], &procedure);
 
-		erc = dbfcmd(dbproc, query, procedure.name, procedure.owner); 
-	
+		erc = dbfcmd(dbproc, query, procedure.name, procedure.owner,
+			     (DBTDS(dbproc) == DBTDS_5_0) ? "c.colid2, ":"");
+
 		/* Send the query to the server (we could use dbsqlexec(), instead) */
 		erc = dbsqlsend(dbproc);
 		if (erc == FAIL) {
@@ -552,7 +553,7 @@ print_results(DBPROCESS *dbproc)
 	
 	/* bound variables */
 	enum column_id { ctext=1, number=2 };
-	char sql_text[16000];
+	char sql_text[16002];
 	int	 sql_text_status;
 	int	 procedure_number; /* for create proc abc;2 */
 	int	 procedure_number_status;
