@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
+ * Copyright (C) 2011  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <ctype.h>
@@ -32,6 +31,14 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
+/* These should be in stdlib */
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#endif
+#ifndef EXIT_FAILURE
+#define EXIT_FAILURE 1
+#endif
+
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
@@ -48,15 +55,12 @@
 #include <locale.h>
 #endif
 
-#include "tds.h"
+#include <freetds/tds.h>
 #include "replacements.h"
 #include <sybfront.h>
 #include <sybdb.h>
 #include "freebcp.h"
 
-static char software_version[] = "$Id: freebcp.c,v 1.59 2011/03/13 21:32:49 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 void pusage(void);
 int process_parameters(int, char **, struct pd *);
 static int unescape(char arg[]);
@@ -70,6 +74,7 @@ int setoptions (DBPROCESS * dbproc, BCPPARAMDATA * params);
 int err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
 int msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname,
 		int line);
+static int set_bcp_hints(BCPPARAMDATA *pdata, DBPROCESS *pdbproc);
 
 int
 main(int argc, char **argv)
@@ -90,7 +95,7 @@ main(int argc, char **argv)
 	params.textsize = 4096;	/* our default text size is 4K */
 
 	if (process_parameters(argc, argv, &params) == FALSE) {
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 	if (getenv("FREEBCP")) {
 		fprintf(stderr, "User name: \"%s\"\n", params.user);
@@ -98,7 +103,7 @@ main(int argc, char **argv)
 
 
 	if (login_to_database(&params, &dbproc) == FALSE) {
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (!setoptions(dbproc, &params)) 
@@ -114,7 +119,7 @@ main(int argc, char **argv)
 		ok = FALSE;
 	}
 
-	exit((ok == TRUE) ? 0 : 1);
+	exit((ok == TRUE) ? EXIT_SUCCESS : EXIT_FAILURE);
 
 	return 0;
 }
@@ -186,7 +191,7 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 	}
 
 	/* argument 2 - the direction */
-	tds_strlcpy(pdata->dbdirection, argv[2], sizeof(pdata->dbdirection));
+	strlcpy(pdata->dbdirection, argv[2], sizeof(pdata->dbdirection));
 
 	if (strcasecmp(pdata->dbdirection, "in") == 0) {
 		pdata->direction = DB_IN;
@@ -207,11 +212,11 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 	 * Get the rest of the arguments 
 	 */
 	optind = 4; /* start processing options after table, direction, & filename */
-	while ((ch = getopt(argc, argv, "m:f:e:F:L:b:t:r:U:P:i:I:S:h:T:A:o:O:0:C:ncEdvV")) != -1) {
+	while ((ch = getopt(argc, argv, "m:f:e:F:L:b:t:r:U:P:i:I:S:h:T:A:o:O:0:C:ncEdvVD:")) != -1) {
 		switch (ch) {
 		case 'v':
 		case 'V':
-			printf("freebcp version %s\n", software_version);
+			printf("freebcp version %s\n", TDS_VERSION_NO);
 			return FALSE;
 			break;
 		case 'm':
@@ -267,18 +272,7 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 			break;
 		case 'P':
 			pdata->Pflag++;
-			if ((strcmp(optarg, "-")) == 0) {
-				char pwd[255], *nl;
-				memset(pwd, 0, 255);
-				fgets(pwd, 255, stdin);
-				nl = strchr(pwd, '\n');
-				if(nl) *nl = '\0';
-				pdata->pass = strdup(pwd);
-				memset(pwd, 0, 255);
-			} else {
-				pdata->pass = strdup(optarg);
-				memset(optarg, 0, strlen(optarg));
-			}
+			pdata->pass = tds_getpassarg(optarg);
 			break;
 		case 'i':
 			free(pdata->inputfile);
@@ -293,6 +287,9 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 			pdata->Sflag++;
 			pdata->server = strdup(optarg);
 			break;
+		case 'D':
+			pdata->dbname = strdup(optarg);
+			break;
 		case 'h':
 			pdata->hint = strdup(optarg);
 			break;
@@ -367,14 +364,14 @@ process_parameters(int argc, char **argv, BCPPARAMDATA *pdata)
 	if (pdata->inputfile) {
 		if (freopen(pdata->inputfile, "rb", stdin) == NULL) {
 			fprintf(stderr, "%s: unable to open %s: %s\n", "freebcp", pdata->inputfile, strerror(errno));
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	}
 
 	if (pdata->outputfile) {
 		if (freopen(pdata->outputfile, "wb", stdout) == NULL) {
 			fprintf(stderr, "%s: unable to open %s: %s\n", "freebcp", pdata->outputfile, strerror(errno));
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -428,6 +425,9 @@ login_to_database(BCPPARAMDATA * pdata, DBPROCESS ** pdbproc)
 		DBSETLPACKET(login, pdata->packetsize);
 	}
 
+	if (pdata->dbname)
+		DBSETLDBNAME(login, pdata->dbname);
+
 	/* Enable bulk copy for this connection. */
 
 	BCP_SETL(login, TRUE);
@@ -444,15 +444,6 @@ login_to_database(BCPPARAMDATA * pdata, DBPROCESS ** pdbproc)
 	dbloginfree(login);
 	login = NULL;
 
-	/* set hint if any */
-	if (pdata->hint) {
-		int erc = bcp_options(*pdbproc, BCPHINTS, (BYTE *) pdata->hint, strlen(pdata->hint));
-
-		if (erc != SUCCEED)
-			fprintf(stderr, "db-lib: Unable to set hint \"%s\"\n", pdata->hint);
-		return FALSE;
-	}
-
 	return (TRUE);
 
 }
@@ -465,6 +456,34 @@ file_character(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 	int li_numcols = 0;
 	RETCODE ret_code = 0;
 
+	if (FAIL == bcp_init(dbproc, pdata->dbobject, pdata->hostfilename, pdata->errorfile, dir))
+		return FALSE;
+
+	if (!set_bcp_hints(pdata, dbproc))
+		return FALSE;
+
+	if (pdata->Eflag) {
+
+		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
+	
+		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
+			printf("dbfcmd failed\n");
+			return FALSE;
+		}
+	
+		if (dbsqlexec(dbproc) == FAIL) {
+			printf("dbsqlexec failed\n");
+			return FALSE;
+		}
+
+		while (NO_MORE_RESULTS != dbresults(dbproc))
+			continue;
+	}
+
+	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
+	bcp_control(dbproc, BCPLAST, pdata->lastrow);
+	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
+
 	if (dir == DB_QUERYOUT) {
 		if (dbfcmd(dbproc, "SET FMTONLY ON %s SET FMTONLY OFF", pdata->dbobject) == FAIL) {
 			printf("dbfcmd failed\n");
@@ -493,30 +512,6 @@ file_character(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		return FALSE;
 	}
 
-	if (FAIL == bcp_init(dbproc, pdata->dbobject, pdata->hostfilename, pdata->errorfile, dir))
-		return FALSE;
-
-	if (pdata->Eflag) {
-
-		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
-	
-		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
-			printf("dbfcmd failed\n");
-			return FALSE;
-		}
-	
-		if (dbsqlexec(dbproc) == FAIL) {
-			printf("dbsqlexec failed\n");
-			return FALSE;
-		}
-
-		while (NO_MORE_RESULTS != dbresults(dbproc));
-	}
-
-	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
-	bcp_control(dbproc, BCPLAST, pdata->lastrow);
-	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
-
 	if (bcp_columns(dbproc, li_numcols) == FAIL) {
 		printf("Error in bcp_columns.\n");
 		return FALSE;
@@ -557,9 +552,36 @@ file_native(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 	int i;
 	int li_numcols = 0;
 	int li_coltype;
-	int li_collen;
 	RETCODE ret_code = 0;
 
+	if (FAIL == bcp_init(dbproc, pdata->dbobject, pdata->hostfilename, pdata->errorfile, dir))
+		return FALSE;
+
+	if (!set_bcp_hints(pdata, dbproc))
+		return FALSE;
+
+	if (pdata->Eflag) {
+
+		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
+	
+		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
+			printf("dbfcmd failed\n");
+			return FALSE;
+		}
+	
+		if (dbsqlexec(dbproc) == FAIL) {
+			printf("dbsqlexec failed\n");
+			return FALSE;
+		}
+
+		while (NO_MORE_RESULTS != dbresults(dbproc))
+			continue;
+	}
+
+	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
+	bcp_control(dbproc, BCPLAST, pdata->lastrow);
+	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
+
 	if (dir == DB_QUERYOUT) {
 		if (dbfcmd(dbproc, "SET FMTONLY ON %s SET FMTONLY OFF", pdata->dbobject) == FAIL) {
 			printf("dbfcmd failed\n");
@@ -588,30 +610,6 @@ file_native(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 		return FALSE;
 	}
 
-	if (FAIL == bcp_init(dbproc, pdata->dbobject, pdata->hostfilename, pdata->errorfile, dir))
-		return FALSE;
-
-	if (pdata->Eflag) {
-
-		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
-	
-		if (dbfcmd(dbproc, "set identity_insert %s on", pdata->dbobject) == FAIL) {
-			printf("dbfcmd failed\n");
-			return FALSE;
-		}
-	
-		if (dbsqlexec(dbproc) == FAIL) {
-			printf("dbsqlexec failed\n");
-			return FALSE;
-		}
-
-		while (NO_MORE_RESULTS != dbresults(dbproc));
-	}
-
-	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
-	bcp_control(dbproc, BCPLAST, pdata->lastrow);
-	bcp_control(dbproc, BCPMAXERRS, pdata->maxerrors);
-
 	if (bcp_columns(dbproc, li_numcols) == FAIL) {
 		printf("Error in bcp_columns.\n");
 		return FALSE;
@@ -619,7 +617,6 @@ file_native(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 
 	for (i = 1; i <= li_numcols; i++) {
 		li_coltype = dbcoltype(dbproc, i);
-		li_collen = dbcollen(dbproc, i);
 
 		if (bcp_colfmt(dbproc, i, li_coltype, -1, -1, NULL, -1, i) == FAIL) {
 			printf("Error in bcp_colfmt col %d\n", i);
@@ -649,6 +646,9 @@ file_formatted(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 	if (FAIL == bcp_init(dbproc, pdata->dbobject, pdata->hostfilename, pdata->errorfile, dir))
 		return FALSE;
 
+	if (!set_bcp_hints(pdata, dbproc))
+		return FALSE;
+
 	if (pdata->Eflag) {
 
 		bcp_control(dbproc, BCPKEEPIDENTITY, 1);
@@ -663,7 +663,8 @@ file_formatted(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 			return FALSE;
 		}
 
-		while (NO_MORE_RESULTS != dbresults(dbproc));
+		while (NO_MORE_RESULTS != dbresults(dbproc))
+			continue;
 	}
 
 	bcp_control(dbproc, BCPFIRST, pdata->firstrow);
@@ -688,9 +689,8 @@ file_formatted(BCPPARAMDATA * pdata, DBPROCESS * dbproc, DBINT dir)
 
 
 int
-setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params){
-	FILE *optFile;
-	char optBuf[256];
+setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params)
+{
 	RETCODE fOK;
 
 	if (dbfcmd(dbproc, "set textsize %d ", params->textsize) == FAIL) {
@@ -703,26 +703,30 @@ setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params){
 	 * Else pass the option verbatim to the server.
 	 */
 	if (params->options) {
+		FILE *optFile;
+		char optBuf[256];
+
 		if ((optFile = fopen(params->options, "r")) == NULL) {
-			if (dbfcmd(dbproc, params->options) == FAIL) {
+			if (dbcmd(dbproc, params->options) == FAIL) {
 				fprintf(stderr, "setoptions() failed preparing options at %s:%d\n", __FILE__, __LINE__);
 				return FALSE;
 			}
 		} else {
 			while (fgets (optBuf, sizeof(optBuf), optFile) != NULL) {
-				if (dbfcmd(dbproc, optBuf) == FAIL) {
+				if (dbcmd(dbproc, optBuf) == FAIL) {
 					fprintf(stderr, "setoptions() failed preparing options at %s:%d\n", __FILE__, __LINE__);
+					fclose(optFile);
 					return FALSE;
 				}
 			}
 			if (!feof (optFile)) {
 				perror("freebcp");
         			fprintf(stderr, "error reading options file \"%s\" at %s:%d\n", params->options, __FILE__, __LINE__);
+				fclose(optFile);
 				return FALSE;
 			}
 			fclose(optFile);
 		}
-	
 	}
 	
 	if (dbsqlexec(dbproc) == FAIL) {
@@ -731,7 +735,8 @@ setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params){
 	}
 	
 	while ((fOK = dbresults(dbproc)) == SUCCEED) {
-		while ((fOK = dbnextrow(dbproc)) == REG_ROW);
+		while ((fOK = dbnextrow(dbproc)) == REG_ROW)
+			continue;
 		if (fOK == FAIL) {
 			fprintf(stderr, "setoptions() failed sending options at %s:%d\n", __FILE__, __LINE__);
 			return FALSE;
@@ -745,14 +750,27 @@ setoptions(DBPROCESS * dbproc, BCPPARAMDATA * params){
 	return TRUE;
 }
 
+static int
+set_bcp_hints(BCPPARAMDATA *pdata, DBPROCESS *pdbproc)
+{
+	/* set hint if any */
+	if (pdata->hint) {
+		if (bcp_options(pdbproc, BCPHINTS, (BYTE *) pdata->hint, strlen(pdata->hint)) != SUCCEED) {
+			fprintf(stderr, "db-lib: Unable to set hint \"%s\"\n", pdata->hint);
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 void
 pusage(void)
 {
-	fprintf(stderr, "usage:  freebcp [[database_name.]owner.]table_name {in | out} datafile\n");
+	fprintf(stderr, "usage:  freebcp [[database_name.]owner.]table_name|query {in | out | queryout } datafile\n");
 	fprintf(stderr, "        [-m maxerrors] [-f formatfile] [-e errfile]\n");
 	fprintf(stderr, "        [-F firstrow] [-L lastrow] [-b batchsize]\n");
 	fprintf(stderr, "        [-n] [-c] [-t field_terminator] [-r row_terminator]\n");
-	fprintf(stderr, "        [-U username] [-P password] [-I interfaces_file] [-S server]\n");
+	fprintf(stderr, "        [-U username] [-P password] [-I interfaces_file] [-S server] [-D database]\n");
 	fprintf(stderr, "        [-v] [-d] [-h \"hint [,...]\" [-O \"set connection_option on|off, ...]\"\n");
 	fprintf(stderr, "        [-A packet size] [-T text or image size] [-E]\n");
 	fprintf(stderr, "        [-i input_file] [-o output_file]\n");
