@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Brian Bruns
- * Copyright (C) 2006, 2007, 2008, 2009  Frediano Ziglio
+ * Copyright (C) 2006-2015  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,20 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <freetds/time.h>
 
 #include <stdio.h>
 #include <assert.h>
@@ -62,6 +51,8 @@
 # include <unistd.h>
 #elif defined(_WIN32)
 # include <io.h>
+# undef isatty
+# define isatty(fd) _isatty(fd)
 #endif /* HAVE_UNISTD_H */
 
 /* HP-UX require some constants defined by limits.h */
@@ -81,13 +72,16 @@
 #include <langinfo.h>
 #endif /* HAVE_LANGINFO_H */
 
-#include "tds.h"
-#include "tdsiconv.h"
-#include "tdsstring.h"
-#include "tdsconvert.h"
-#include "replacements.h"
+#ifdef HAVE_LOCALCHARSET_H
+#include <localcharset.h>
+#endif /* HAVE_LOCALCHARSET_H */
 
-TDS_RCSID(var, "$Id: tsql.c,v 1.140 2011/03/22 17:54:09 jklowden Exp $");
+#include <freetds/tds.h>
+#include <freetds/iconv.h>
+#include <freetds/string.h>
+#include <freetds/convert.h>
+#include <freetds/data.h>
+#include "replacements.h"
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
 
@@ -108,9 +102,9 @@ static int global_opt_flags = 0;
 #define QUIET (global_opt_flags & OPT_QUIET)
 #define VERBOSE (global_opt_flags & OPT_VERBOSE)
 
-static char *opt_col_term = "\t";
-static char *opt_row_term = "\n";
-static char *opt_default_db = NULL;
+static const char *opt_col_term = "\t";
+static const char *opt_row_term = "\n";
+static const char *opt_default_db = NULL;
 
 static int do_query(TDSSOCKET * tds, char *buf, int opt_flags);
 static int get_opt_flags(char *s, int *opt_flags);
@@ -132,7 +126,7 @@ tsql_readline(char *prompt)
 
 	sz = 1024;
 	pos = 0;
-	line = (char*) malloc(sz);
+	line = tds_new(char, sz);
 	if (!line)
 		return NULL;
 
@@ -176,11 +170,31 @@ tsql_add_history(const char *s)
 #endif
 }
 
+/**
+ * Returns the version of the TDS protocol in effect for the link
+ * as a decimal integer.  
+ *	Typical returned values are 42, 50, 70, 80.
+ * Also fills pversion_string unless it is null.
+ * 	Typical pversion_string values are "4.2" and "7.0".
+ */
+static int
+tds_version(TDSCONNECTION * conn, char *pversion_string)
+{
+	int iversion = 0;
+
+	iversion = 10 * TDS_MAJOR(conn) + TDS_MINOR(conn);
+
+	sprintf(pversion_string, "%d.%d", TDS_MAJOR(conn), TDS_MINOR(conn));
+
+	return iversion;
+}
+
 static int
 do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 {
 	int rows = 0;
-	int rc, i;
+	TDSRET rc;
+	int i;
 	TDSCOLUMN *col;
 	int ctype;
 	CONV_RESULT dres;
@@ -192,12 +206,12 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 	char message[128];
 
 	rc = tds_submit_query(tds, buf);
-	if (rc != TDS_SUCCEED) {
+	if (TDS_FAILED(rc)) {
 		fprintf(stderr, "tds_submit_query() failed\n");
 		return 1;
 	}
 
-	while ((rc = tds_process_tokens(tds, &resulttype, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &resulttype, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 		const int stop_mask = TDS_STOPAT_ROWFMT|TDS_RETURN_DONE|TDS_RETURN_ROW|TDS_RETURN_COMPUTE;
 		if (opt_flags & OPT_TIMER) {
 			gettimeofday(&start, NULL);
@@ -208,7 +222,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 			if ((!(opt_flags & OPT_NOHEADER)) && tds->current_results) {
 				for (i = 0; i < tds->current_results->num_cols; i++) {
 					if (i) fputs(opt_col_term, stdout);
-					fputs(tds->current_results->columns[i]->column_name, stdout);
+					fputs(tds_dstr_cstr(&tds->current_results->columns[i]->column_name), stdout);
 				}
 				fputs(opt_row_term, stdout);
 			}
@@ -216,7 +230,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 		case TDS_COMPUTE_RESULT:
 		case TDS_ROW_RESULT:
 			rows = 0;
-			while ((rc = tds_process_tokens(tds, &resulttype, NULL, stop_mask)) == TDS_SUCCEED) {
+			while ((rc = tds_process_tokens(tds, &resulttype, NULL, stop_mask)) == TDS_SUCCESS) {
 				if (resulttype != TDS_ROW_RESULT && resulttype != TDS_COMPUTE_RESULT)
 					break;
 
@@ -242,7 +256,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 					srclen = col->column_cur_size;
 
 
-					if (tds_convert(tds->tds_ctx, ctype, (TDS_CHAR *) src, srclen, SYBVARCHAR, &dres) < 0)
+					if (tds_convert(tds_get_ctx(tds), ctype, (TDS_CHAR *) src, srclen, SYBVARCHAR, &dres) < 0)
 						continue;
 					if (print_rows)  {
 						if (i) fputs(opt_col_term, stdout);
@@ -268,14 +282,14 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 			char version[64];
 			int line = 0;
 
-			line = tds_version(tds, version);
+			line = tds_version(tds->conn, version);
 			if (line) {
 				TDSMESSAGE msg;
 				memset(&msg, 0, sizeof(TDSMESSAGE));
 				msg.server = "tsql";
 				sprintf(message, "using TDS version %s", version);
 				msg.message = message;
-				tsql_handle_message(tds->tds_ctx, tds, &msg);
+				tsql_handle_message(tds_get_ctx(tds), tds, &msg);
 			}
 		}
 		if (opt_flags & OPT_TIMER) {
@@ -287,12 +301,29 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 			memset(&msg, 0, sizeof(TDSMESSAGE));
 			msg.server = "tsql";
 			msg.message = message;
-			tsql_handle_message(tds->tds_ctx, tds, &msg);
+			tsql_handle_message(tds_get_ctx(tds), tds, &msg);
 		}
 	}
 	return 0;
 }
 
+static void
+tsql_print_usage(const char *progname)
+{
+	fprintf(stderr,
+		"Usage:\t%s [-a <appname>] [-S <server> | -H <hostname> -p <port>] -U <username> [-P <password>] [-I <config file>] [-o <options>] [-t delim] [-r delim] [-D database]\n"
+		"\t%s -C\n"
+		"Options:\n"
+		"\tf\tDo not print footer\n"
+		"\th\tDo not print header\n"
+		"\tt\tPrint time informations\n"
+		"\tv\tPrint TDS version\n"
+		"\tq\tQuiet\n\n"
+		"\tDelimiters can be multi-char strings appropriately escaped for your shell.\n"
+		"\tDefault column delimitor is <tab>; default row delimiter is <newline>\n",
+		progname, progname);
+}
+
 static void
 reset_getopt(void)
 {
@@ -318,7 +349,7 @@ get_opt_flags(char *s, int *opt_flags)
 
 	/* make sure we have enough elements */
 	assert(s && opt_flags);
-	argv = (char **) calloc(strlen(s) + 2, sizeof(char*));
+	argv = tds_new0(char*, strlen(s) + 2);
 	if (!argv)
 		return 0;
 
@@ -360,14 +391,17 @@ get_opt_flags(char *s, int *opt_flags)
 static int
 get_default_instance_port(const char hostname[])
 {
-	char ip[24] = {'\0'};
+	int port;
+	struct addrinfo *addr;
 	
-	tds_lookup_host(hostname, ip);
-	
-	if (!*ip) 
+	if ((addr = tds_lookup_host(hostname)) == NULL)
 		return 0;
+
+	port = tds7_get_instance_port(addr, "MSSQLSERVER");
+
+	freeaddrinfo(addr);
 	
-	return tds7_get_instance_port(ip, "MSSQLSERVER");
+	return port;
 }
 
 #if !defined(LC_ALL)
@@ -392,20 +426,24 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	char *hostname = NULL, *servername = NULL;
 	char *username = NULL, *password = NULL;
 	char *confile = NULL;
+	const char *appname = "TSQL";
 	int opt, port=0, use_domain_login=0;
-	const char *charset = NULL;
+	char *charset = NULL;
 	char *opt_flags_str = NULL;
 
-	while ((opt = getopt(argc, argv, "H:S:I:J:P:U:p:Co:t:r:D:Lv")) != -1) {
+	while ((opt = getopt(argc, argv, "a:H:S:I:J:P:U:p:Co:t:r:D:Lv")) != -1) {
 		switch (opt) {
+		case 'a':
+			appname = optarg;
+			break;
 		case 't':
-			opt_col_term = strdup(optarg);
+			opt_col_term = optarg;
 			break;
 		case 'r':
-			opt_row_term = strdup(optarg);
+			opt_row_term = optarg;
 			break;
 		case 'D':
-			opt_default_db = strdup(optarg);
+			opt_default_db = optarg;
 			break;
 		case 'o':
 			opt_flags_str = optarg;
@@ -424,14 +462,14 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			break;
 		case 'P':
 			free(password);
-			password = strdup(optarg);
-			memset(optarg, 0, strlen(optarg));
+			password = tds_getpassarg(optarg);
 			break;
 		case 'I':
 			free(confile);
 			confile = strdup(optarg);
 			break;
 		case 'J':
+			free(charset);
 			charset = strdup(optarg);
 			break;
 		case 'p':
@@ -445,7 +483,7 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			break;
 		case 'C':
 			settings = tds_get_compiletime_settings();
-			printf("%s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n",
+			printf("%s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n%35s: %s\n",
 			       "Compile-time settings (established with the \"configure\" script)",
 			       "Version", settings->freetds_version,
 			       "freetds.conf directory", settings->sysconfdir, 
@@ -459,18 +497,21 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			       "iODBC", settings->iodbc ? "yes" : "no", 
 			       "unixodbc", settings->unixodbc ? "yes" : "no", 
 			       "SSPI \"trusted\" logins", have_sspi, 
-			       "Kerberos", enable_krb5);
+			       "Kerberos", enable_krb5,
+			       "OpenSSL", settings->openssl ? "yes" : "no",
+			       "GnuTLS", settings->gnutls ? "yes" : "no",
+			       "MARS", settings->mars ? "yes" : "no");
 			exit(0);
 			break;
 		default:
-			fprintf(stderr, "%s: error: invalid option %c\n", argv[0], (char)opt);
+			tsql_print_usage(argv[0]);
 			exit(1);
 			break;
 		}
 	}
 
 	if (opt_flags_str != NULL) {
-		char *minus_flags = malloc(strlen(opt_flags_str) + 5);
+		char *minus_flags = tds_new(char, strlen(opt_flags_str) + 5);
 		if (minus_flags != NULL) {
 			strcpy(minus_flags, "go -");
 			strcat(minus_flags, opt_flags_str);
@@ -480,19 +521,19 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	}
 
 	if ((global_opt_flags & OPT_INSTANCES) && hostname) {
-		static const char template[] = "%s.instances";
-		char ip[24] = {'\0'};
+		struct addrinfo *addr;
 		char *filename = getenv("TDSDUMP");
 
 		if (filename) {
-			if ((filename = malloc(sizeof(template) + strlen(filename))) == NULL) 
+			if (asprintf(&filename, "%s.instances", filename) < 0)
 				exit(1);
-			sprintf(filename, template, getenv("TDSDUMP"));
 			tdsdump_open(filename);
 			free(filename);
 		}
-		tds_lookup_host(hostname, ip);
-		tds7_get_instance_ports(stderr, ip);
+		if ((addr = tds_lookup_host(hostname)) != NULL) {
+			tds7_get_instance_ports(stderr, addr);
+			freeaddrinfo(addr);
+		}
 		tdsdump_close();
 		exit(0);
 	}
@@ -524,11 +565,11 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	}
 	/* A NULL username indicates a domain (trusted) login */
 	if (!username) {
-		username = calloc(1, 1);
+		username = tds_new0(char, 1);
 		use_domain_login = 1;
 	}
 	if (!password) {
-		password = calloc(1, 128);
+		password = tds_new0(char, 128);
 		if (!use_domain_login)
 			readpassphrase("Password: ", password, 128, RPP_ECHO_OFF);
 	}
@@ -542,29 +583,27 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	}
 
 	/* all validated, let's do it */
+	if (!tds_set_user(login, username)
+	    || !tds_set_app(login, appname)
+	    || !tds_set_library(login, "TDS-Library")
+	    || !tds_set_language(login, "us_english")
+	    || !tds_set_passwd(login, password))
+		goto out_of_memory;
+	if (charset && !tds_set_client_charset(login, charset))
+		goto out_of_memory;
 
 	/* if it's a servername */
 	if (servername) {
-		tds_set_user(login, username);
-		tds_set_app(login, "TSQL");
-		tds_set_library(login, "TDS-Library");
-		tds_set_server(login, servername);
-		if (charset) tds_set_client_charset(login, charset);
-		tds_set_language(login, "us_english");
-		tds_set_passwd(login, password);
+		if (!tds_set_server(login, servername))
+			goto out_of_memory;
 		if (confile) {
 			tds_set_interfaces_file_loc(confile);
 		}
 		/* else we specified hostname/port */
 	} else {
-		tds_set_user(login, username);
-		tds_set_app(login, "TSQL");
-		tds_set_library(login, "TDS-Library");
-		tds_set_server(login, hostname);
+		if (!tds_set_server(login, hostname))
+			goto out_of_memory;
 		tds_set_port(login, port);
-		if (charset) tds_set_client_charset(login, charset);
-		tds_set_language(login, "us_english");
-		tds_set_passwd(login, password);
 	}
 
 	memset(password, 0, strlen(password));
@@ -575,6 +614,12 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	free(username);
 	free(password);
 	free(servername);
+	free(charset);
+	return;
+
+out_of_memory:
+	fprintf(stderr, "%s: out of memory\n", argv[0]);
+	exit(1);
 }
 
 static int
@@ -585,8 +630,14 @@ tsql_handle_message(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * ms
 		return 0;
 	}
 
-	if (msg->msgno != 5701 && msg->msgno != 5703
-	    && msg->msgno != 20018) {
+	switch (msg->msgno) {
+	case 5701: 	/* changed_database */
+	case 5703: 	/* changed_language */
+	case 20018:	/* The @optional_command_line is too long */
+		if (VERBOSE)
+			fprintf(stderr, "%s\n", msg->message);
+		break;
+	default:
 		fprintf(stderr, "Msg %d (severity %d, state %d) from %s", 
 			msg->msgno, msg->severity, msg->state, msg->server);
 		if (msg->proc_name && strlen(msg->proc_name))
@@ -594,6 +645,7 @@ tsql_handle_message(const TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * ms
 		if (msg->line_number > 0)
 			fprintf(stderr, " Line %d", msg->line_number);
 		fprintf(stderr, ":\n\t\"%s\"\n", msg->message);
+		break;
 	}
 
 	return 0;
@@ -638,13 +690,13 @@ slurp_input_file(char *fname, char **mybuf, size_t *bufsz, size_t *buflen, int *
 }
 
 static void
-print_instance_data(TDSCONNECTION *connection) 
+print_instance_data(TDSLOGIN *login) 
 {
-	if (!connection)
+	if (!login)
 		return;
 	
-	if (!tds_dstr_isempty(&connection->instance_name))
-		printf("connecting to instance %s on port %d\n", tds_dstr_cstr(&connection->instance_name), connection->port);
+	if (!tds_dstr_isempty(&login->instance_name))
+		printf("connecting to instance %s on port %d\n", tds_dstr_cstr(&login->instance_name), login->port);
 }
 
 int
@@ -659,7 +711,7 @@ main(int argc, char **argv)
 	TDSSOCKET *tds;
 	TDSLOGIN *login;
 	TDSCONTEXT *context;
-	TDSCONNECTION *connection;
+	TDSLOGIN *connection;
 	int opt_flags = 0;
 #if HAVE_FORK
 	pid_t timer_pid = 0;
@@ -678,7 +730,7 @@ main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 
 	/* grab a login structure */
-	login = tds_alloc_login();
+	login = tds_alloc_login(1);
 
 	context = tds_alloc_context(NULL);
 	if (context->locale && !context->locale->date_fmt) {
@@ -697,6 +749,8 @@ main(int argc, char **argv)
 	assert(tds);
 	tds_set_parent(tds, NULL);
 	connection = tds_read_config_info(tds, login, context->locale);
+	if (!connection)
+		return 1;
 
 	locale = setlocale(LC_ALL, NULL);
 
@@ -718,14 +772,17 @@ main(int argc, char **argv)
 		if (!charset)
 			charset = "ISO-8859-1";
 
-		tds_set_client_charset(login, charset);
-		tds_dstr_dup(&connection->client_charset, &login->client_charset);
+		if (!tds_set_client_charset(login, charset))
+			return 1;
+		if (!tds_dstr_dup(&connection->client_charset, &login->client_charset))
+			return 1;
 	}
 	if (!QUIET) printf("using default charset \"%s\"\n", tds_dstr_cstr(&connection->client_charset));
 
 	if (opt_default_db) {
-		tds_dstr_copy(&connection->database, opt_default_db);
-		if (!QUIET) fprintf(stderr, "Default database being set to %s\n", opt_default_db);
+		if (!tds_dstr_copy(&connection->database, opt_default_db))
+			return 1;
+		if (!QUIET) fprintf(stderr, "Setting %s as default database in login packet\n", opt_default_db);
 	}
 
 	/* 
@@ -768,8 +825,8 @@ main(int argc, char **argv)
 		close(pipes[0]);
 	}
 #endif
-	if (!connection || tds_connect_and_login(tds, connection) != TDS_SUCCEED) {
-		if( VERBOSE ) 
+	if (!connection || TDS_FAILED(tds_connect_and_login(tds, connection))) {
+		if (VERBOSE)
 			print_instance_data(connection);
 		tds_free_socket(tds);
 		tds_free_login(login);
@@ -779,10 +836,10 @@ main(int argc, char **argv)
 	}
 	if (VERBOSE) 
 		print_instance_data(connection);
-	tds_free_connection(connection);
+	tds_free_login(connection);
 	/* give the buffer an initial size */
 	bufsz = 4096;
-	mybuf = malloc(bufsz);
+	mybuf = tds_new(char, bufsz);
 	mybuf[0] = '\0';
 	buflen = 0;
 
@@ -839,7 +896,7 @@ main(int argc, char **argv)
 		if (!strcasecmp(cmd, "exit") || !strcasecmp(cmd, "quit") || !strcasecmp(cmd, "bye"))
 			break;
 		if (!strcasecmp(cmd, "version")) {
-			tds_version(tds, mybuf);
+			tds_version(tds->conn, mybuf);
 			printf("using TDS version %s\n", mybuf);
 			line = 0;
 			mybuf[0] = '\0';
@@ -856,7 +913,7 @@ main(int argc, char **argv)
 			while (buflen + strlen(s) + 2 > bufsz) {
 				char *newbuf; 
 				bufsz *= 2;
-				if ((newbuf = realloc(mybuf, bufsz)) == NULL) {
+				if ((newbuf = (char *) realloc(mybuf, bufsz)) == NULL) {
 					perror("tsql: ");
 					exit(1);
 				}
@@ -872,6 +929,7 @@ main(int argc, char **argv)
 
 	/* close up shop */
 	free(mybuf);
+	tds_close_socket(tds);
 	tds_free_socket(tds);
 	tds_free_login(login);
 	tds_free_context(context);
