@@ -1,7 +1,4 @@
-
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 
@@ -12,9 +9,6 @@
 #include <ctpublic.h>
 #include "common.h"
 
-static char software_version[] = "$Id: ct_cursors.c,v 1.3 2006/01/30 15:31:56 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 int
 main(int argc, char **argv)
 {
@@ -39,7 +33,9 @@ main(int argc, char **argv)
 	CS_INT datalength;
 	CS_CHAR text[128];
 	CS_INT num_cols, i;
-	int is_return_status = 0;
+
+	memset(col1, 0, sizeof(col1));
+	memset(col2, 0, sizeof(col2));
 
 	fprintf(stdout, "%s: use multiple cursors on the same connection\n", __FILE__);
 
@@ -131,14 +127,17 @@ main(int argc, char **argv)
 
 	if (ret != CS_SUCCEED) {
 		fprintf(stderr, "ct_send failed\n");
+		return 1;
 	}
 
 	while ((results_ret = ct_results(cmd, &result_type)) == CS_SUCCEED) {
 		switch ((int) result_type) {
 
+		case CS_CMD_FAIL:
+			fprintf(stderr, "ct_results failed\n");
+			return 1;
 		case CS_CMD_SUCCEED:
 		case CS_CMD_DONE:
-		case CS_CMD_FAIL:
 		case CS_STATUS_RESULT:
 			break;
 
@@ -167,7 +166,6 @@ main(int argc, char **argv)
 
 				if (datafmt.status & CS_RETURN) {
 					fprintf(stdout, "ct_describe() column %d \n", i);
-					is_return_status = i + 1;
 				}
 
 				datafmt.datatype = CS_CHAR_TYPE;
@@ -229,9 +227,11 @@ main(int argc, char **argv)
 	while ((results_ret = ct_results(cmd2, &result_type)) == CS_SUCCEED) {
 		switch ((int) result_type) {
 
+		case CS_CMD_FAIL:
+			fprintf(stderr, "ct_results failed\n");
+			return 1;
 		case CS_CMD_SUCCEED:
 		case CS_CMD_DONE:
-		case CS_CMD_FAIL:
 		case CS_STATUS_RESULT:
 			break;
 
@@ -260,7 +260,6 @@ main(int argc, char **argv)
 
 				if (datafmt2.status & CS_RETURN) {
 					fprintf(stdout, "ct_describe() column %d \n", i);
-					is_return_status = i + 1;
 				}
 
 				datafmt2.datatype = CS_CHAR_TYPE;
