@@ -1,6 +1,4 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -12,15 +10,13 @@
 #include <ctpublic.h>
 #include "common.h"
 
-static char software_version[] = "$Id: get_send_data.c,v 1.5 2007/09/17 08:46:02 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+static CS_CONNECTION *conn = NULL;
 
 /* Testing: Retrieve CS_TEXT_TYPE using ct_bind() */
 int
 main(int argc, char **argv)
 {
 	CS_CONTEXT *ctx;
-	CS_CONNECTION *conn;
 	CS_COMMAND *cmd;
 	int i, verbose = 0;
 
@@ -48,6 +44,8 @@ main(int argc, char **argv)
 	char *textptr;
 	CS_IODESC iodesc;
 
+	int tds_version;
+
 	len600[0] = 0;
 	name[0] = 0;
 	for (i = 0; i < 60; i++) {
@@ -74,6 +72,15 @@ main(int argc, char **argv)
 		return 1;
 	}
 
+	ret = ct_con_props(conn, CS_GET, CS_TDS_VERSION, &tds_version, CS_UNUSED, NULL);
+	if (ret == CS_SUCCEED) {
+		if (tds_version >= CS_TDS_72) {
+			printf("Protocol TDS7.2+ detected, test not supported\n");
+			try_ctlogout(ctx, conn, cmd, verbose);
+			return 0;
+		}
+	}
+
 	ret = run_command(cmd, "CREATE TABLE #test_table (id int, name text)");
 	if (ret != CS_SUCCEED)
 		return 1;
@@ -139,11 +146,11 @@ main(int argc, char **argv)
 						fprintf(stdout, "id = '%d'\n", id);
 					}
 
-                    nameptr = name;
-                    while ((ret2 = ct_get_data(cmd, 2 , nameptr, 200, &getlen )) == CS_SUCCEED) {
-                        nameptr += getlen;
+					nameptr = name;
+					while ((ret2 = ct_get_data(cmd, 2 , nameptr, 200, &getlen )) == CS_SUCCEED) {
+						nameptr += getlen;
 					}
-                    if (ret2 != CS_END_DATA) {
+					if (ret2 != CS_END_DATA) {
 						fprintf(stderr, "ct_get_data() failed\n");
 						return 1;
 					}
@@ -154,7 +161,7 @@ main(int argc, char **argv)
 					}
 					fprintf(stdout, "%s: Trying ct_data_info on text column\n", __FILE__);
 
-                    if (ct_data_info(cmd, CS_GET, 2, &iodesc) != CS_SUCCEED) {
+					if (ct_data_info(cmd, CS_GET, 2, &iodesc) != CS_SUCCEED) {
 						fprintf(stderr, "ct_data_info() failed\n");
 						return 1;
 					} else {
@@ -199,7 +206,7 @@ main(int argc, char **argv)
 	if ((ret = ct_command(cmd, CS_SEND_DATA_CMD, NULL, CS_UNUSED, CS_COLUMN_DATA)) != CS_SUCCEED) {
 		fprintf(stderr, "ct_command(CS_SEND_DATA_CMD) failed.\n");
 		return 1;
-	}               
+	}
 
 	iodesc.total_txtlen = 800;
 	iodesc.log_on_update = CS_TRUE;
