@@ -1,17 +1,13 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
 #include <stdio.h>
+#include <assert.h>
 #include <ctpublic.h>
 
-static char software_version[] = "$Id: t0006.c,v 1.14 2010/10/05 08:36:36 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 CS_CONTEXT *ctx;
 static int allSuccess = 1;
 
@@ -38,6 +34,8 @@ DoTest(
 	char buffer[1024];
 	const char *err = "";
 
+	assert(tolen >= 0);
+
 	memset(&destfmt, 0, sizeof(destfmt));
 	destfmt.datatype = totype;
 	destfmt.maxlength = tomaxlen;
@@ -75,7 +73,7 @@ DoTest(
 
 	/* test buffer */
 	if (todata && memcmp(todata, buffer, tolen) != 0) {
-		unsigned n;
+		int n;
 		for (n = 0; n < tolen; ++n)
 			printf("%02x ", ((unsigned char*)todata)[n]);
 		printf("\n");
