@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -29,35 +27,26 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include <tds.h>
+#if HAVE_ERRNO_H
+# include <errno.h>
+#endif /* HAVE_ERRNO_H */
+
+#include <freetds/tds.h>
+#include <freetds/convert.h>
 #include <sybdb.h>
 #include <syberror.h>
 #include <dblib.h>
 
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: dbutil.c,v 1.47.2.3 2011/06/09 13:02:49 freddy77 Exp $");
-
 /*
  * test include consistency 
  * I don't think all compiler are able to compile this code... if not comment it
  */
 #if ENABLE_EXTRA_CHECKS
 
-#if defined(__GNUC__) && __GNUC__ >= 2
-#define COMPILE_CHECK(name,check) \
-    extern int name[(check)?1:-1] __attribute__ ((unused))
-#else
-#define COMPILE_CHECK(name,check) \
-    extern int name[(check)?1:-1]
-#endif
-
 /* TODO test SYBxxx consistency */
 
 #define TEST_ATTRIBUTE(t,sa,fa,sb,fb) \
-	COMPILE_CHECK(t,sizeof(((sa*)0)->fa) == sizeof(((sb*)0)->fb) && (TDS_INTPTR)(&((sa*)0)->fa) == (TDS_INTPTR)(&((sb*)0)->fb))
+	TDS_COMPILE_CHECK(t,sizeof(((sa*)0)->fa) == sizeof(((sb*)0)->fb) && TDS_OFFSET(sa,fa) == TDS_OFFSET(sb,fb))
 
 TEST_ATTRIBUTE(t21,TDS_MONEY4,mny4,DBMONEY4,mny4);
 TEST_ATTRIBUTE(t22,TDS_OLD_MONEY,mnyhigh,DBMONEY,mnyhigh);
@@ -80,7 +69,7 @@ TEST_ATTRIBUTE(t30,TDS_NUMERIC,array,DBNUMERIC,array);
 int
 _dblib_handle_info_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
-	DBPROCESS *dbproc = (tds && tds->parent)? (DBPROCESS *) tds->parent : NULL;
+	DBPROCESS *dbproc = (tds && tds_get_parent(tds))? (DBPROCESS *) tds_get_parent(tds) : NULL;
 
 	tdsdump_log(TDS_DBG_FUNC, "_dblib_handle_info_message(%p, %p, %p)\n", tds_ctx, tds, msg);
 	tdsdump_log(TDS_DBG_FUNC, "msgno %d: \"%s\"\n", msg->msgno, msg->message);
@@ -105,9 +94,9 @@ _dblib_handle_info_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSA
 		 */
 		/* Cannot call dbperror() here because server messsage numbers (and text) are not in its lookup table. */
 		static const char message[] = "General SQL Server error: Check messages from the SQL Server";
-		(*_dblib_err_handler)(dbproc, msg->severity, msg->msgno, DBNOERR, (char *) message, NULL);
+		(*_dblib_err_handler)(dbproc, msg->severity, SYBESMSG, DBNOERR, (char *) message, NULL);
 	}
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 /** \internal
@@ -137,7 +126,7 @@ _dblib_handle_info_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSA
 int
 _dblib_handle_err_message(const TDSCONTEXT * tds_ctx, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
-	DBPROCESS *dbproc = (tds && tds->parent)? (DBPROCESS *) tds->parent : NULL;
+	DBPROCESS *dbproc = (tds && tds_get_parent(tds))? (DBPROCESS *) tds_get_parent(tds) : NULL;
 	int rc = INT_CANCEL;
 
 	assert(_dblib_err_handler);
@@ -203,9 +192,9 @@ _dblib_check_and_handle_interrupt(void * vdbproc)
 {
 	DBPROCESS * dbproc = (DBPROCESS*) vdbproc;
 	int ret = INT_CONTINUE;
-	
-	assert( ! (dbproc == NULL && DBDEAD(dbproc)) );  /* a non-process can't be a dead process */
-	
+
+	assert( dbproc != NULL );
+
 	if (dbproc->chkintr == NULL || dbproc->hndlintr == NULL)
 		return INT_CONTINUE;
 		
@@ -247,3 +236,27 @@ _dblib_setTDS_version(TDSLOGIN * tds_login, DBINT version)
 		break;
 	}
 }
+
+void
+_dblib_convert_err(DBPROCESS * dbproc, TDS_INT len)
+{
+	switch (len) {
+	case TDS_CONVERT_NOAVAIL:
+		dbperror(dbproc, SYBERDCN, 0);
+		break;
+	case TDS_CONVERT_SYNTAX:
+		dbperror(dbproc, SYBECSYN, 0);
+		break;
+	case TDS_CONVERT_NOMEM:
+		dbperror(dbproc, SYBEMEM, ENOMEM);
+		break;
+	case TDS_CONVERT_OVERFLOW:
+		dbperror(dbproc, SYBECOFL, 0);
+		break;
+	default:
+	case TDS_CONVERT_FAIL:
+		dbperror(dbproc, SYBECINTERNAL, 0);
+		break;
+	}
+}
+
