@@ -12,30 +12,27 @@
 #include <sys/param.h>
 #endif /* HAVE_SYS_PARAM_H */
 
-#ifndef DBNTWIN32
+#include <freetds/time.h>
+
+#if HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif /* HAVE_SYS_RESOURCE_H */
+
 #include "replacements.h"
+
+#if !defined(PATH_MAX)
+#define PATH_MAX 256
 #endif
 
-static char software_version[] = "$Id: common.c,v 1.23 2007/11/30 08:55:13 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-typedef struct _tag_memcheck_t
-{
-	int item_number;
-	unsigned int special;
-	struct _tag_memcheck_t *next;
-}
-memcheck_t;
-
-
-static memcheck_t *breadcrumbs = NULL;
-static int num_breadcrumbs = 0;
-static const unsigned int BREADCRUMB = 0xABCD7890;
-
 char USER[512];
 char SERVER[512];
 char PASSWORD[512];
 char DATABASE[512];
+
+static char sql_file[PATH_MAX];
+static FILE* input_file = NULL;
+
+static char *ARGV0 = NULL;
 static char *DIRNAME = NULL;
 static const char *BASENAME = NULL;
 
@@ -73,6 +70,11 @@ tds_dirname(char* path)
 	p2 = strrchr(p, '\\');
 	if (p2)
 		p = p2;
+	if (p == path) {
+		if (*p == '/' || *p == '\\')
+			return "\\";
+		return ".";
+	}
 	*p = 0;
 	return path;
 }
@@ -80,28 +82,69 @@ tds_dirname(char* path)
 
 #endif
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 512
-#endif
+char free_file_registered = 0;
+static void
+free_file(void)
+{
+	if (input_file) {
+		fclose(input_file);
+		input_file = NULL;
+	}
+	if (ARGV0) {
+		DIRNAME = NULL;
+		BASENAME = NULL;
+		free(ARGV0);
+		ARGV0 = NULL;
+	}
+}
 
 int
 read_login_info(int argc, char **argv)
 {
-	extern char *optarg;
-	extern int optind;
-	
-	FILE *in;
+	int len;
+	FILE *in = NULL;
 #if !defined(__MINGW32__) && !defined(_MSC_VER)
 	int ch;
 #endif
 	char line[512];
 	char *s1, *s2;
-	char filename[MAXPATHLEN];
+	char filename[PATH_MAX];
 	static const char *PWD = "../../../PWD";
 	struct { char *username, *password, *servername, *database; char fverbose; } options;
+
+#if defined(HAVE_SETRLIMIT) && defined(RLIMIT_STACK)
+#define MAX_STACK (8*1024*1024)
+
+	struct rlimit rlim;
+
+	if (!getrlimit(RLIMIT_STACK, &rlim) && (rlim.rlim_cur == RLIM_INFINITY || rlim.rlim_cur > MAX_STACK)) {
+		rlim.rlim_cur = MAX_STACK;
+		setrlimit(RLIMIT_STACK, &rlim);
+	}
+#endif
+
+	setbuf(stdout, NULL);
+	setbuf(stderr, NULL);
+
+	free(ARGV0);
+#ifdef __VMS
+	{
+		/* basename expects unix format */
+		s1 = strrchr(argv[0], ';'); /* trim version; extension trimmed later */
+		if (s1) *s1 = 0;
+		const char *unixspec = decc$translate_vms(argv[0]);
+		ARGV0 = strdup(unixspec);
+	}
+#else
+	ARGV0 = strdup(argv[0]);
+#endif
 	
-	BASENAME = tds_basename((char *)argv[0]);
-	DIRNAME = dirname((char *)argv[0]);
+	BASENAME = basename(ARGV0);
+#if defined(_WIN32) || defined(__VMS)
+	s1 = strrchr(BASENAME, '.');
+	if (s1) *s1 = 0;
+#endif
+	DIRNAME = dirname(ARGV0);
 	
 	memset(&options, 0, sizeof(options));
 	
@@ -140,8 +183,12 @@ read_login_info(int argc, char **argv)
 	}
 #endif
 	strcpy(filename, PWD);
-	
-	in = fopen(filename, "r");
+
+	s1 = getenv("TDSPWDFILE");
+	if (s1 && s1[0])
+		in = fopen(s1, "r");
+	if (!in)
+		in = fopen(filename, "r");
 	if (!in)
 		in = fopen("PWD", "r");
 	if (!in) {
@@ -150,7 +197,7 @@ read_login_info(int argc, char **argv)
 		in = fopen(filename, "r");
 		if (!in) {
 			fprintf(stderr, "Can not open %s file\n\n", filename);
-			return 1;
+			goto Override;
 		}
 	}
 
@@ -171,6 +218,7 @@ read_login_info(int argc, char **argv)
 	}
 	fclose(in);
 	
+	Override:
 	/* apply command-line overrides */
 	if (options.username) {
 		strcpy(USER, options.username);
@@ -188,75 +236,96 @@ read_login_info(int argc, char **argv)
 		strcpy(DATABASE, options.database);
 		free(options.database);
 	}
+
+	if (!*SERVER) {
+		fprintf(stderr, "no servername provided, quitting.\n");
+		exit(1);
+	}
 	
 	printf("found %s.%s for %s in \"%s\"\n", SERVER, DATABASE, USER, filename);
+	
+#if 0
+	dbrecftos(BASENAME);
+#endif
+	len = snprintf(sql_file, sizeof(sql_file), "%s/%s.sql", FREETDS_SRCDIR, BASENAME);
+	assert(len >= 0 && len <= sizeof(sql_file));
+
+	if (input_file)
+		fclose(input_file);
+	if ((input_file = fopen(sql_file, "r")) == NULL) {
+		fflush(stdout);
+		fprintf(stderr, "could not open SQL input file \"%s\"\n", sql_file);
+	}
+
+	if (!free_file_registered)
+		atexit(free_file);
+	free_file_registered = 1;
+	
+	printf("SQL text will be read from %s\n", sql_file);
+
 	return 0;
 }
 
-void
-check_crumbs(void)
+/*
+ * Fill the command buffer from a file while echoing it to standard output.
+ */
+RETCODE 
+sql_cmd(DBPROCESS *dbproc)
 {
-	int i;
-	memcheck_t *ptr = breadcrumbs;
+	char line[2048], *p = line;
+	int i = 0;
+	RETCODE erc=SUCCEED;
 
-	i = num_breadcrumbs;
-	while (ptr != NULL) {
-		if (ptr->special != BREADCRUMB || ptr->item_number != i) {
-			fprintf(stderr, "Somebody overwrote one of the bread crumbs!!!\n");
-			abort();
-		}
-
-		i--;
-		ptr = ptr->next;
-	}
-}
-
-
-
-void
-add_bread_crumb(void)
-{
-	memcheck_t *tmp;
-
-	check_crumbs();
-
-	tmp = (memcheck_t *) calloc(sizeof(memcheck_t), 1);
-	if (tmp == NULL) {
-		fprintf(stderr, "Out of memory");
-		abort();
+	if (!input_file) {
+		fprintf(stderr, "%s: error: SQL input file \"%s\" not opened\n", BASENAME, sql_file);
 		exit(1);
 	}
 
-	num_breadcrumbs++;
+	while ((p = fgets(line, (int)sizeof(line), input_file)) != NULL && strcasecmp("go\n", p) != 0) {
+		printf("\t%3d: %s", ++i, p);
+		if ((erc = dbcmd(dbproc, p)) != SUCCEED) {
+			fprintf(stderr, "%s: error: could write \"%s\" to dbcmd()\n", BASENAME, p);
+			exit(1);
+		}
+	}
 
-	tmp->item_number = num_breadcrumbs;
-	tmp->special = BREADCRUMB;
-	tmp->next = breadcrumbs;
+	if (ferror(input_file)) {
+		fprintf(stderr, "%s: error: could not read SQL input file \"%s\"\n", BASENAME, sql_file);
+		exit(1);
+	}
 
-	breadcrumbs = tmp;
+	return erc;
 }
 
-void
-free_bread_crumb(void)
+RETCODE
+sql_rewind(void)
 {
-	memcheck_t *tmp, *ptr = breadcrumbs;
+	if (!input_file)
+		return FAIL;
+	rewind(input_file);
+	return SUCCEED;
+}
 
-	check_crumbs();
+RETCODE
+sql_reopen(const char *fn)
+{
+	int len = snprintf(sql_file, sizeof(sql_file), "%s/%s.sql", FREETDS_SRCDIR, fn);
+	assert(len >= 0 && len <= sizeof(sql_file));
 
-	while (ptr) {
-		tmp = ptr->next;
-		free(ptr);
-		ptr = tmp;
+	if (input_file)
+		fclose(input_file);
+	if ((input_file = fopen(sql_file, "r")) == NULL) {
+		fflush(stdout);
+		fprintf(stderr, "could not open SQL input file \"%s\"\n", sql_file);
+		sql_file[0] = 0;
+		return FAIL;
 	}
-	num_breadcrumbs = 0;
+	return SUCCEED;
 }
 
 int
 syb_msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line)
 {
-	char var_value[31];
-	int i;
-	char *c;
 	int *pexpected_msgno;
 	
 	/*
@@ -270,13 +339,6 @@ syb_msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, cha
 	    msgno == 5704) {	/* charset changed */
 
 		/* fprintf( stderr, "msgno = %d: %s\n", msgno, msgtext ) ; */
-
-		if (msgtext != NULL && (c = strchr(msgtext, '\'')) != NULL) {
-			i = 0;
-			for (++c; i <= 30 && *c != '\0' && *c != '\''; ++c)
-				var_value[i++] = *c;
-			var_value[i] = '\0';
-		}
 		return 0;
 	}
 
@@ -287,7 +349,7 @@ syb_msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, cha
 	if (dbproc != NULL) {
 		pexpected_msgno = (int *) dbgetuserdata(dbproc);
 		if (pexpected_msgno && *pexpected_msgno == msgno) {
-			fprintf(stdout, "OK: anticipated message arrived: %d %s\n", msgno, msgtext);
+			fprintf(stdout, "OK: anticipated message arrived: %d %s\n", (int) msgno, msgtext);
 			*pexpected_msgno = 0;
 			return 0;
 		}
@@ -296,6 +358,7 @@ syb_msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, cha
 	 * If the severity is something other than 0 or the msg number is
 	 * 0 (user informational messages).
 	 */
+	fflush(stdout);
 	if (severity >= 0 || msgno == 0) {
 		/*
 		 * If the message was something other than informational, and
@@ -319,10 +382,14 @@ syb_msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, cha
 			 */
 			fprintf(stdout, "%s\n", msgtext);
 			fflush(stdout);
+			severity = 0;
 		}
 	}
 
-	assert(0); /* no unanticipated messages allowed in unit tests */
+	if (severity) {
+		fprintf(stderr, "exit: no unanticipated messages allowed in unit tests\n");
+		exit(EXIT_FAILURE);
+	}
 	return 0;
 }
 
@@ -351,9 +418,10 @@ syb_err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *db
 		}
 	}
 
+	fflush(stdout);
 	fprintf(stderr,
-		"DB-LIBRARY error (severity %d, dberr %d, oserr %d, dberrstr %s, oserrstr %s):\n",
-		severity, dberr, oserr, dberrstr ? dberrstr : "(null)", oserrstr ? oserrstr : "(null)");
+		"DB-LIBRARY error (dberr %d (severity %d): \"%s\"; oserr %d: \"%s\")\n",
+		dberr, severity, dberrstr ? dberrstr : "(null)", oserr, oserrstr ? oserrstr : "(null)");
 	fflush(stderr);
 
 	/*
@@ -369,6 +437,10 @@ syb_err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *db
 		}
 	}
 
-	assert(0); /* no unanticipated errors allowed in unit tests */
+	if (severity) {
+		fprintf(stderr, "error: no unanticipated errors allowed in unit tests\n");
+		exit(EXIT_FAILURE);
+	}
+
 	return INT_CANCEL;
 }
