@@ -2,17 +2,18 @@
 #ifndef COMMON_h
 #define COMMON_h
 
-static char rcsid_common_h[] = "$Id: common.h,v 1.20 2009/09/01 05:47:31 freddy77 Exp $";
-static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
+#undef NDEBUG
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
 #include <assert.h>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -32,15 +33,19 @@ static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_wa
 #include <sqlfront.h>
 #include <sqldb.h>
 
-#if !defined(FREETDS_SRCDIR)
-#define FREETDS_SRCDIR "../../../.."
-#endif
+#include <freetds/sysdep_private.h>
 
 #if !defined(EXIT_FAILURE)
 #define EXIT_FAILURE 1
 #define EXIT_SUCCESS 0
 #endif
 
+#define FREETDS_SRCDIR FREETDS_TOPDIR "/src/dblib/unittests"
+
+#if defined(HAVE__SNPRINTF) && !defined(HAVE_SNPRINTF)
+#define snprintf _snprintf
+#endif
+
 #ifdef DBNTWIN32
 /*
  * Define Sybase's symbols in terms of Microsoft's. 
@@ -84,15 +89,13 @@ extern char DATABASE[512];
 
 void set_malloc_options(void);
 int read_login_info(int argc, char **argv);
-void check_crumbs(void);
-void add_bread_crumb(void);
-void free_bread_crumb(void);
 int syb_msg_handler(DBPROCESS * dbproc,
 		    DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line);
 int syb_err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
 
 RETCODE sql_cmd(DBPROCESS *dbproc);
 RETCODE sql_rewind(void);
+RETCODE sql_reopen(const char *fn);
 
 #define int2ptr(i) ((void*)(((char*)0)+(i)))
 #define ptr2int(p) ((int)(((char*)(p))-((char*)0)))
