@@ -5,10 +5,7 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: rpc.c,v 1.39 2010/05/07 02:55:27 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-static int init_proc(DBPROCESS * dbproc, const char *name);
+static RETCODE init_proc(DBPROCESS * dbproc, const char *name);
 int ignore_err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
 int ignore_msg_handler(DBPROCESS * dbproc, DBINT msgno, int state, int severity, char *text, char *server, char *proc, int line);
 
@@ -19,26 +16,6 @@ typedef struct {
 
 static RETPARAM* save_retparam(RETPARAM *param, char *name, char *value, int type, int len);
 
-static const char procedure_sql[] = 
-		"CREATE PROCEDURE %s \n"
-			"  @null_input varchar(30) OUTPUT \n"
-			", @first_type varchar(30) OUTPUT \n"
-			", @nullout int OUTPUT\n"
-			", @nrows int OUTPUT \n"
-			", @c varchar(20)\n"
-			", @nv nvarchar(20) = N'hello'\n"
-		"AS \n"
-		"BEGIN \n"
-			"select @null_input = max(convert(varchar(30), name)) from systypes \n"
-			"select @first_type = min(convert(varchar(30), name)) from systypes \n"
-			"select name from sysobjects where 0=1\n"
-			"select distinct convert(varchar(30), name) as 'type'  from systypes \n"
-				"where name in ('int', 'char', 'text') \n"
-			"select @nrows = @@rowcount \n"
-			"select distinct @nv as '@nv', convert(varchar(30), name) as name  from sysobjects where type = 'S' \n"
-			"return 42 \n"
-		"END \n";
-
 static RETCODE
 init_proc(DBPROCESS * dbproc, const char *name)
 {
@@ -46,21 +23,16 @@ init_proc(DBPROCESS * dbproc, const char *name)
 
 	if (name[0] != '#') {
 		fprintf(stdout, "Dropping procedure %s\n", name);
-		add_bread_crumb();
 		sql_cmd(dbproc);
-		add_bread_crumb();
 		dbsqlexec(dbproc);
-		add_bread_crumb();
 		while (dbresults(dbproc) != NO_MORE_RESULTS) {
 			/* nop */
 		}
-		add_bread_crumb();
 	}
 
 	fprintf(stdout, "Creating procedure %s\n", name);
 	sql_cmd(dbproc);
 	if ((ret = dbsqlexec(dbproc)) == FAIL) {
-		add_bread_crumb();
 		if (name[0] == '#')
 			fprintf(stdout, "Failed to create procedure %s. Wrong permission or not MSSQL.\n", name);
 		else
@@ -87,12 +59,24 @@ save_retparam(RETPARAM *param, char *name, char *value, int type, int len)
 	return param;
 }
 
+static void
+free_retparam(RETPARAM *param)
+{
+	free(param->name);
+	free(param->value);
+	param->name = param->value = NULL;
+}
+
 int
 ignore_msg_handler(DBPROCESS * dbproc, DBINT msgno, int state, int severity, char *text, char *server, char *proc, int line)
 {
+	int ret;
+
 	dbsetuserdata(dbproc, (BYTE*) &msgno);
 	/* printf("(ignoring message %d)\n", msgno); */
-	return syb_msg_handler(dbproc, msgno, state, severity, text, server, proc, line);
+	ret = syb_msg_handler(dbproc, msgno, state, severity, text, server, proc, line);
+	dbsetuserdata(dbproc, NULL);
+	return ret;
 }
 /*
  * The bad procedure name message has severity 15, causing db-lib to call the error handler after calling the message handler.
@@ -111,12 +95,13 @@ ignore_err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char
 	
 	if (recursion_depth++) {
 		printf("error %d: \"%s\"\n", dberr, dberrstr? dberrstr : "");
-		printf("logic error: recursive call to ingnore_err_handler\n");
+		printf("logic error: recursive call to ignore_err_handler\n");
 		exit(1);
 	}
 	dbsetuserdata(dbproc, (BYTE*) &dberr);
 	/* printf("(ignoring error %d)\n", dberr); */
 	erc = syb_err_handler(dbproc, severity, dberr, oserr, dberrstr, oserrstr);
+	dbsetuserdata(dbproc, NULL);
 	recursion_depth--;
 	return erc;
 }
@@ -128,8 +113,8 @@ colwidth( DBPROCESS * dbproc, int icol )
 	return 255 == width? dbcollen(dbproc, icol) : width;
 }
 
-char param_data1[64];
-int param_data2, param_data3;
+char param_data1[64], param_data3[8000+1], param_data4[2 * 4000 + 1];
+int param_data2, param_data5;
 
 struct parameters_t {
 	char         *name;
@@ -142,8 +127,11 @@ struct parameters_t {
 	{ { "@null_input", DBRPCRETURN, SYBCHAR,  -1,   0, NULL }
 	, { "@first_type", DBRPCRETURN, SYBCHAR,  sizeof(param_data1), 0, (BYTE *) &param_data1 }
 	, { "@nullout",    DBRPCRETURN, SYBINT4,  -1,   0, (BYTE *) &param_data2 }
-	, { "@nrows",      DBRPCRETURN, SYBINT4,  -1,  -1, (BYTE *) &param_data3 }
-	, { "@c",          0,        SYBVARCHAR,   0,   0, NULL }
+	, { "@varchar_tds7_out", DBRPCRETURN, SYBVARCHAR,  sizeof(param_data3),   0, (BYTE *) &param_data3 }
+	, { "@nvarchar_tds7_out", DBRPCRETURN, 231,  sizeof(param_data4),   0, (BYTE *) &param_data4 }
+	, { "@nrows",      DBRPCRETURN, SYBINT4,  -1,  -1, (BYTE *) &param_data5 }
+	, { "@c_this_name_is_way_more_than_thirty_characters_charlie",
+		           0,        SYBVARCHAR,   0,   0, NULL }
 	, { "@nv",         0,        SYBVARCHAR,  -1,   2, (BYTE *) "OK:" }
 	}, *pb = bindings;
 
@@ -152,9 +140,9 @@ main(int argc, char **argv)
 {
 	LOGINREC *login;
 	DBPROCESS *dbproc;
-	RETPARAM save_param;
+	RETPARAM save_param, save_varchar_tds7_param, save_nvarchar_tds7_param;
 	
-	char teststr[1024];
+	char teststr[8000+1], abbrev_data[10+3+1], *output;
 	char *retname = NULL;
 	int i, failed = 0;
 	int rettype = 0, retlen = 0, return_status = 0;
@@ -162,35 +150,35 @@ main(int argc, char **argv)
 	char *proc_name = proc;
 
 	int num_resultset = 0, num_empty_resultset = 0;
+	int num_params = 6;
 
 	static const char dashes30[] = "------------------------------";
 	static const char  *dashes5 = dashes30 + (sizeof(dashes30) - 5), 
-			  *dashes15 = dashes30 + (sizeof(dashes30) - 15);
+			  *dashes20 = dashes30 + (sizeof(dashes30) - 20);
 
 	RETCODE erc, row_code;
 
 	set_malloc_options();
 	
 	memset(&save_param, 0, sizeof(save_param));
+	memset(&save_varchar_tds7_param, 0, sizeof(save_varchar_tds7_param));
+	memset(&save_nvarchar_tds7_param, 0, sizeof(save_nvarchar_tds7_param));
 
 	read_login_info(argc, argv);
 
 	fprintf(stdout, "Starting %s\n", argv[0]);
-	add_bread_crumb();
 
 	dbinit();
 
-	add_bread_crumb();
 	dberrhandle(syb_err_handler);
 	dbmsghandle(syb_msg_handler);
 
 	fprintf(stdout, "About to logon\n");
 
-	add_bread_crumb();
 	login = dblogin();
 	DBSETLPWD(login, PASSWORD);
 	DBSETLUSER(login, USER);
-	DBSETLAPP(login, "#t0022");
+	DBSETLAPP(login, "rpc");
 	dberrhandle(ignore_err_handler);
 	DBSETLPACKET(login, -1);
 	dberrhandle(syb_err_handler);
@@ -198,21 +186,32 @@ main(int argc, char **argv)
 
 	fprintf(stdout, "About to open %s.%s\n", SERVER, DATABASE);
 
-	add_bread_crumb();
 	dbproc = dbopen(login, SERVER);
 	if (strlen(DATABASE))
 		dbuse(dbproc, DATABASE);
-	add_bread_crumb();
 	dbloginfree(login);
-	add_bread_crumb();
 
-	add_bread_crumb();
+	printf("Check if server support long identifiers\n");
+	sql_cmd(dbproc);
+	i = 103;
+	dbsetuserdata(dbproc, (BYTE*) &i);
+	dbsqlexec(dbproc);
+	while (dbresults(dbproc) != NO_MORE_RESULTS)
+		while (dbnextrow(dbproc) != NO_MORE_ROWS)
+			continue;
+	dbsetuserdata(dbproc, NULL);
+	if (i == 0) {
+		fprintf(stderr, "This server does not support long identifiers\n");
+		dbexit();
+		return 0;
+	}
 
 	dberrhandle(ignore_err_handler);
 	dbmsghandle(ignore_msg_handler);
 
 	printf("trying to create a temporary stored procedure\n");
 	if (FAIL == init_proc(dbproc, proc_name)) {
+		num_params = 4;
 		printf("trying to create a permanent stored procedure\n");
 		if (FAIL == init_proc(dbproc, ++proc_name))
 			exit(EXIT_FAILURE);
@@ -231,8 +230,10 @@ main(int argc, char **argv)
 		failed = 1;
 	}
 
-	for (pb = bindings; pb < bindings + sizeof(bindings)/sizeof(bindings[0]); pb++) {
+	for (pb = bindings, i = 0; pb < bindings + sizeof(bindings)/sizeof(bindings[0]); pb++, i++) {
 		printf("executing dbrpcparam for %s\n", pb->name);
+		if (num_params == 4 && (i == 3 || i == 4))
+			continue;
 		if ((erc = dbrpcparam(dbproc, pb->name, pb->status, pb->type, pb->maxlen, pb->datalen, pb->value)) == FAIL) {
 			fprintf(stderr, "Failed line %d: dbrpcparam\n", __LINE__);
 			failed++;
@@ -240,7 +241,7 @@ main(int argc, char **argv)
 
 	}
 	printf("executing dbrpcsend\n");
-	param_data3 = 0x11223344;
+	param_data5 = 0x11223344;
 	erc = dbrpcsend(dbproc);
 	if (erc == FAIL) {
 		fprintf(stderr, "Failed line %d: dbrpcsend\n", __LINE__);
@@ -255,8 +256,6 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	add_bread_crumb();
-
 	/* retrieve outputs per usual */
 	printf("fetching results\n");
 	while ((erc = dbresults(dbproc)) != NO_MORE_RESULTS) {
@@ -303,7 +302,6 @@ main(int argc, char **argv)
 			if (empty_resultset)
 				++num_empty_resultset;
 		} else {
-			add_bread_crumb();
 			fprintf(stderr, "Expected a result set.\n");
 			exit(1);
 		}
@@ -320,30 +318,41 @@ main(int argc, char **argv)
 	/* 
 	 * Check output parameter values 
 	 */
-	if (dbnumrets(dbproc) < 4) {	/* dbnumrets missed something */
-		fprintf(stderr, "Expected 4 output parameters.\n");
+	if (dbnumrets(dbproc) != num_params) {	/* dbnumrets missed something */
+		fprintf(stderr, "Expected %d output parameters.\n", num_params);
 		exit(1);
 	}
 	printf("retrieving output parameters...\n");
-	printf("%-5s %-15s %5s %6s  %-30s\n", "param", "name", "type", "length", "data"); 
-	printf("%-5s %-15s %5s %5s- %-30s\n", dashes5, dashes15, dashes5, dashes5, dashes30); 
+	printf("%-5s %-20s %5s %6s  %-30s\n", "param", "name", "type", "length", "data");
+	printf("%-5s %-20s %5s %5s- %-30s\n", dashes5, dashes20, dashes5, dashes5, dashes30);
 	for (i = 1; i <= dbnumrets(dbproc); i++) {
-		add_bread_crumb();
 		retname = dbretname(dbproc, i);
 		rettype = dbrettype(dbproc, i);
 		retlen = dbretlen(dbproc, i);
 		dbconvert(dbproc, rettype, dbretdata(dbproc, i), retlen, SYBVARCHAR, (BYTE*) teststr, -1);
-		printf("%-5d %-15s %5d %6d  %-30s\n", i, retname, rettype, retlen, teststr); 
-		add_bread_crumb();
+		if(retlen <= 10) {
+			output = teststr;
+		} else {
+			memcpy(abbrev_data, teststr, 10);
+			sprintf(&abbrev_data[10], "...");
+			output = abbrev_data;
+		}
+		printf("%-5d %-20s %5d %6d  %-30s\n", i, retname, rettype, retlen, output);
 
 		save_retparam(&save_param, retname, teststr, rettype, retlen);
+		if (i == 4) {
+			save_retparam(&save_varchar_tds7_param, retname, teststr, rettype, retlen);
+		}
+		if (i == 5) {
+			save_retparam(&save_nvarchar_tds7_param, retname, teststr, rettype, retlen);
+		}
 	}
 
 	/* 
 	 * Test the last parameter for expected outcome 
 	 */
-	if ((save_param.name == NULL) || strcmp(save_param.name, bindings[3].name)) {
-		fprintf(stderr, "Expected retname to be '%s', got ", bindings[3].name);
+	if ((save_param.name == NULL) || strcmp(save_param.name, bindings[5].name)) {
+		fprintf(stderr, "Expected retname to be '%s', got ", bindings[5].name);
 		if (save_param.name == NULL) 
 			fprintf(stderr, "<NULL> instead.\n");
 		else
@@ -363,12 +372,50 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	if (num_params == 6) {
+		/*
+		 * Test name, size, contents of the VARCHAR(8000) output parameter
+		 */
+		if ((save_varchar_tds7_param.name == NULL) || strcmp(save_varchar_tds7_param.name, bindings[3].name)) {
+			fprintf(stderr, "Expected retname to be '%s', got ", bindings[3].name);
+			if (save_varchar_tds7_param.name == NULL)
+				fprintf(stderr, "<NULL> instead.\n");
+			else
+				fprintf(stderr, "'%s' instead.\n", save_varchar_tds7_param.name);
+			exit(1);
+		}
+		if (save_varchar_tds7_param.type != SYBVARCHAR) {
+			fprintf(stderr, "Expected rettype to be SYBVARCHAR was %d.\n", save_varchar_tds7_param.type);
+			exit(1);
+		}
+		if (save_varchar_tds7_param.len != 8000) {
+			fprintf(stderr, "Expected retlen to be 8000 was %d.\n", save_varchar_tds7_param.len);
+			exit(1);
+		}
+
+		/*
+		 * Test name, size, contents of the NVARCHAR(4000) output parameter
+		 */
+		if ((save_nvarchar_tds7_param.name == NULL) || strcmp(save_nvarchar_tds7_param.name, bindings[4].name)) {
+			fprintf(stderr, "Expected retname to be '%s', got ", bindings[4].name);
+			if (save_varchar_tds7_param.name == NULL)
+				fprintf(stderr, "<NULL> instead.\n");
+			else
+				fprintf(stderr, "'%s' instead.\n", save_nvarchar_tds7_param.name);
+			exit(1);
+		}
+		if (save_nvarchar_tds7_param.len != 4000) {
+			fprintf(stderr, "Expected retlen to be 4000 was %d.\n", save_nvarchar_tds7_param.len);
+			exit(1);
+		}
+	}
+
 	if(42 != return_status) {
 		fprintf(stderr, "Expected status to be 42.\n");
 		exit(1);
 	}
 
-	printf("Good: Got 4 output parameters and 1 return status of %d.\n", return_status);
+	printf("Good: Got 6 output parameters and 1 return status of %d.\n", return_status);
 
 
 	/* Test number of result sets */
@@ -382,27 +429,21 @@ main(int argc, char **argv)
 	}
 	printf("Good: Got %d resultsets and %d empty resultset.\n", num_resultset, num_empty_resultset);
 
-	add_bread_crumb();
 
 
 	fprintf(stdout, "Dropping procedure\n");
-	add_bread_crumb();
 	sql_cmd(dbproc);
-	add_bread_crumb();
 	dbsqlexec(dbproc);
-	add_bread_crumb();
 	while (dbresults(dbproc) != NO_MORE_RESULTS) {
 		/* nop */
 	}
-	add_bread_crumb();
 	dbexit();
-	add_bread_crumb();
 
 	fprintf(stdout, "%s %s\n", __FILE__, (failed ? "failed!" : "OK"));
-	free_bread_crumb();
 
-	free(save_param.name);
-	free(save_param.value);
+	free_retparam(&save_param);
+	free_retparam(&save_varchar_tds7_param);
+	free_retparam(&save_nvarchar_tds7_param);
 
 	return failed ? 1 : 0;
 }
