@@ -5,11 +5,6 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0003.c,v 1.16 2009/02/27 15:52:48 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-
-
 int failed = 0;
 
 
@@ -29,18 +24,15 @@ main(int argc, char **argv)
 	read_login_info(argc, argv);
 
 	fprintf(stdout, "Starting %s\n", argv[0]);
-	add_bread_crumb();
 
 	/* Fortify_EnterScope(); */
 	dbinit();
 
-	add_bread_crumb();
 	dberrhandle(syb_err_handler);
 	dbmsghandle(syb_msg_handler);
 
 	fprintf(stdout, "About to logon\n");
 
-	add_bread_crumb();
 	login = dblogin();
 	DBSETLPWD(login, PASSWORD);
 	DBSETLUSER(login, USER);
@@ -49,20 +41,16 @@ main(int argc, char **argv)
 
 	fprintf(stdout, "About to open\n");
 
-	add_bread_crumb();
 	dbproc = dbopen(login, SERVER);
 	if (strlen(DATABASE))
 		dbuse(dbproc, DATABASE);
-	add_bread_crumb();
 	dbloginfree(login);
-	add_bread_crumb();
 
 #ifdef MICROSOFT_DBLIB
 	dbsetopt(dbproc, DBBUFFER, "100");
 #else
 	dbsetopt(dbproc, DBBUFFER, "100", 0);
 #endif
-	add_bread_crumb();
 
 	fprintf(stdout, "creating table\n");
 	sql_cmd(dbproc);
@@ -83,49 +71,33 @@ main(int argc, char **argv)
 	fprintf(stdout, "select\n");
 	sql_cmd(dbproc);
 	dbsqlexec(dbproc);
-	add_bread_crumb();
-
 
 	if (dbresults(dbproc) != SUCCEED) {
 		failed = 1;
-		add_bread_crumb();
 		fprintf(stdout, "Was expecting a result set.");
 		exit(1);
 	}
-	add_bread_crumb();
 
-	for (i = 1; i <= dbnumcols(dbproc); i++) {
-		add_bread_crumb();
+	for (i = 1; i <= dbnumcols(dbproc); i++)
 		printf("col %d is %s\n", i, dbcolname(dbproc, i));
-		add_bread_crumb();
-	}
 
-	add_bread_crumb();
 	dbbind(dbproc, 1, INTBIND, 0, (BYTE *) & testint);
-	add_bread_crumb();
 	dbbind(dbproc, 2, STRINGBIND, 0, (BYTE *) teststr);
-	add_bread_crumb();
-
-	add_bread_crumb();
 
 	for (i = 1; i < rows_to_add; i++) {
 	char expected[1024];
 
 		sprintf(expected, "row %03d", i);
 
-		add_bread_crumb();
-
 		if (i % 100 == 0) {
 			dbclrbuf(dbproc, 100);
 		}
 
-		add_bread_crumb();
 		if (REG_ROW != dbnextrow(dbproc)) {
 			failed = 1;
 			fprintf(stderr, "Failed.  Expected a row\n");
 			exit(1);
 		}
-		add_bread_crumb();
 		if (testint != i) {
 			failed = 1;
 			fprintf(stderr, "Failed.  Expected i to be %d, was %d\n", i, (int) testint);
@@ -139,8 +111,6 @@ main(int argc, char **argv)
 		printf("Read a row of data -> %d %s\n", (int) testint, teststr);
 	}
 
-
-	add_bread_crumb();
 	dbclrbuf(dbproc, 1);
 	rc = dbnextrow(dbproc);
 	if (rc != NO_MORE_ROWS) {
@@ -149,11 +119,8 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	add_bread_crumb();
 	dbexit();
-	add_bread_crumb();
 
 	fprintf(stdout, "%s %s\n", __FILE__, (failed ? "failed!" : "OK"));
-	free_bread_crumb();
 	return failed ? 1 : 0;
 }
