@@ -5,9 +5,6 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0014.c,v 1.32 2010/12/30 18:11:07 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 #define BLOB_BLOCK_SIZE 4096
 
 char *testargs[] = { "", FREETDS_SRCDIR "/data.bin", "t0014.out" };
@@ -29,7 +26,6 @@ test(int argc, char **argv, int over4k)
 	char sqlCmd[256];
 	char rbuf[BLOB_BLOCK_SIZE];
 	long numread;
-	BOOL readFirstImage;
 	int numtowrite, numwritten;
 
 	set_malloc_options();
@@ -79,7 +75,8 @@ test(int argc, char **argv, int over4k)
 	result = fseek(fp, 0, SEEK_SET);
 
 	blob = (char *) malloc(isiz);
-	fread((void *) blob, isiz, 1, fp);
+	result = fread((void *) blob, isiz, 1, fp);
+	assert(result == 1);
 	fclose(fp);
 
 	/* FIXME this test seem to not work using temporary tables (sybase?)... */
@@ -122,6 +119,13 @@ test(int argc, char **argv, int over4k)
 			textPtr = dbtxptr(dbproc, 1);
 			timeStamp = dbtxtimestamp(dbproc, 1);
 
+			if (!textPtr && !timeStamp && dbtds(dbproc) >= DBTDS_7_2) {
+				printf("Protocol 7.2+ detected, test not supported\n");
+				free(blob);
+				dbexit();
+				exit(0);
+			}
+
 			/*
 			 * Use #ifdef if you want to test dbmoretext mode (needed for 16-bit apps)
 			 * Use #ifndef for big buffer version (32-bit)
@@ -144,7 +148,8 @@ test(int argc, char **argv, int over4k)
 					dbmoretext(blobproc, (DBINT) numtowrite, (BYTE *) (blob + numwritten));
 				}
 				dbsqlok(blobproc);
-				while (dbresults(blobproc) != NO_MORE_RESULTS);
+				while (dbresults(blobproc) != NO_MORE_RESULTS)
+					continue;
 			}
 		}
 	}
@@ -202,11 +207,8 @@ test(int argc, char **argv, int over4k)
 
 		numread = 0;
 		rblob = NULL;
-		readFirstImage = FALSE;
 		while ((result = dbreadtext(blobproc, rbuf, BLOB_BLOCK_SIZE)) != NO_MORE_ROWS) {
-			if (result == 0) {	/* this indicates end of row */
-				readFirstImage = TRUE;
-			} else {
+			if (result != 0) {	/* this indicates not end of row */
 				rblob = (char*) realloc(rblob, result + numread);
 				memcpy((void *) (rblob + numread), (void *) rbuf, result);
 				numread += result;
