@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -28,18 +26,11 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#include <tds.h>
+#include <freetds/tds.h>
 #include <sybfront.h>
 #include <sybdb.h>
 #include <dblib.h>
 
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: xact.c,v 1.14 2009/03/24 01:22:14 jklowden Exp $");
-
-
 #if defined(DBLIB_UNIMPLEMENTED)
 void
 build_xact_string(char *xact_name, char *service_name, DBINT commid, char *result)
