@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -33,12 +31,8 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tdsodbc.h"
-#include "tdsstring.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
+#include <freetds/odbc.h>
+#include <freetds/string.h>
 
 static void desc_free_record(struct _drecord *drec);
 
@@ -47,12 +41,13 @@ desc_alloc(SQLHANDLE parent, int desc_type, int alloc_type)
 {
 	TDS_DESC *desc;
 
-	desc = (TDS_DESC *) malloc(sizeof(TDS_DESC));
-	if (!desc)
+	desc = tds_new0(TDS_DESC, 1);
+	if (!desc || tds_mutex_init(&desc->mtx)) {
+		free(desc);
 		return NULL;
-	memset(desc, 0, sizeof(TDS_DESC));
+	}
 
-	/* set defualt header values */
+	/* set default header values */
 	desc->htype = SQL_HANDLE_DESC;
 	desc->type = desc_type;
 	desc->parent = parent;
@@ -89,7 +84,7 @@ desc_alloc(SQLHANDLE parent, int desc_type, int alloc_type)
 SQLRETURN
 desc_alloc_records(TDS_DESC * desc, unsigned count)
 {
-	struct _drecord *drec, *drecs;
+	struct _drecord *drec;
 	int i;
 
 	/* shrink records */
@@ -100,13 +95,8 @@ desc_alloc_records(TDS_DESC * desc, unsigned count)
 		return SQL_SUCCESS;
 	}
 
-	if (desc->records)
-		drecs = (struct _drecord *) realloc(desc->records, sizeof(struct _drecord) * (count + 0));
-	else
-		drecs = (struct _drecord *) malloc(sizeof(struct _drecord) * (count + 0));
-	if (!drecs)
+	if (!TDS_RESIZE(desc->records, count))
 		return SQL_ERROR;
-	desc->records = drecs;
 	memset(desc->records + desc->header.sql_desc_count, 0, sizeof(struct _drecord) * (count - desc->header.sql_desc_count));
 
 	for (i = desc->header.sql_desc_count; i < count; ++i) {
@@ -210,7 +200,18 @@ desc_free(TDS_DESC * desc)
 	if (desc) {
 		desc_free_records(desc);
 		odbc_errs_reset(&desc->errs);
+		tds_mutex_free(&desc->mtx);
 		free(desc);
 	}
 	return SQL_SUCCESS;
 }
+
+TDS_DBC *
+desc_get_dbc(TDS_DESC *desc)
+{
+	if (IS_HSTMT(desc->parent))
+		return ((TDS_STMT *) desc->parent)->dbc;
+
+	return (TDS_DBC *) desc->parent;
+}
+
