@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998-1999  Brian Bruns
- * Copyright (C) 2003-2010  Frediano Ziglio
+ * Copyright (C) 2003-2012  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -35,16 +33,11 @@
 
 #include <assert.h>
 
-#include "tdsodbc.h"
+#include <freetds/odbc.h>
 #include "odbcss.h"
-#include "tdsstring.h"
+#include <freetds/string.h>
 #include "replacements.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: error.c,v 1.66 2010/11/26 19:05:30 freddy77 Exp $");
+#include "sqlwparams.h"
 
 static void odbc_errs_pop(struct _sql_errors *errs);
 static const char *odbc_get_msg(const char *sqlstate);
@@ -255,8 +248,16 @@ rank_errors(struct _sql_errors *errs)
 
 	/* Find the highest of all unranked errors until there are none left */
 	for (settled = 0; settled < errs->num_errors; settled++) {
-		best = -1;
+		best = settled;
 		for (current = settled; current < errs->num_errors; current++) {
+			/* always sort by rows */
+			if (errs->errs[best].row < errs->errs[current].row)
+				continue;
+			if (errs->errs[best].row > errs->errs[current].row) {
+				best = current;
+				continue;
+			}
+
 			istrans = 0;
 			switch (errs->errs[current].native) {
 			case 1205:
@@ -292,9 +293,6 @@ rank_errors(struct _sql_errors *errs)
 				break;
 			}
 
-			if (best == -1)
-				best = current;
-
 			/* Non-terminating comparisons only below this point */
 			if (errs->errs[current].msgstate > errs->errs[best].msgstate)
 				best = current;
@@ -332,13 +330,13 @@ odbc_get_v2state(const char *sqlstate, char *dest_state)
 
 	while (pmap->v3[0]) {
 		if (!strcasecmp(pmap->v3, sqlstate)) {
-			tds_strlcpy(dest_state, pmap->v2, 6);
+			strlcpy(dest_state, pmap->v2, 6);
 			return;
 		}
 		++pmap;
 	}
 	/* return the original if a v2 state is not found */
-	tds_strlcpy(dest_state, sqlstate, 6);
+	strlcpy(dest_state, sqlstate, 6);
 }
 
 void
@@ -383,7 +381,6 @@ odbc_errs_pop(struct _sql_errors *errs)
 void
 odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg)
 {
-	struct _sql_error *p;
 	int n;
 
 	assert(sqlstate);
@@ -391,17 +388,14 @@ odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg)
 		return;
 
 	n = errs->num_errors;
-	if (errs->errs)
-		p = (struct _sql_error *) realloc(errs->errs, sizeof(struct _sql_error) * (n + 1));
-	else
-		p = (struct _sql_error *) malloc(sizeof(struct _sql_error));
-	if (!p)
+	if (!TDS_RESIZE(errs->errs, n + 1)) {
+		errs->lastrc = SQL_ERROR;
 		return;
-	errs->errs = p;
+	}
 
 	memset(&errs->errs[n], 0, sizeof(struct _sql_error));
 	errs->errs[n].native = 0;
-	tds_strlcpy(errs->errs[n].state3, sqlstate, 6);
+	strlcpy(errs->errs[n].state3, sqlstate, 6);
 	odbc_get_v2state(errs->errs[n].state3, errs->errs[n].state2);
 
 	/* TODO why driver ?? -- freddy77 */
@@ -409,29 +403,32 @@ odbc_errs_add(struct _sql_errors *errs, const char *sqlstate, const char *msg)
 	errs->errs[n].msg = msg ? strdup(msg) : odbc_get_msg(errs->errs[n].state3);
 	++errs->num_errors;
 
+	/* updated last error */
+	if (!strcmp(sqlstate, "01004") || !strcmp(sqlstate, "01S02")) {
+		if (errs->lastrc != SQL_ERROR)
+			errs->lastrc = SQL_SUCCESS_WITH_INFO;
+	} else {
+		errs->lastrc = SQL_ERROR;
+	}
+
 	tdsdump_log(TDS_DBG_FUNC, "odbc_errs_add: \"%s\"\n", errs->errs[n].msg);
 }
 
 /* TODO check if TDS_UINT is correct for native error */
 void
 odbc_errs_add_rdbms(struct _sql_errors *errs, TDS_UINT native, const char *sqlstate, const char *msg, int linenum, int msgstate,
-		    const char *server)
+		    const char *server, int row)
 {
-	struct _sql_error *p;
 	int n = errs->num_errors;
 
-	if (errs->errs)
-		p = (struct _sql_error *) realloc(errs->errs, sizeof(struct _sql_error) * (n + 1));
-	else
-		p = (struct _sql_error *) malloc(sizeof(struct _sql_error));
-	if (!p)
+	if (!TDS_RESIZE(errs->errs, n + 1))
 		return;
-	errs->errs = p;
 
 	memset(&errs->errs[n], 0, sizeof(struct _sql_error));
+	errs->errs[n].row = row;
 	errs->errs[n].native = native;
 	if (sqlstate)
-		tds_strlcpy(errs->errs[n].state2, sqlstate, 6);
+		strlcpy(errs->errs[n].state2, sqlstate, 6);
 	else
 		errs->errs[n].state2[0] = '\0';
 	strcpy(errs->errs[n].state3, errs->errs[n].state2);
@@ -500,9 +497,8 @@ sqlstate2to3(char *state)
 	SQLS_MAP("S1T00", "HYT00");
 }
 
-#define FUNC NAME(SQLGetDiagRec) (P(SQLSMALLINT,handleType), P(SQLHANDLE,handle), P(SQLSMALLINT,numRecord), PCHAR(szSqlState),\
-	P(SQLINTEGER FAR *,pfNativeError), PCHAROUT(ErrorMsg,SQLSMALLINT) WIDE)
-#include "sqlwparams.h"
+ODBC_FUNC(SQLGetDiagRec, (P(SQLSMALLINT,handleType), P(SQLHANDLE,handle), P(SQLSMALLINT,numRecord), PCHAR(szSqlState),
+	P(SQLINTEGER FAR *,pfNativeError), PCHAROUT(ErrorMsg,SQLSMALLINT) WIDE))
 {
 	SQLRETURN result;
 	struct _sql_errors *errs;
@@ -514,9 +510,6 @@ sqlstate2to3(char *state)
 
 	SQLINTEGER odbc_ver = SQL_OV_ODBC2;
 
-	tdsdump_log(TDS_DBG_FUNC, "SQLGetDiagRec(%d, %p, %d, %p, %p, %p, %d, %p)\n",
-			handleType, handle, numRecord, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
-
 	if (numRecord <= 0 || cbErrorMsgMax < 0)
 		return SQL_ERROR;
 
@@ -574,17 +567,13 @@ sqlstate2to3(char *state)
 	return result;
 }
 
-#define FUNC NAME(SQLError) (P(SQLHENV,henv), P(SQLHDBC,hdbc), P(SQLHSTMT,hstmt), PCHAR(szSqlState), P(SQLINTEGER FAR *,pfNativeError),\
-	PCHAROUT(ErrorMsg,SQLSMALLINT) WIDE)
-#include "sqlwparams.h"
+ODBC_FUNC(SQLError,  (P(SQLHENV,henv), P(SQLHDBC,hdbc), P(SQLHSTMT,hstmt), PCHAR(szSqlState), P(SQLINTEGER FAR *,pfNativeError),
+	PCHAROUT(ErrorMsg,SQLSMALLINT) WIDE))
 {
 	SQLRETURN result;
 	SQLSMALLINT type;
 	SQLHANDLE handle;
 
-	tdsdump_log(TDS_DBG_FUNC, "SQLError(%p, %p, %p, %p, %p, %p, %d, %p)\n", 
-			henv, hdbc, hstmt, szSqlState, pfNativeError, szErrorMsg, cbErrorMsgMax, pcbErrorMsg);
-
 	if (hstmt) {
 		handle = hstmt;
 		type = SQL_HANDLE_STMT;
@@ -607,9 +596,8 @@ sqlstate2to3(char *state)
 	return result;
 }
 
-#define FUNC NAME(SQLGetDiagField) (P(SQLSMALLINT,handleType), P(SQLHANDLE,handle), P(SQLSMALLINT,numRecord),\
-	P(SQLSMALLINT,diagIdentifier), P(SQLPOINTER,buffer), P(SQLSMALLINT,cbBuffer), P(SQLSMALLINT FAR *,pcbBuffer) WIDE)
-#include "sqlwparams.h"
+ODBC_FUNC(SQLGetDiagField, (P(SQLSMALLINT,handleType), P(SQLHANDLE,handle), P(SQLSMALLINT,numRecord),
+	P(SQLSMALLINT,diagIdentifier), P(SQLPOINTER,buffer), P(SQLSMALLINT,cbBuffer), P(SQLSMALLINT FAR *,pcbBuffer) WIDE))
 {
 	SQLRETURN result = SQL_SUCCESS;
 	struct _sql_errors *errs;
@@ -622,9 +610,6 @@ sqlstate2to3(char *state)
 	TDS_ENV *env = NULL;
 	char tmp[16];
 
-	tdsdump_log(TDS_DBG_FUNC, "SQLGetDiagField(%d, %p, %d, %d, %p, %d, %p)\n", 
-			handleType, handle, numRecord, diagIdentifier, buffer, cbBuffer, pcbBuffer);
-
 	if (cbBuffer < 0)
 		return SQL_ERROR;
 
@@ -703,7 +688,8 @@ sqlstate2to3(char *state)
 
 	switch (diagIdentifier) {
 	case SQL_DIAG_ROW_NUMBER:
-		*(SQLINTEGER *) buffer = SQL_ROW_NUMBER_UNKNOWN;
+		*(SQLINTEGER *) buffer =
+			errs->errs[numRecord].row > 0 ? errs->errs[numRecord].row : SQL_ROW_NUMBER_UNKNOWN;
 		break;
 
 	case SQL_DIAG_CLASS_ORIGIN:
@@ -733,8 +719,8 @@ sqlstate2to3(char *state)
 		break;
 
 	case SQL_DIAG_CONNECTION_NAME:
-		if (dbc && dbc->tds_socket && dbc->tds_socket->spid > 0)
-			cplen = sprintf(tmp, "%d", dbc->tds_socket->spid);
+		if (dbc && dbc->tds_socket && dbc->tds_socket->conn->spid > 0)
+			cplen = sprintf(tmp, "%d", dbc->tds_socket->conn->spid);
 		else
 			cplen = 0;
 
@@ -751,26 +737,27 @@ sqlstate2to3(char *state)
 		break;
 
 	case SQL_DIAG_SERVER_NAME:
-		msg = "";
+		msg = NULL;
 		switch (handleType) {
 		case SQL_HANDLE_ENV:
 			break;
 		case SQL_HANDLE_DBC:
-			msg = tds_dstr_cstr(&dbc->server);
+			if (dbc->tds_socket)
+				msg = dbc->tds_socket->conn->server;
 			break;
 		case SQL_HANDLE_STMT:
-			msg = tds_dstr_cstr(&stmt->dbc->server);
+			if (stmt->dbc->tds_socket)
+				msg = stmt->dbc->tds_socket->conn->server;
 			/*
 			 * if dbc->server is not initialized, init it
 			 * from the errs structure
 			 */
-			if (!msg[0] && errs->errs[numRecord].server) {
-				tds_dstr_copy(&stmt->dbc->server, errs->errs[numRecord].server);
+			if (!msg && errs->errs[numRecord].server) {
 				msg = errs->errs[numRecord].server;
 			}
 			break;
 		}
-		result = odbc_set_string_oct(dbc, buffer, cbBuffer, pcbBuffer, msg, -1);
+		result = odbc_set_string_oct(dbc, buffer, cbBuffer, pcbBuffer, msg ? msg : "", -1);
 		break;
 
 	case SQL_DIAG_SQLSTATE:
@@ -788,3 +775,5 @@ sqlstate2to3(char *state)
 	return result;
 }
 
+#include "error_export.h"
+
