@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998-2002  Brian Bruns
+ * Copyright (C) 2004, 2005 Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,29 +18,28 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
+
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#if HAVE_ERRNO_H
+#include <errno.h>
+#endif /* HAVE_ERRNO_H */
+
 #include <ctype.h>
 #include <assert.h>
 
-#include "tds.h"
-#include "tdsodbc.h"
-#include "prepare_query.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-static const char software_version[] = "$Id: native.c,v 1.18 2004/10/28 12:42:12 freddy77 Exp $";
-static const void *const no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/odbc.h>
+#include <freetds/string.h>
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
+#define TDS_ISALPHA(c) isalpha((unsigned char) (c))
 
 /*
  * Function transformation (from ODBC to Sybase)
@@ -74,23 +74,35 @@ static const void *const no_unused_var_warn[] = { software_version, no_unused_va
  * TRUNCATE -> ??
  */
 static SQLRETURN
-to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
+to_native(struct _hdbc *dbc, struct _hstmt *stmt, DSTR *str)
 {
-	char *d, *s, *p;
+	char *d, *s;
 	int nest_syntax = 0;
+	char *buf = tds_dstr_buf(str);
 
 	/* list of bit, used as stack, is call ? FIXME limites size... */
 	unsigned long is_calls = 0;
 	int server_scalar;
 
-	assert(dbc && buf);
+	assert(dbc);
 
-	server_scalar = TDS_IS_MSSQL(dbc->tds_socket) && dbc->tds_socket->product_version >= TDS_MS_VER(7, 0, 0);
+	server_scalar = TDS_IS_MSSQL(dbc->tds_socket) && dbc->tds_socket->conn->product_version >= TDS_MS_VER(7, 0, 0);
 
-	/* we can do it because result string will be
-	 * not bigger than source string */
+	/*
+	 * we can do it because result string will be
+	 * not bigger than source string
+	 */
 	d = s = buf;
 	while (*s) {
+		if (*s == '-' || *s == '/') {
+			size_t len_comment = tds_skip_comment(s) - s;
+
+			memmove(d, s, len_comment);
+			s += len_comment;
+			d += len_comment;
+			continue;
+		}
+
 		/* TODO: test syntax like "select 1 as [pi]]p)p{?=call]]]]o], 2" on mssql7+ */
 		if (*s == '"' || *s == '\'' || *s == '[') {
 			size_t len_quote = tds_skip_quoted(s) - s;
@@ -104,7 +116,8 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 		if (*s == '{') {
 			char *pcall;
 
-			while (TDS_ISSPACE(*++s));
+			while (TDS_ISSPACE(*++s))
+				continue;
 			pcall = s;
 			/* FIXME if nest_syntax > 0 problems */
 			if (server_scalar && strncasecmp(pcall, "fn ", 3) == 0) {
@@ -113,9 +126,11 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 			}
 			if (*pcall == '?') {
 				/* skip spaces after ? */
-				while (TDS_ISSPACE(*++pcall));
+				while (TDS_ISSPACE(*++pcall))
+					continue;
 				if (*pcall == '=') {
-					while (TDS_ISSPACE(*++pcall));
+					while (TDS_ISSPACE(*++pcall))
+						continue;
 				} else {
 					/* avoid {?call ... syntax */
 					pcall = s;
@@ -124,14 +139,16 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 			if (strncasecmp(pcall, "call ", 5) != 0)
 				pcall = NULL;
 
+			if (stmt)
+				stmt->prepared_query_is_rpc = 1;
 			++nest_syntax;
 			is_calls <<= 1;
 			if (!pcall) {
 				/* assume syntax in the form {type ...} */
-				p = strchr(s, ' ');
-				if (!p)
-					break;
-				s = p + 1;
+				while (TDS_ISALPHA(*s))
+					++s;
+				while (TDS_ISSPACE(*s))
+					++s;
 			} else {
 				if (*s == '?' && stmt)
 					stmt->prepared_query_is_func = 1;
@@ -158,32 +175,94 @@ to_native(struct _hdbc *dbc, struct _hstmt *stmt, char *buf)
 			*d++ = *s++;
 		}
 	}
-	*d = '\0';
+	tds_dstr_setlen(str, d - buf);
 	return SQL_SUCCESS;
 }
 
+const char *
+parse_const_param(const char *s, TDS_SERVER_TYPE *type)
+{
+	char *end;
+
+	/* binary */
+	if (strncasecmp(s, "0x", 2) == 0) {
+		s += 2;
+		while (isxdigit(*s))
+			++s;
+		*type = SYBVARBINARY;
+		return s;
+	}
+
+	/* string */
+	if (*s == '\'') {
+		*type = SYBVARCHAR;
+		return tds_skip_quoted(s);
+	}
+
+	/* integer/float */
+	if (isdigit(*s) || *s == '+' || *s == '-') {
+		errno = 0;
+		strtod(s, &end);
+		if (end != s && strcspn(s, ".eE") < (size_t) (end-s) && errno == 0) {
+			*type = SYBFLT8;
+			return end;
+		}
+		errno = 0;
+		/* FIXME success if long is 64bit */
+		strtol(s, &end, 10);
+		if (end != s && errno == 0) {
+			*type = SYBINT4;
+			return end;
+		}
+	}
+
+	/* TODO date, time */
+
+	return NULL;
+}
+
+const char *
+odbc_skip_rpc_name(const char *s)
+{
+	for (;*s; ++s) {
+		if (*s == '[') {
+			/* handle [dbo].[name] and [master]..[name] syntax */
+			s = tds_skip_quoted(s);
+			if (*s != '.')
+				break;
+		} else if (TDS_ISSPACE(*s)) {
+			/* FIXME: stop at other characters ??? */
+			break;
+		}
+	}
+	return s;
+}
+
 SQLRETURN
 prepare_call(struct _hstmt * stmt)
 {
-	char *s, *d, *p, *buf;
+	const char *s, *p, *param_start;
+	char *buf;
 	SQLRETURN rc;
+	TDS_SERVER_TYPE type;
 
-	if (stmt->prepared_query)
-		buf = stmt->prepared_query;
-	else if (stmt->query)
-		buf = stmt->query;
-	else
+	if (tds_dstr_isempty(&stmt->query))
 		return SQL_ERROR;
 
-	if ((rc = to_native(stmt->dbc, stmt, buf)) != SQL_SUCCESS)
+	if ((!tds_dstr_isempty(&stmt->attr.qn_msgtext) || !tds_dstr_isempty(&stmt->attr.qn_options)) && !IS_TDS72_PLUS(stmt->dbc->tds_socket->conn)) {
+		odbc_errs_add(&stmt->errs, "HY000", "Feature is not supported by this server");
+		return SQL_SUCCESS_WITH_INFO;
+	}
+
+	if ((rc = to_native(stmt->dbc, stmt, &stmt->query)) != SQL_SUCCESS)
 		return rc;
 
-	/* TODO optimize ? */
-	d = strchr(buf, 0);
-
 	/* now detect RPC */
+	if (stmt->prepared_query_is_rpc == 0)
+		return SQL_SUCCESS;
 	stmt->prepared_query_is_rpc = 0;
-	s = buf;
+
+	s = buf = tds_dstr_buf(&stmt->query);
 	while (TDS_ISSPACE(*s))
 		++s;
 	if (strncasecmp(s, "exec", 4) == 0) {
@@ -191,46 +270,59 @@ prepare_call(struct _hstmt * stmt)
 			s += 5;
 		else if (strncasecmp(s, "execute", 7) == 0 && TDS_ISSPACE(s[7]))
 			s += 8;
-		else
+		else {
+			stmt->prepared_query_is_func = 0;
 			return SQL_SUCCESS;
+		}
 	}
 	while (TDS_ISSPACE(*s))
 		++s;
 	p = s;
-	if (*s == '[') {
-		/* FIXME handle [dbo].[name] and [master]..[name] syntax */
-		s = (char *) tds_skip_quoted(s);
-	} else {
-		/* FIXME: stop at other characters ??? */
-		while (*s && !TDS_ISSPACE(*s))
-			++s;
-	}
+	s = (char *) odbc_skip_rpc_name(s);
+	param_start = s;
 	--s;			/* trick, now s point to no blank */
-	/* TODO support constant and empty parameters */
 	for (;;) {
-		while (TDS_ISSPACE(*++s));
+		while (TDS_ISSPACE(*++s))
+			continue;
 		if (!*s)
 			break;
-		if (*s != '?')
-			return SQL_SUCCESS;
-		while (TDS_ISSPACE(*++s));
+		switch (*s) {
+		case '?':
+			break;
+		case ',':
+			--s;
+			break;
+		default:
+			if (!(s = parse_const_param(s, &type))) {
+				stmt->prepared_query_is_func = 0;
+				return SQL_SUCCESS;
+			}
+			--s;
+			break;
+		}
+		while (TDS_ISSPACE(*++s))
+			continue;
 		if (!*s)
 			break;
-		if (*s != ',')
+		if (*s != ',') {
+			stmt->prepared_query_is_func = 0;
 			return SQL_SUCCESS;
+		}
 	}
 	stmt->prepared_query_is_rpc = 1;
 
 	/* remove unneeded exec */
-	assert(!*d);
-	memmove(buf, p, d - p + 1);
+	s += strlen(s);
+	memmove(buf, p, s - p);
+	tds_dstr_setlen(&stmt->query, s - p);
+	stmt->prepared_pos = buf + (param_start - p);
 
 	return SQL_SUCCESS;
 }
 
 /* TODO handle output parameter and not terminated string */
 SQLRETURN
-native_sql(struct _hdbc * dbc, char *s)
+native_sql(struct _hdbc * dbc, DSTR *s)
 {
 	return to_native(dbc, NULL, s);
 }
