@@ -1,9 +1,6 @@
 #include "common.h"
 
 
-static char software_version[] = "$Id: connect.c,v 1.14 2010/07/19 11:52:15 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static void init_connect(void);
 
 static void
@@ -78,7 +75,7 @@ main(int argc, char *argv[])
 	printf("connect string DSN connect..\n");
 	init_connect();
 	sprintf(tmp, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;", odbc_server, odbc_user, odbc_password, odbc_database);
-	CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "SI");
+	CHKDriverConnect(NULL, T(tmp), SQL_NTS, (SQLTCHAR *) tmp, sizeof(tmp)/sizeof(SQLTCHAR), &len, SQL_DRIVER_NOPROMPT, "SI");
 	odbc_disconnect();
 	++succeeded;
 
@@ -89,7 +86,7 @@ main(int argc, char *argv[])
 	/* this is expected to work with unixODBC */
 	init_connect();
 	sprintf(tmp, "DRIVER=FreeTDS;SERVERNAME=%s;UID=%s;PWD=%s;DATABASE=%s;", odbc_server, odbc_user, odbc_password, odbc_database);
-	rc = CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "SIE");
+	rc = CHKDriverConnect(NULL, T(tmp), SQL_NTS, (SQLTCHAR *) tmp, sizeof(tmp)/sizeof(SQLTCHAR), &len, SQL_DRIVER_NOPROMPT, "SIE");
 	if (rc == SQL_ERROR) {
 		printf("Unable to open data source (ret=%d)\n", rc);
 	} else {
@@ -97,16 +94,20 @@ main(int argc, char *argv[])
 	}
 	odbc_disconnect();
 
-	/* this is expected to work with iODBC */
-	init_connect();
-	sprintf(tmp, "DRIVER=%s;SERVERNAME=%s;UID=%s;PWD=%s;DATABASE=%s;", odbc_driver, odbc_server, odbc_user, odbc_password, odbc_database);
-	rc = CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "SIE");
-	if (rc == SQL_ERROR) {
-		printf("Unable to open data source (ret=%d)\n", rc);
-	} else {
-		++succeeded;
+	/* this is expected to work with iODBC
+	 * (passing shared object name as driver)
+	 */
+	if (odbc_driver[0]) {
+		init_connect();
+		sprintf(tmp, "DRIVER=%s;SERVERNAME=%s;UID=%s;PWD=%s;DATABASE=%s;", odbc_driver, odbc_server, odbc_user, odbc_password, odbc_database);
+		rc = CHKDriverConnect(NULL, T(tmp), SQL_NTS, (SQLTCHAR *) tmp, sizeof(tmp)/sizeof(SQLTCHAR), &len, SQL_DRIVER_NOPROMPT, "SIE");
+		if (rc == SQL_ERROR) {
+			printf("Unable to open data source (ret=%d)\n", rc);
+		} else {
+			++succeeded;
+		}
+		odbc_disconnect();
 	}
-	odbc_disconnect();
 
 #ifdef _WIN32
 	if (get_entry("SERVER")) {
@@ -114,7 +115,9 @@ main(int argc, char *argv[])
 		sprintf(tmp, "DRIVER=FreeTDS;SERVER=%s;UID=%s;PWD=%s;DATABASE=%s;", entry, odbc_user, odbc_password, odbc_database);
 		if (get_entry("TDS_Version"))
 			sprintf(strchr(tmp, 0), "TDS_Version=%s;", entry);
-		rc = CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "SIE");
+		if (get_entry("Port"))
+			sprintf(strchr(tmp, 0), "Port=%s;", entry);
+		rc = CHKDriverConnect(NULL, T(tmp), SQL_NTS, (SQLTCHAR *) tmp, sizeof(tmp)/sizeof(SQLTCHAR), &len, SQL_DRIVER_NOPROMPT, "SIE");
 		if (rc == SQL_ERROR) {
 			printf("Unable to open data source (ret=%d)\n", rc);
 		} else {
