@@ -2,34 +2,10 @@
 
 /* Test cursors */
 
-static char software_version[] = "$Id: cursor1.c,v 1.20 2010/07/05 09:20:32 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 #define SWAP_STMT(b) do { SQLHSTMT xyz = odbc_stmt; odbc_stmt = b; b = xyz; } while(0)
 
 static int mssql2005 = 0;
 
-static void
-CheckNoRow(const char *query)
-{
-	SQLRETURN rc;
-
-	rc = CHKExecDirect((SQLCHAR *) query, SQL_NTS, "SINo");
-	if (rc == SQL_NO_DATA)
-		return;
-
-	do {
-		SQLSMALLINT cols;
-
-		CHKNumResultCols(&cols, "S");
-		if (cols != 0) {
-			fprintf(stderr, "Data not expected here, query:\n\t%s\n", query);
-			odbc_disconnect();
-			exit(1);
-		}
-	} while (CHKMoreResults("SNo") == SQL_SUCCESS);
-}
-
 static void
 Test0(int use_sql, const char *create_sql, const char *insert_sql, const char *select_sql)
 {
@@ -63,10 +39,10 @@ Test0(int use_sql, const char *create_sql, const char *insert_sql, const char *s
 	CHKSetStmtAttr(SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER) ROWS, 0, "S");
 	CHKSetStmtAttr(SQL_ATTR_ROW_STATUS_PTR, (SQLPOINTER) statuses, 0, "S");
 	CHKSetStmtAttr(SQL_ATTR_ROWS_FETCHED_PTR, &num_row, 0, "S");
-	CHKSetCursorName((SQLCHAR *) "C1", SQL_NTS, "S");
+	CHKSetCursorName(T("C1"), SQL_NTS, "S");
 
 	/* */
-	CHKExecDirect((SQLCHAR *) select_sql, SQL_NTS, "S");
+	CHKExecDirect(T(select_sql), SQL_NTS, "S");
 
 	/* bind some rows at a time */
 	CHKBindCol(1, SQL_C_ULONG, n, 0, n_len, "S");
@@ -84,13 +60,10 @@ Test0(int use_sql, const char *create_sql, const char *insert_sql, const char *s
 		/* delete a row */
 		i = 1;
 		if (i > 0 && i <= num_row) {
-			if (mssql2005)
-				CHKSetPos(i, use_sql ? SQL_POSITION : SQL_DELETE, SQL_LOCK_NO_CHANGE, "SI");
-			else
-				CHKSetPos(i, use_sql ? SQL_POSITION : SQL_DELETE, SQL_LOCK_NO_CHANGE, "S");
+			CHKSetPos(i, use_sql ? SQL_POSITION : SQL_DELETE, SQL_LOCK_NO_CHANGE, mssql2005 ? "SI": "S");
 			if (use_sql) {
 				SWAP_STMT(stmt2);
-				CHKPrepare((SQLCHAR *) "DELETE FROM #test WHERE CURRENT OF C1", SQL_NTS, "S");
+				CHKPrepare(T("DELETE FROM #test WHERE CURRENT OF C1"), SQL_NTS, "S");
 				CHKExecute("S");
 				SWAP_STMT(stmt2);
 			}
@@ -104,21 +77,21 @@ Test0(int use_sql, const char *create_sql, const char *insert_sql, const char *s
 			if (strstr(select_sql, "#a") == NULL || use_sql) {
 				CHKSetPos(i, use_sql ? SQL_POSITION : SQL_UPDATE, SQL_LOCK_NO_CHANGE, "S");
 			} else {
-				unsigned char sqlstate[6];
-				unsigned char msg[256];
+				SQLTCHAR sqlstate[6];
+				SQLTCHAR msg[256];
 
 				n[i - 1] = 321;
 				CHKSetPos(i, use_sql ? SQL_POSITION : SQL_UPDATE, SQL_LOCK_NO_CHANGE, "E");
 
-				CHKGetDiagRec(SQL_HANDLE_STMT, odbc_stmt, 1, sqlstate, NULL, msg, sizeof(msg), NULL, "S");
-				if (strstr((char *) msg, "Invalid column name 'c'") == NULL) {
+				CHKGetDiagRec(SQL_HANDLE_STMT, odbc_stmt, 1, sqlstate, NULL, msg, ODBC_VECTOR_SIZE(msg), NULL, "S");
+				if (strstr(C(msg), "Invalid column name 'c'") == NULL) {
 					fprintf(stderr, "Expected message not found at line %d\n", __LINE__);
 					exit(1);
 				}
 			}
 			if (use_sql) {
 				SWAP_STMT(stmt2);
-				CHKPrepare((SQLCHAR *) "UPDATE #test SET c=? WHERE CURRENT OF C1", SQL_NTS, "S");
+				CHKPrepare(T("UPDATE #test SET c=? WHERE CURRENT OF C1"), SQL_NTS, "S");
 				CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, C_LEN, 0, c[i - 1], 0, NULL, "S");
 				CHKExecute("S");
 				/* FIXME this is not necessary for mssql driver */
@@ -135,12 +108,12 @@ Test0(int use_sql, const char *create_sql, const char *insert_sql, const char *s
 	odbc_reset_statement();
 
 	/* test values */
-	CheckNoRow("IF (SELECT COUNT(*) FROM #test) <> 4 SELECT 1");
-	CheckNoRow("IF NOT EXISTS(SELECT * FROM #test WHERE i = 3 AND c = 'ccc') SELECT 1");
-	CheckNoRow("IF NOT EXISTS(SELECT * FROM #test WHERE i = 4 AND c = 'dddd') SELECT 1");
+	odbc_check_no_row("IF (SELECT COUNT(*) FROM #test) <> 4 SELECT 1");
+	odbc_check_no_row("IF NOT EXISTS(SELECT * FROM #test WHERE i = 3 AND c = 'ccc') SELECT 1");
+	odbc_check_no_row("IF NOT EXISTS(SELECT * FROM #test WHERE i = 4 AND c = 'dddd') SELECT 1");
 	if (strstr(select_sql, "#a") == NULL || use_sql) {
-		CheckNoRow("IF NOT EXISTS(SELECT * FROM #test WHERE i = 2 AND c = 'foo') SELECT 1");
-		CheckNoRow("IF NOT EXISTS(SELECT * FROM #test WHERE i = 6 AND c = 'foo') SELECT 1");
+		odbc_check_no_row("IF NOT EXISTS(SELECT * FROM #test WHERE i = 2 AND c = 'foo') SELECT 1");
+		odbc_check_no_row("IF NOT EXISTS(SELECT * FROM #test WHERE i = 6 AND c = 'foo') SELECT 1");
 	}
 }
 
