@@ -1,28 +1,14 @@
 #include "common.h"
 #include <ctype.h>
+#include "parser.h"
+#include <odbcss.h>
 
 /*
  * SQLDescribeCol test for precision
  * test what say SQLDescribeCol about precision using some type
  */
 
-static char software_version[] = "$Id: describecol.c,v 1.19 2010/08/04 06:55:45 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static int g_result = 0;
-static unsigned int line_num;
-
-static void
-fatal(const char *msg, ...)
-{
-	va_list ap;
-
-	va_start(ap, msg);
-	vfprintf(stderr, msg, ap);
-	va_end(ap);
-
-	exit(1);
-}
 
 static int
 get_int(const char *s)
@@ -31,10 +17,10 @@ get_int(const char *s)
 	long l;
 
 	if (!s)
-		fatal("Line %u: NULL int\n", line_num);
+		odbc_fatal(": NULL int\n");
 	l = strtol(s, &end, 0);
 	if (end[0])
-		fatal("Line %u: Invalid int\n", line_num);
+		odbc_fatal(": Invalid int\n");
 	return (int) l;
 }
 
@@ -57,6 +43,23 @@ lookup(const char *name, const struct lookup_int *table)
 	return get_int(name);
 }
 
+static const char*
+unlookup(long int value, const struct lookup_int *table)
+{
+	static char buf[32];
+
+	sprintf(buf, "%ld", value);
+	if (!table)
+		return buf;
+
+	for (; table->name; ++table)
+		if (table->value == value)
+			return table->name;
+
+	return buf;
+}
+
+
 static struct lookup_int sql_types[] = {
 #define TYPE(s) { #s, s }
 	TYPE(SQL_CHAR),
@@ -84,10 +87,23 @@ static struct lookup_int sql_types[] = {
 	TYPE(SQL_TYPE_DATE),
 	TYPE(SQL_TYPE_TIME),
 	TYPE(SQL_TYPE_TIMESTAMP),
+	TYPE(SQL_DATETIME),
+	TYPE(SQL_SS_VARIANT),
+	TYPE(SQL_SS_UDT),
+	TYPE(SQL_SS_XML),
+	TYPE(SQL_SS_TABLE),
+	TYPE(SQL_SS_TIME2),
+	TYPE(SQL_SS_TIMESTAMPOFFSET),
 #undef TYPE
 	{ NULL, 0 }
 };
 
+static struct lookup_int sql_bools[] = {
+	{ "SQL_TRUE",  SQL_TRUE  },
+	{ "SQL_FALSE", SQL_FALSE },
+	{ NULL, 0 }
+};
+
 typedef enum
 {
 	type_INTEGER,
@@ -117,7 +133,8 @@ static const struct attribute attributes[] = {
 	ATTR(SQL_DESC_DISPLAY_SIZE, INTEGER),
 	ATTR(SQL_DESC_TYPE_NAME, CHARP),
 	ATTR2(SQL_DESC_CONCISE_TYPE, SMALLINT, sql_types),
-	ATTR2(SQL_DESC_TYPE, SMALLINT, sql_types)
+	ATTR2(SQL_DESC_TYPE, SMALLINT, sql_types),
+	ATTR2(SQL_DESC_UNSIGNED, SMALLINT, sql_bools)
 #undef ATTR2
 #undef ATTR
 };
@@ -128,16 +145,14 @@ lookup_attr(const char *name)
 	unsigned int i;
 
 	if (!name)
-		fatal("Line %u: NULL attribute\n", line_num);
+		odbc_fatal(": NULL attribute\n");
 	for (i = 0; i < sizeof(attributes) / sizeof(attributes[0]); ++i)
 		if (strcmp(attributes[i].name, name) == 0 || strcmp(attributes[i].name + 4, name) == 0)
 			return &attributes[i];
-	fatal("Line %u: attribute %s not found\n", line_num, name);
+	odbc_fatal(": attribute %s not found\n", name);
 	return NULL;
 }
 
-#define SEP " \t\n"
-
 #define ATTR_PARAMS const struct attribute *attr, const char *expected_value
 typedef void (*check_attr_t) (ATTR_PARAMS);
 
@@ -148,16 +163,16 @@ check_attr_ird(ATTR_PARAMS)
 	SQLRETURN ret;
 
 	if (attr->type == type_CHARP) {
-		char buf[128];
+		char buf[256];
 		SQLSMALLINT len;
 
 		ret = SQLColAttribute(odbc_stmt, 1, attr->value, buf, sizeof(buf), &len, NULL);
 		if (!SQL_SUCCEEDED(ret))
-			fatal("Line %u: failure not expected\n", line_num);
+			odbc_fatal(": failure not expected\n");
 		buf[sizeof(buf)-1] = 0;
-		if (strcmp(buf, expected_value) != 0) {
+		if (strcmp(C((SQLTCHAR*) buf), expected_value) != 0) {
 			g_result = 1;
-			fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", line_num, attr->name, buf, expected_value);
+			fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", odbc_line_num, attr->name, buf, expected_value);
 		}
 		return;
 	}
@@ -165,18 +180,25 @@ check_attr_ird(ATTR_PARAMS)
 	i = 0xdeadbeef;
 	ret = SQLColAttribute(odbc_stmt, 1, attr->value, NULL, SQL_IS_INTEGER, NULL, &i);
 	if (!SQL_SUCCEEDED(ret))
-		fatal("Line %u: failure not expected\n", line_num);
+		odbc_fatal(": failure not expected\n");
 	/* SQL_DESC_LENGTH is the same of SQLDescribeCol len */
 	if (attr->value == SQL_DESC_LENGTH) {
-		SQLSMALLINT si;
-		SQLULEN li;
-		CHKDescribeCol(1, NULL, 0, NULL, &si, &li, &si, &si, "S");
-		if (i != li)
-			fatal("Line %u: attr %s SQLDescribeCol len %ld != SQLColAttribute len %ld\n", line_num, attr->name, (long) li, (long) i);
+		SQLSMALLINT scale, si;
+		SQLULEN prec;
+		CHKDescribeCol(1, NULL, 0, NULL, &si, &prec, &scale, &si, "S");
+		if (i != prec)
+			odbc_fatal(": attr %s SQLDescribeCol len %ld != SQLColAttribute len %ld\n", attr->name, (long) prec, (long) i);
+	}
+	if (attr->value == SQL_DESC_SCALE) {
+		SQLSMALLINT scale, si;
+		SQLULEN prec;
+		CHKDescribeCol(1, NULL, 0, NULL, &si, &prec, &scale, &si, "S");
+		if (i != scale)
+			odbc_fatal(": attr %s SQLDescribeCol scale %ld != SQLColAttribute len %ld\n", attr->name, (long) scale, (long) i);
 	}
 	if (i != lookup(expected_value, attr->lookup)) {
 		g_result = 1;
-		fprintf(stderr, "Line %u: invalid %s got %ld expected %s\n", line_num, attr->name, (long int) i, expected_value);
+		fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", odbc_line_num, attr->name, unlookup(i, attr->lookup), expected_value);
 	}
 }
 
@@ -188,7 +210,7 @@ check_attr_ard(ATTR_PARAMS)
 	SQLLEN li;
 	SQLRETURN ret;
 	SQLHDESC desc = SQL_NULL_HDESC;
-	char buf[128];
+	char buf[256];
 
 	/* get ARD */
 	SQLGetStmtAttr(odbc_stmt, SQL_ATTR_APP_ROW_DESC, &desc, sizeof(desc), &ind);
@@ -198,32 +220,32 @@ check_attr_ard(ATTR_PARAMS)
 	case type_INTEGER:
 		i = 0xdeadbeef;
 		ret = SQLGetDescField(desc, 1, attr->value, (SQLPOINTER) & i, sizeof(SQLINTEGER), &ind);
+		li = i;
 		break;
 	case type_SMALLINT:
 		si = 0xbeef;
 		ret = SQLGetDescField(desc, 1, attr->value, (SQLPOINTER) & si, sizeof(SQLSMALLINT), &ind);
-		i = si;
+		li = si;
 		break;
 	case type_LEN:
 		li = 0xdeadbeef;
 		ret = SQLGetDescField(desc, 1, attr->value, (SQLPOINTER) & li, sizeof(SQLLEN), &ind);
-		i = li;
 		break;
 	case type_CHARP:
 		ret = SQLGetDescField(desc, 1, attr->value, buf, sizeof(buf), &ind);
 		if (!SQL_SUCCEEDED(ret))
-			fatal("Line %u: failure not expected\n", line_num);
-		if (strcmp(buf, expected_value) != 0) {
+			odbc_fatal(": failure not expected\n");
+		if (strcmp(C((SQLTCHAR*) buf), expected_value) != 0) {
 			g_result = 1;
-			fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", line_num, attr->name, buf, expected_value);
+			fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", odbc_line_num, attr->name, buf, expected_value);
 		}
 		return;
 	}
 	if (!SQL_SUCCEEDED(ret))
-		fatal("Line %u: failure not expected\n", line_num);
-	if (i != lookup(expected_value, attr->lookup)) {
+		odbc_fatal(": failure not expected\n");
+	if (li != lookup(expected_value, attr->lookup)) {
 		g_result = 1;
-		fprintf(stderr, "Line %u: invalid %s got %ld expected %s\n", line_num, attr->name, (long int) i, expected_value);
+		fprintf(stderr, "Line %u: invalid %s got %s expected %s\n", odbc_line_num, attr->name, unlookup(i, attr->lookup), expected_value);
 	}
 }
 
@@ -236,43 +258,43 @@ check_attr_none(ATTR_PARAMS)
 int
 main(int argc, char *argv[])
 {
+	int cond = 1;
 #define TEST_FILE "describecol.in"
 	const char *in_file = FREETDS_SRCDIR "/" TEST_FILE;
 	FILE *f;
-	char buf[256];
 	SQLINTEGER i;
 	SQLLEN len;
 	check_attr_t check_attr_p = check_attr_none;
 
 	odbc_connect();
+	odbc_init_bools();
 	odbc_command("SET TEXTSIZE 4096");
 
 	SQLBindCol(odbc_stmt, 1, SQL_C_SLONG, &i, sizeof(i), &len);
 
 	f = fopen(in_file, "r");
 	if (!f)
-		fopen(TEST_FILE, "r");
+		f = fopen(TEST_FILE, "r");
 	if (!f) {
 		fprintf(stderr, "error opening test file\n");
 		exit(1);
 	}
 
-	line_num = 0;
-	while (fgets(buf, sizeof(buf), f)) {
-		char *p = buf, *cmd;
+	odbc_init_parser(f);
+	for (;;) {
+		char *p;
+		const char *cmd;
 
-		++line_num;
+		cmd = odbc_get_cmd_line(&p, &cond);
+		if (!cmd)
+			break;
 
-		while (isspace((unsigned char) *p))
-			++p;
-		cmd = strtok(p, SEP);
-
-		/* skip comments */
-		if (!cmd || cmd[0] == '#' || cmd[0] == 0 || cmd[0] == '\n')
-			continue;
+		ODBC_FREE();
 
 		if (strcmp(cmd, "odbc") == 0) {
-			int odbc3 = get_int(strtok(NULL, SEP)) == 3 ? 1 : 0;
+			int odbc3 = get_int(odbc_get_tok(&p)) == 3 ? 1 : 0;
+
+			if (!cond) continue;
 
 			if (odbc_use_version3 != odbc3) {
 				odbc_use_version3 = odbc3;
@@ -285,14 +307,16 @@ main(int argc, char *argv[])
 
 		/* select type */
 		if (strcmp(cmd, "select") == 0) {
-			const char *type = strtok(NULL, SEP);
-			const char *value = strtok(NULL, SEP);
-			char sql[sizeof(buf) + 40];
+			const char *type = odbc_get_str(&p);
+			const char *value = odbc_get_str(&p);
+			char sql[1024];
+
+			if (!cond) continue;
 
 			SQLMoreResults(odbc_stmt);
 			odbc_reset_statement();
 
-			sprintf(sql, "SELECT CONVERT(%s, %s) AS col", type, value);
+			snprintf(sql, sizeof(sql), "SELECT CONVERT(%s, %s) AS col", type, value);
 
 			/* ignore error, we only need precision of known types */
 			check_attr_p = check_attr_none;
@@ -309,14 +333,16 @@ main(int argc, char *argv[])
 
 		/* set attribute */
 		if (strcmp(cmd, "set") == 0) {
-			const struct attribute *attr = lookup_attr(strtok(NULL, SEP));
-			char *value = strtok(NULL, SEP);
+			const struct attribute *attr = lookup_attr(odbc_get_tok(&p));
+			const char *value = odbc_get_str(&p);
 			SQLHDESC desc;
 			SQLRETURN ret;
 			SQLINTEGER ind;
 
 			if (!value)
-				fatal("Line %u: value not defined\n", line_num);
+				odbc_fatal(": value not defined\n");
+
+			if (!cond) continue;
 
 			/* get ARD */
 			SQLGetStmtAttr(odbc_stmt, SQL_ATTR_APP_ROW_DESC, &desc, sizeof(desc), &ind);
@@ -340,23 +366,26 @@ main(int argc, char *argv[])
 				break;
 			}
 			if (!SQL_SUCCEEDED(ret))
-				fatal("Line %u: failure not expected setting ARD attribute\n", line_num);
+				odbc_fatal(": failure not expected setting ARD attribute\n");
 			check_attr_p = check_attr_ard;
 		}
 
 		/* test attribute */
 		if (strcmp(cmd, "attr") == 0) {
-			const struct attribute *attr = lookup_attr(strtok(NULL, SEP));
-			char *expected = strtok(NULL, SEP);
+			const struct attribute *attr = lookup_attr(odbc_get_tok(&p));
+			const char *expected = odbc_get_str(&p);
+
+			if (!cond) continue;
 
 			if (!expected)
-				fatal("Line %u: value not defined\n", line_num);
+				odbc_fatal(": value not defined\n");
 
 			check_attr_p(attr, expected);
 		}
 	}
 
 	fclose(f);
+	odbc_clear_bools();
 	odbc_disconnect();
 
 	printf("Done.\n");
