@@ -18,9 +18,6 @@
  * Also we have to check normal char and wide char
  */
 
-static char software_version[] = "$Id: genparams.c,v 1.47 2010/10/29 08:49:30 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 #ifdef TDS_NO_DM
 static const char tds_no_dm = 1;
 #else
@@ -31,15 +28,14 @@ static char precision = 18;
 static char exec_direct = 0;
 static char prepare_before = 0;
 static char use_cursors = 0;
+static int only_test = 0;
 
-static void
+static int
 TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_type, SQLSMALLINT out_sql_type, const char *expected)
 {
 	char sbuf[1024];
 	unsigned char out_buf[256];
 	SQLLEN out_len = 0;
-	SQL_NUMERIC_STRUCT *num;
-	int i;
 	const char *sep;
 
 	odbc_reset_statement();
@@ -65,16 +61,16 @@ TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_typ
 			     sizeof(out_buf), &out_len, "S");
 
 		/* call store procedure */
-		CHKExecDirect((SQLCHAR *) "{call spTestProc(?)}", SQL_NTS, "S");
+		CHKExecDirect(T("{call spTestProc(?)}"), SQL_NTS, "S");
 	} else {
 		if (prepare_before)
-			CHKPrepare((SQLCHAR *) "{call spTestProc(?)}", SQL_NTS, "S");
+			CHKPrepare(T("{call spTestProc(?)}"), SQL_NTS, "S");
 
 		CHKBindParameter(1, SQL_PARAM_OUTPUT, out_c_type, out_sql_type, precision, 0, out_buf,
 			     sizeof(out_buf), &out_len, "S");
 
 		if (!prepare_before)
-			CHKPrepare((SQLCHAR *) "{call spTestProc(?)}", SQL_NTS, "S");
+			CHKPrepare(T("{call spTestProc(?)}"), SQL_NTS, "S");
 
 		CHKExecute("S");
 	}
@@ -87,35 +83,21 @@ TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_typ
 		SQLMoreResults(odbc_stmt);
 
 	/* test results */
-	sbuf[0] = 0;
-	switch (out_c_type) {
-	case SQL_C_NUMERIC:
-		num = (SQL_NUMERIC_STRUCT *) out_buf;
-		sprintf(sbuf, "%d %d %d ", num->precision, num->scale, num->sign);
-		i = SQL_MAX_NUMERIC_LEN;
-		for (; i > 0 && !num->val[--i];);
-		for (; i >= 0; --i)
-			sprintf(strchr(sbuf, 0), "%02X", num->val[i]);
-		break;
-	case SQL_C_BINARY:
-		assert(out_len >= 0);
-		for (i = 0; i < out_len; ++i)
-			sprintf(strchr(sbuf, 0), "%02X", (int) out_buf[i]);
-		break;
-	case SQL_C_CHAR:
-		sprintf(sbuf, "%d %s", (int) out_len, out_buf);
-		break;
-	default:
-		/* not supported */
-		assert(0);
-		break;
-	}
+	odbc_c2string(sbuf, out_c_type, out_buf, out_len);
 
 	if (strcmp(sbuf, expected) != 0) {
+		if (only_test) {
+			odbc_command("drop proc spTestProc");
+			ODBC_FREE();
+			return 1;
+		}
 		fprintf(stderr, "Wrong result\n  Got: %s\n  Expected: %s\n", sbuf, expected);
 		exit(1);
 	}
+	only_test = 0;
 	odbc_command("drop proc spTestProc");
+	ODBC_FREE();
+	return 0;
 }
 
 static char check_truncation = 0;
@@ -169,17 +151,17 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 		CHKBindParameter(1, SQL_PARAM_INPUT, out_c_type, out_sql_type, 20, 0, out_buf, sizeof(out_buf), &out_len, "S");
 
 		if (check_truncation)
-			CHKExecDirect((SQLCHAR *) sbuf, SQL_NTS, "E");
+			CHKExecDirect(T(sbuf), SQL_NTS, "E");
 		else
-			CHKExecDirect((SQLCHAR *) sbuf, SQL_NTS, "SNo");
+			CHKExecDirect(T(sbuf), SQL_NTS, "SNo");
 	} else {
 		if (prepare_before)
-			CHKPrepare((SQLCHAR *) sbuf, SQL_NTS, "S");
+			CHKPrepare(T(sbuf), SQL_NTS, "S");
 
 		CHKBindParameter(1, SQL_PARAM_INPUT, out_c_type, out_sql_type, 20, 0, out_buf, sizeof(out_buf), &out_len, "S");
 
 		if (!prepare_before)
-			CHKPrepare((SQLCHAR *) sbuf, SQL_NTS, "S");
+			CHKPrepare(T(sbuf), SQL_NTS, "S");
 
 		if (check_truncation)
 			CHKExecute("E");
@@ -209,6 +191,7 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 	}
 	check_truncation = 0;
 	odbc_command("DROP TABLE #tmp_insert");
+	ODBC_FREE();
 }
 
 /* stripped down version of TestInput for NULLs */
@@ -236,15 +219,15 @@ NullInput(SQLSMALLINT out_c_type, SQLSMALLINT out_sql_type, const char *param_ty
 	if (exec_direct) {
 		CHKBindParameter(1, SQL_PARAM_INPUT, out_c_type, out_sql_type, 20, 0, NULL, 1, &out_len, "S");
 
-		CHKExecDirect((SQLCHAR *) sbuf, SQL_NTS, "SNo");
+		CHKExecDirect(T(sbuf), SQL_NTS, "SNo");
 	} else {
 		if (prepare_before)
-			CHKPrepare((SQLCHAR *) sbuf, SQL_NTS, "S");
+			CHKPrepare(T(sbuf), SQL_NTS, "S");
 
 		CHKBindParameter(1, SQL_PARAM_INPUT, out_c_type, out_sql_type, 20, 0, NULL, 1, &out_len, "S");
 
 		if (!prepare_before)
-			CHKPrepare((SQLCHAR *) sbuf, SQL_NTS, "S");
+			CHKPrepare(T(sbuf), SQL_NTS, "S");
 
 		CHKExecute("SNo");
 	}
@@ -252,7 +235,7 @@ NullInput(SQLSMALLINT out_c_type, SQLSMALLINT out_sql_type, const char *param_ty
 	/* check if row is present */
 	odbc_reset_statement();
 	if (!odbc_db_is_microsoft() && strcmp(param_type, "TEXT") == 0)
-		odbc_command("SELECT * FROM #tmp_insert WHERE col LIKE ''");
+		odbc_command("SELECT * FROM #tmp_insert WHERE DATALENGTH(col) = 0 OR DATALENGTH(col) IS NULL");
 	else
 		odbc_command("SELECT * FROM #tmp_insert WHERE col IS NULL");
 
@@ -260,11 +243,43 @@ NullInput(SQLSMALLINT out_c_type, SQLSMALLINT out_sql_type, const char *param_ty
 	CHKFetch("No");
 	CHKMoreResults("No");
 	odbc_command("DROP TABLE #tmp_insert");
+	ODBC_FREE();
 }
 
 
 static int big_endian = 1;
 
+static const char*
+pack(const char *fmt, ...)
+{
+	static char out[80];
+	char *p = out;
+
+	va_list v;
+	va_start(v, fmt);
+	for (; *fmt; ++fmt) {
+		unsigned n = va_arg(v, unsigned);
+		int i, l = 2;
+
+		assert(p - out + 8 < sizeof(out));
+		switch (*fmt) {
+		case 'l':
+			l += 2;
+		case 's':
+			for (i = 0; i < l; ++i) {
+				sprintf(p, "%02X", (n >> (8*(big_endian ? l-1-i : i))) & 0xffu);
+				p += 2;
+			}
+			break;
+		default:
+			assert(0);
+		}
+	}
+	*p = 0;
+	va_end(v);
+	return out;
+}
+
 static void
 AllTests(void)
 {
@@ -303,13 +318,20 @@ AllTests(void)
 	 */
 	if (odbc_driver_is_freetds())
 		TestOutput("VARCHAR(20)", "313233", SQL_C_BINARY, SQL_VARCHAR, "333133323333");
-	/* FIXME our driver ignore precision for date */
+
+	only_test = 1;
 	precision = 3;
-	/* Some MS driver incorrectly prepare with smalldatetime*/
-	if (!use_cursors || odbc_driver_is_freetds())
-		TestOutput("DATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP, big_endian ? "0000949700FBAA2C" : "979400002CAAFB00");
-	TestOutput("SMALLDATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP,
-	     big_endian ? "0000949700FB9640" : "979400004096FB00");
+	if (TestOutput("DATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP, pack("ssssssl", 2004, 2, 24, 15, 16, 17, 0))) {
+		/* FIXME our driver ignore precision for date */
+		precision = 3;
+		/* Some MS driver incorrectly prepare with smalldatetime*/
+		if (!use_cursors || odbc_driver_is_freetds()) {
+				TestOutput("DATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP, pack("ll", 0x9497, 0xFBAA2C));
+		}
+		TestOutput("SMALLDATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP, pack("ll", 0x9497, 0xFB9640));
+	} else {
+		TestOutput("SMALLDATETIME", "2004-02-24 15:16:17", SQL_C_BINARY, SQL_TIMESTAMP, pack("ssssssl", 2004, 2, 24, 15, 16, 0, 0));
+	}
 	TestInput(SQL_C_TYPE_TIMESTAMP, "DATETIME", SQL_TYPE_TIMESTAMP, "DATETIME", "2005-07-22 09:51:34");
 
 	/* test timestamp millisecond round off */
@@ -384,9 +406,12 @@ AllTests(void)
 	/* output from char with conversions */
 	TestOutput("VARCHAR(20)", "foo test", SQL_C_CHAR, SQL_VARCHAR, "6 foo te");
 	/* TODO use collate in sintax if available */
-	TestOutput("VARCHAR(20)", "0xf8f9", SQL_C_CHAR, SQL_VARCHAR, "2 \xf8\xf9");
+	/* while usually on Microsoft database this encoding is valid on Sybase the database
+	 * could use UTF-8 encoding where \xf8\xf9 is an invalid encoded string */
+	if (odbc_db_is_microsoft() && odbc_tds_version() > 0x700)
+		TestOutput("VARCHAR(20)", "0xf8f9", SQL_C_CHAR, SQL_VARCHAR, "2 \xf8\xf9");
 
-	if ((odbc_db_is_microsoft() && odbc_db_version_int() >= 0x08000000u)
+	if ((odbc_db_is_microsoft() && odbc_db_version_int() >= 0x08000000u && odbc_tds_version() > 0x700)
 	    || (!odbc_db_is_microsoft() && strncmp(odbc_db_version(), "15.00.", 6) >= 0)) {
 		TestOutput("BIGINT", "-987654321065432", SQL_C_BINARY, SQL_BIGINT, big_endian ? "FFFC7DBBCF083228" : "283208CFBB7DFCFF");
 		TestInput(SQL_C_SBIGINT, "BIGINT", SQL_BIGINT, "BIGINT", "-12345678901234");
@@ -454,6 +479,8 @@ main(int argc, char *argv[])
 		if (use_cursors) {
 			if (!tds_no_dm || !odbc_driver_is_freetds())
 				odbc_reset_statement();
+			/* if connection does not support cursors returns success */
+			setenv("TDS_SKIP_SUCCESS", "1", 1);
 			odbc_check_cursor();
 		}
 
