@@ -1,9 +1,9 @@
+/*
+ * Test reading data with SQLGetData
+ */
 #include "common.h"
 #include <assert.h>
 
-static char software_version[] = "$Id: getdata.c,v 1.18 2010/07/05 09:20:33 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static void
 test_err(const char *data, int c_type, const char *state)
 {
@@ -48,6 +48,86 @@ mycmp(const char *s1, const char *s2)
 	return memcmp(s1, buf, l * lc + lc);
 }
 
+static void
+test_split(const char *n_flag)
+{
+#define CheckLen(x) do { \
+	if (len != (x)) { \
+		fprintf(stderr, "Wrong len %ld at line %d expected %d\n", (long int) len, __LINE__, (x)); \
+		exit(1); \
+	} \
+	} while(0)
+
+	char sql[80];
+	char *buf = NULL;
+	SQLLEN len;
+
+	/* TODO test with VARCHAR too */
+	sprintf(sql, "SELECT CONVERT(%sTEXT,'Prova' + REPLICATE('x',500))", n_flag);
+	odbc_command(sql);
+
+	CHKFetch("S");
+
+	/* these 2 tests test an old severe BUG in FreeTDS */
+	buf = (char *) ODBC_GET(1);
+	CHKGetData(1, type, buf, 0, &len, "I");
+	if (len != SQL_NO_TOTAL)
+		CheckLen(505*lc);
+	CHKGetData(1, type, buf, 0, &len, "I");
+	if (len != SQL_NO_TOTAL)
+		CheckLen(505*lc);
+	buf = (char *) ODBC_GET(3*lc);
+	CHKGetData(1, type, buf, 3 * lc, &len, "I");
+	if (len != SQL_NO_TOTAL)
+		CheckLen(505*lc);
+	if (mycmp(buf, "Pr") != 0) {
+		printf("Wrong data result 1\n");
+		exit(1);
+	}
+
+	buf = (char *) ODBC_GET(16*lc);
+	CHKGetData(1, type, buf, 16 * lc, &len, "I");
+	if (len != SQL_NO_TOTAL)
+		CheckLen(503*lc);
+	if (mycmp(buf, "ovaxxxxxxxxxxxx") != 0) {
+		printf("Wrong data result 2 res = '%s'\n", buf);
+		exit(1);
+	}
+
+	buf = (char *) ODBC_GET(256*lc);
+	CHKGetData(1, type, buf, 256 * lc, &len, "I");
+	if (len != SQL_NO_TOTAL)
+		CheckLen(488*lc);
+	CHKGetData(1, type, buf, 256 * lc, &len, "S");
+	CheckLen(233*lc);
+	CHKGetData(1, type, buf, 256 * lc, &len, "No");
+
+	odbc_reset_statement();
+
+	/* test with varchar, not blob but variable */
+	sprintf(sql, "SELECT CONVERT(%sVARCHAR(100), 'Other test')", n_flag);
+	odbc_command(sql);
+
+	CHKFetch("S");
+
+	buf = (char *) ODBC_GET(7*lc);
+	CHKGetData(1, type, buf, 7 * lc, NULL, "I");
+	if (mycmp(buf, "Other ") != 0) {
+		printf("Wrong data result 1\n");
+		exit(1);
+	}
+
+	buf = (char *) ODBC_GET(5*lc);
+	CHKGetData(1, type, buf, 20, NULL, "S");
+	if (mycmp(buf, "test") != 0) {
+		printf("Wrong data result 2 res = '%s'\n", buf);
+		exit(1);
+	}
+	ODBC_FREE();
+
+	odbc_reset_statement();
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -60,54 +140,20 @@ main(int argc, char *argv[])
 
 	lc = 1;
 	type = SQL_C_CHAR;
+	test_split("");
 
-	for (;;) {
-		/* TODO test with VARCHAR too */
-		odbc_command("SELECT CONVERT(TEXT,'Prova')");
+	lc = sizeof(SQLWCHAR);
+	type = SQL_C_WCHAR;
+	test_split("");
 
-		CHKFetch("S");
+	if (odbc_db_is_microsoft() && odbc_db_version_int() >= 0x07000000u) {
+		lc = 1;
+		type = SQL_C_CHAR;
+		test_split("N");
 
-		/* these 2 tests test an old severe BUG in FreeTDS */
-		CHKGetData(1, type, buf, 0, NULL, "I");
-		CHKGetData(1, type, buf, 0, NULL, "I");
-		CHKGetData(1, type, buf, 3 * lc, NULL, "I");
-		if (mycmp(buf, "Pr") != 0) {
-			printf("Wrong data result 1\n");
-			exit(1);
-		}
-
-		CHKGetData(1, type, buf, 16, NULL, "S");
-		if (mycmp(buf, "ova") != 0) {
-			printf("Wrong data result 2 res = '%s'\n", buf);
-			exit(1);
-		}
-
-		odbc_reset_statement();
-
-		/* test with varchar, not blob but variable */
-		odbc_command("SELECT CONVERT(VARCHAR(100), 'Other test')");
-
-		CHKFetch("S");
-
-		CHKGetData(1, type, buf, 7 * lc, NULL, "I");
-		if (mycmp(buf, "Other ") != 0) {
-			printf("Wrong data result 1\n");
-			exit(1);
-		}
-
-		CHKGetData(1, type, buf, 20, NULL, "S");
-		if (mycmp(buf, "test") != 0) {
-			printf("Wrong data result 2 res = '%s'\n", buf);
-			exit(1);
-		}
-
-		odbc_reset_statement();
-
-		if (type != SQL_C_CHAR)
-			break;
-
-		type = SQL_C_WCHAR;
 		lc = sizeof(SQLWCHAR);
+		type = SQL_C_WCHAR;
+		test_split("N");
 	}
 
 	/* test with fixed length */
@@ -218,6 +264,8 @@ main(int argc, char *argv[])
 		type = SQL_C_CHAR;
 
 		for (;;) {
+			void *buf = ODBC_GET(lc);
+
 			odbc_command("SELECT CONVERT(TEXT,'')");
 
 			CHKFetch("S");
@@ -232,6 +280,25 @@ main(int argc, char *argv[])
 
 			CHKGetData(1, type, buf, lc, NULL, "No");
 			odbc_reset_statement();
+			ODBC_FREE();
+
+			buf = ODBC_GET(4096*lc);
+
+			odbc_command("SELECT CONVERT(TEXT,'')");
+
+			CHKFetch("S");
+
+			len = 1234;
+			CHKGetData(1, type, buf, lc*4096, &len, "S");
+
+			if (len != 0) {
+				fprintf(stderr, "Wrong len returned, returned %ld\n", (long) len);
+				return 1;
+			}
+
+			CHKGetData(1, type, buf, lc*4096, NULL, "No");
+			odbc_reset_statement();
+			ODBC_FREE();
 
 			if (type != SQL_C_CHAR)
 				break;
