@@ -4,7 +4,7 @@
  * Try to make core dump using SQLBindParameter
  */
 
-static char software_version[] = "$Id: paramcore.c,v 1.8 2010/07/05 09:20:33 freddy77 Exp $";
+static char software_version[] = "$Id: paramcore.c,v 1.9 2011-07-12 10:16:59 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 #define SP_TEXT "{call sp_paramcore_test(?)}"
@@ -26,14 +26,14 @@ main(int argc, char *argv[])
 	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_VARCHAR, OUTSTRING_LEN, 0, NULL, OUTSTRING_LEN, &cb, "S");
 
 	cb = SQL_NTS;
-	CHKExecDirect((SQLCHAR*) SP_TEXT, SQL_NTS, "E");
+	CHKExecDirect(T(SP_TEXT), SQL_NTS, "E");
 	odbc_reset_statement();
 
 	/* here we pass a NULL buffer for input */
 	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_LONG, SQL_VARCHAR, 18, 0, NULL, OUTSTRING_LEN, &cb, "S");
 
 	cb = 1;
-	CHKExecDirect((SQLCHAR*) SP_TEXT, SQL_NTS, "E");
+	CHKExecDirect(T(SP_TEXT), SQL_NTS, "E");
 	odbc_reset_statement();
 
 	odbc_command("drop proc sp_paramcore_test");
