@@ -16,9 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <assert.h>
 
@@ -26,11 +24,8 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include "tds.h"
-#include "tdssrv.h"
-
-static char software_version[] = "$Id: query.c,v 1.18 2008/12/15 05:31:14 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/tds.h>
+#include <freetds/server.h>
 
 static char *query;
 static size_t query_buflen = 0;
@@ -45,7 +40,7 @@ tds_get_query(TDSSOCKET * tds)
 
 	if (query_buflen == 0) {
 		query_buflen = 1024;
-		query = (char *) malloc(query_buflen);
+		query = tds_new(char, query_buflen);
 	}
 	tds_get_byte(tds);	/* 33 */
 	len = tds_get_int(tds);	/* query size +1 */
@@ -61,6 +56,15 @@ tds_get_query(TDSSOCKET * tds)
 	return query;
 }
 
+static int
+tds_lastpacket(TDSSOCKET * tds) 
+{
+	if (!tds || !tds->in_buf || tds->recv_packet->capacity < 2)
+		return 1;
+	
+	return tds->in_buf[1] != 0;
+}
+
 /**
  * Read a query, and return it as an ASCII string with a \0 terminator.  This
  * should work for TDS4, TDS5, and TDS7+ connections.  Also, it converts RPC
