@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -36,16 +34,9 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#include <tds.h>
-#include <tdsstring.h>
-#include <tdssrv.h>
-
-#ifdef __MINGW32__
-#define sleep(s) Sleep((s)*1000)
-#endif
-
-static char software_version[] = "$Id: unittest.c,v 1.19 2009/08/25 14:25:35 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/tds.h>
+#include <freetds/string.h>
+#include <freetds/server.h>
 
 static void dump_login(TDSLOGIN * login);
 
@@ -84,7 +75,7 @@ main(int argc, char **argv)
 		tds_env_change(tds, TDS_ENV_PACKSIZE, NULL, "512");
 		/* TODO set mssql if tds7+ */
 		tds_send_login_ack(tds, "sql server");
-		if (IS_TDS50(tds))
+		if (IS_TDS50(tds->conn))
 			tds_send_capabilities_token(tds);
 		tds_send_done_token(tds, 0, 1);
 	} else {
@@ -100,8 +91,8 @@ main(int argc, char **argv)
 	resinfo = tds_alloc_results(1);
 	resinfo->columns[0]->column_type = SYBVARCHAR;
 	resinfo->columns[0]->column_size = 30;
-	strcpy(resinfo->columns[0]->column_name, "name");
-	resinfo->columns[0]->column_namelen = 4;
+	if (!tds_dstr_copy(&resinfo->columns[0]->column_name, "name"))
+		exit(1);
 	resinfo->current_row = (TDS_UCHAR*) "pubs2";
 	resinfo->columns[0]->column_data = resinfo->current_row;
 	tds_send_result(tds, resinfo);
@@ -109,7 +100,7 @@ main(int argc, char **argv)
 	tds_send_row(tds, resinfo);
 	tds_send_done_token(tds, 16, 1);
 	tds_flush_packet(tds);
-	sleep(30);
+	tds_sleep_s(30);
 
 	tds_free_results(resinfo);
 	tds_free_socket(tds);
