@@ -380,6 +380,8 @@ tds_generic_get_info(TDSSOCKET *tds, TDSCOLUMN *col)
 	case 5:
 	case 4:
 		col->column_size = tds_get_int(tds);
+		if (col->column_size < 0)
+			return TDS_FAIL;
 		break;
 	case 2:
 		/* assure > 0 */
@@ -389,6 +391,8 @@ tds_generic_get_info(TDSSOCKET *tds, TDSCOLUMN *col)
 			col->column_size = 0x3ffffffflu;
 			col->column_varint_size = 8;
 		}
+		if (col->column_size < 0)
+			return TDS_FAIL;
 		break;
 	case 1:
 		col->column_size = tds_get_byte(tds);
@@ -465,9 +469,6 @@ tds_get_char_dynamic(TDSSOCKET *tds, TDSCOLUMN *curcol, void **pp, size_t alloca
 		res = tds_convert_stream(tds, curcol->char_conv, to_client, r_stream, &w.stream);
 	else
 		res = tds_copy_stream(tds, r_stream, &w.stream);
-	if (TDS_FAILED(res))
-		return res;
-
 	curcol->column_cur_size = w.size;
 	return res;
 }
@@ -766,6 +767,7 @@ tds_generic_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 	if (is_blob_col(curcol)) {
 		TDSDATAINSTREAM r;
 		size_t allocated;
+		TDSRET ret;
 
 		blob = (TDSBLOB *) dest; 	/* cf. column_varint_size case 4, above */
 
@@ -786,7 +788,12 @@ tds_generic_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 		}
 
 		tds_datain_stream_init(&r, tds, colsize);
-		return tds_get_char_dynamic(tds, curcol, (void **) &blob->textvalue, allocated, &r.stream);
+		ret = tds_get_char_dynamic(tds, curcol, (void **) &blob->textvalue, allocated, &r.stream);
+		if (TDS_FAILED(ret) && TDS_UNLIKELY(r.wire_size > 0)) {
+			tds_get_n(tds, NULL, r.wire_size);
+			return ret;
+		}
+		return TDS_SUCCESS;
 	}
 
 	/* non-numeric and non-blob */
@@ -1012,7 +1019,10 @@ tds_generic_put(TDSSOCKET * tds, TDSCOLUMN * curcol, int bcp7)
 
 		switch (curcol->column_varint_size) {
 		case 8:
-			tds_put_int8(tds, colsize);
+			/* this difference for BCP operation is due to
+			 * a bug in different server version that does
+			 * not accept a length here */
+			tds_put_int8(tds, bcp7 ? -2 : colsize);
 			tds_put_int(tds, colsize);
 			break;
 		case 4:	/* It's a BLOB... */
@@ -1493,6 +1503,42 @@ tds_sybbigtime_put(TDSSOCKET *tds, TDSCOLUMN *col, int bcp7)
 	return TDS_SUCCESS;
 }
 
+TDSRET
+tds_invalid_get_info(TDSSOCKET * tds, TDSCOLUMN * col)
+{
+	return TDS_FAIL;
+}
+
+TDS_INT
+tds_invalid_row_len(TDSCOLUMN *col)
+{
+	return 0;
+}
+
+TDSRET
+tds_invalid_get(TDSSOCKET * tds, TDSCOLUMN * col)
+{
+	return TDS_FAIL;
+}
+
+TDSRET
+tds_invalid_put_info(TDSSOCKET * tds, TDSCOLUMN * col)
+{
+	return TDS_FAIL;
+}
+
+unsigned
+tds_invalid_put_info_len(TDSSOCKET * tds, TDSCOLUMN * col)
+{
+	return 0;
+}
+
+TDSRET
+tds_invalid_put(TDSSOCKET *tds, TDSCOLUMN *col, int bcp7)
+{
+	return TDS_FAIL;
+}
+
 #if ENABLE_EXTRA_CHECKS
 int
 tds_generic_check(const TDSCOLUMN *col)
@@ -1565,6 +1611,12 @@ tds_numeric_check(const TDSCOLUMN *col)
 
 	return 1;
 }
+
+int
+tds_invalid_check(const TDSCOLUMN *col)
+{
+	return 1;
+}
 #endif
 
 
@@ -1578,6 +1630,7 @@ TDS_DECLARE_FUNCS(variant);
 TDS_DECLARE_FUNCS(msdatetime);
 TDS_DECLARE_FUNCS(clrudt);
 TDS_DECLARE_FUNCS(sybbigtime);
+TDS_DECLARE_FUNCS(invalid);
 #include <freetds/popvis.h>
 
 static const TDSCOLUMNFUNCS *
