@@ -18,9 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
+/**
+ * \file
+ * \brief Handle character conversions to/from server
+ */
+
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -33,16 +36,11 @@
 #include <errno.h>
 #endif
 
-#include "tds.h"
-#include "tdsiconv.h"
+#include <freetds/tds.h>
+#include <freetds/iconv.h>
 #if HAVE_ICONV
 #include <iconv.h>
 #endif
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: iconv.c,v 1.148 2011/01/22 20:11:28 jklowden Exp $");
 
 #define CHARSIZE(charset) ( ((charset)->min_bytes_per_char == (charset)->max_bytes_per_char )? \
 				(charset)->min_bytes_per_char : 0 )
@@ -248,17 +246,15 @@ tds_iconv_reset(TDSICONV *conv)
 	 * (min|max)_bytes_per_char can be used to divide
 	 * so init to safe values
 	 */
-	conv->server_charset.min_bytes_per_char = 1;
-	conv->server_charset.max_bytes_per_char = 1;
-	conv->client_charset.min_bytes_per_char = 1;
-	conv->client_charset.max_bytes_per_char = 1;
+	conv->to.charset.min_bytes_per_char = 1;
+	conv->to.charset.max_bytes_per_char = 1;
+	conv->from.charset.min_bytes_per_char = 1;
+	conv->from.charset.max_bytes_per_char = 1;
 
-	conv->server_charset.name = conv->client_charset.name = "";
-	conv->server_charset.canonic = conv->client_charset.canonic = 0;
-	conv->to_wire = (iconv_t) -1;
-	conv->to_wire2 = (iconv_t) -1;
-	conv->from_wire = (iconv_t) -1;
-	conv->from_wire2 = (iconv_t) -1;
+	conv->to.charset.name = conv->from.charset.name = "";
+	conv->to.charset.canonic = conv->from.charset.canonic = 0;
+	conv->to.cd = (iconv_t) -1;
+	conv->from.cd = (iconv_t) -1;
 }
 
 /**
@@ -266,28 +262,28 @@ tds_iconv_reset(TDSICONV *conv)
  * \return 0 for success
  */
 int
-tds_iconv_alloc(TDSSOCKET * tds)
+tds_iconv_alloc(TDSCONNECTION * conn)
 {
 	int i;
 	TDSICONV *char_conv;
 
-	assert(!tds->char_convs);
-	if (!(tds->char_convs = (TDSICONV **) malloc(sizeof(TDSICONV *) * (initial_char_conv_count + 1))))
+	assert(!conn->char_convs);
+	if (!(conn->char_convs = tds_new(TDSICONV *, initial_char_conv_count + 1)))
 		return 1;
-	char_conv = (TDSICONV *) calloc(initial_char_conv_count, sizeof(TDSICONV));
+	char_conv = tds_new0(TDSICONV, initial_char_conv_count);
 	if (!char_conv) {
-		TDS_ZERO_FREE(tds->char_convs);
+		TDS_ZERO_FREE(conn->char_convs);
 		return 1;
 	}
-	tds->char_conv_count = initial_char_conv_count + 1;
+	conn->char_conv_count = initial_char_conv_count + 1;
 
 	for (i = 0; i < initial_char_conv_count; ++i) {
-		tds->char_convs[i] = &char_conv[i];
+		conn->char_convs[i] = &char_conv[i];
 		tds_iconv_reset(&char_conv[i]);
 	}
 
 	/* chardata is just a pointer to another iconv info */
-	tds->char_convs[initial_char_conv_count] = tds->char_convs[client2server_chardata];
+	conn->char_convs[initial_char_conv_count] = conn->char_convs[client2server_chardata];
 
 	return 0;
 }
@@ -300,7 +296,6 @@ tds_iconv_alloc(TDSSOCKET * tds)
  * They have fixed meanings:
  * 	\li 0. Client <-> UCS-2 (client2ucs2)
  * 	\li 1. Client <-> server single-byte charset (client2server_chardata)
- *	\li 2. ISO8859-1  <-> server meta data	(iso2server_metadata)
  *
  * Other designs that use less data are possible, but these three conversion needs are 
  * very often needed.  By reserving them, we avoid searching the array for our most common purposes.
@@ -315,19 +310,19 @@ tds_iconv_alloc(TDSSOCKET * tds)
  * (one for other iconv implementations and another for Sybase). For every
  * canonic charset name we cache the iconv name found during discovery. 
  */
-void
-tds_iconv_open(TDSSOCKET * tds, const char *charset)
+TDSRET
+tds_iconv_open(TDSCONNECTION * conn, const char *charset, int use_utf16)
 {
 	static const char UCS_2LE[] = "UCS-2LE";
 	int canonic;
 	int canonic_charset = tds_canonical_charset(charset);
-	int canonic_env_charset = tds->env.charset ? tds_canonical_charset(tds->env.charset) : -1;
+	int canonic_env_charset = conn->env.charset ? tds_canonical_charset(conn->env.charset) : -1;
 	int fOK, ret;
 
-	TDS_ENCODING *client = &tds->char_convs[client2ucs2]->client_charset;
-	TDS_ENCODING *server = &tds->char_convs[client2ucs2]->server_charset;
+	TDS_ENCODING *client = &conn->char_convs[client2ucs2]->from.charset;
+	TDS_ENCODING *server = &conn->char_convs[client2ucs2]->to.charset;
 
-	tdsdump_log(TDS_DBG_FUNC, "tds_iconv_open(%p, %s)\n", tds, charset);
+	tdsdump_log(TDS_DBG_FUNC, "tds_iconv_open(%p, %s)\n", conn, charset);
 
 	/* initialize */
 	if (!iconv_initialized) {
@@ -337,8 +332,7 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 			tdsdump_log(TDS_DBG_FUNC, "error: tds_iconv_init() returned %d; "
 						  "could not find a name for %s that your iconv accepts.\n"
 						  "use: \"configure --disable-libiconv\"", ret, names[ret-1]);
-			assert(ret == 0);
-			return;
+			return TDS_FAIL;
 		}
 		iconv_initialized = 1;
 	}
@@ -350,9 +344,17 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 
 	tdsdump_log(TDS_DBG_FUNC, "preparing iconv for \"%s\" <-> \"%s\" conversion\n", charset, UCS_2LE);
 
-	fOK = tds_iconv_info_init(tds->char_convs[client2ucs2], canonic_charset, TDS_CHARSET_UCS_2LE);
+	fOK = 0;
+	if (use_utf16) {
+		canonic = TDS_CHARSET_UTF_16LE;
+		fOK = tds_iconv_info_init(conn->char_convs[client2ucs2], canonic_charset, canonic);
+	}
+	if (!fOK) {
+		canonic = TDS_CHARSET_UCS_2LE;
+		fOK = tds_iconv_info_init(conn->char_convs[client2ucs2], canonic_charset, canonic);
+	}
 	if (!fOK)
-		return;
+		return TDS_FAIL;
 
 	/* 
 	 * How many UTF-8 bytes we need is a function of what the input character set is.
@@ -368,30 +370,19 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 	 * TODO: the server hasn't reported its charset yet, so this logic can't work here.  
 	 *       not sure what to do about that yet.  
 	 */
-	tds->char_convs[client2server_chardata]->flags = TDS_ENCODING_MEMCPY;
+	conn->char_convs[client2server_chardata]->flags = TDS_ENCODING_MEMCPY;
 	if (canonic_env_charset >= 0) {
-		tdsdump_log(TDS_DBG_FUNC, "preparing iconv for \"%s\" <-> \"%s\" conversion\n", charset, tds->env.charset);
-		fOK = tds_iconv_info_init(tds->char_convs[client2server_chardata], canonic_charset, canonic_env_charset);
+		tdsdump_log(TDS_DBG_FUNC, "preparing iconv for \"%s\" <-> \"%s\" conversion\n", charset, conn->env.charset);
+		fOK = tds_iconv_info_init(conn->char_convs[client2server_chardata], canonic_charset, canonic_env_charset);
 		if (!fOK)
-			return;
+			return TDS_FAIL;
 	} else {
-		tds->char_convs[client2server_chardata]->client_charset = canonic_charsets[canonic_charset];
-		tds->char_convs[client2server_chardata]->server_charset = canonic_charsets[canonic_charset];
+		conn->char_convs[client2server_chardata]->from.charset = canonic_charsets[canonic_charset];
+		conn->char_convs[client2server_chardata]->to.charset = canonic_charsets[canonic_charset];
 	}
 
-	/* 
-	 * ISO8859-1 <-> server meta data
-	 */
-	canonic = TDS_CHARSET_UCS_2LE;
-	if (!IS_TDS7_PLUS(tds)) {
-		canonic = TDS_CHARSET_ISO_8859_1;
-		if (canonic_env_charset >= 0)
-			canonic = canonic_env_charset;
-	}
-	tdsdump_log(TDS_DBG_FUNC, "preparing iconv for \"%s\" <-> \"%s\" conversion\n", "ISO-8859-1", canonic_charsets[canonic].name);
-	fOK = tds_iconv_info_init(tds->char_convs[iso2server_metadata], TDS_CHARSET_ISO_8859_1, canonic);
-
 	tdsdump_log(TDS_DBG_FUNC, "tds_iconv_open: done\n");
+	return TDS_SUCCESS;
 }
 
 /**
@@ -406,13 +397,11 @@ tds_iconv_open(TDSSOCKET * tds, const char *charset)
 static int
 tds_iconv_info_init(TDSICONV * char_conv, int client_canonical, int server_canonical)
 {
-	TDS_ENCODING *client = &char_conv->client_charset;
-	TDS_ENCODING *server = &char_conv->server_charset;
+	TDS_ENCODING *client = &char_conv->from.charset;
+	TDS_ENCODING *server = &char_conv->to.charset;
 
-	assert(char_conv->to_wire == (iconv_t) -1);
-	assert(char_conv->to_wire2 == (iconv_t) -1);
-	assert(char_conv->from_wire == (iconv_t) -1);
-	assert(char_conv->from_wire2 == (iconv_t) -1);
+	assert(char_conv->to.cd == (iconv_t) -1);
+	assert(char_conv->from.cd == (iconv_t) -1);
 
 	if (client_canonical < 0) {
 		tdsdump_log(TDS_DBG_FUNC, "tds_iconv_info_init: client charset name \"%d\" invalid\n", client_canonical);
@@ -429,25 +418,13 @@ tds_iconv_info_init(TDSICONV * char_conv, int client_canonical, int server_canon
 
 	/* special case, same charset, no conversion */
 	if (client_canonical == server_canonical) {
-		char_conv->to_wire = (iconv_t) -1;
-		char_conv->from_wire = (iconv_t) -1;
+		char_conv->to.cd = (iconv_t) -1;
+		char_conv->from.cd = (iconv_t) -1;
 		char_conv->flags = TDS_ENCODING_MEMCPY;
 		return 1;
 	}
 
 	char_conv->flags = 0;
-	if (!iconv_names[server_canonical]) {
-		switch (server_canonical) {
-		case POS_UCS2LE:
-			server_canonical = POS_UCS2BE;
-			char_conv->flags = TDS_ENCODING_SWAPBYTE;
-			break;
-		case POS_UCS2BE:
-			server_canonical = POS_UCS2LE;
-			char_conv->flags = TDS_ENCODING_SWAPBYTE;
-			break;
-		}
-	}
 
 	/* get iconv names */
 	if (!iconv_names[client_canonical]) {
@@ -464,38 +441,16 @@ tds_iconv_info_init(TDSICONV * char_conv, int client_canonical, int server_canon
 		}
 	}
 
-	char_conv->to_wire = tds_sys_iconv_open(iconv_names[server_canonical], iconv_names[client_canonical]);
-	if (char_conv->to_wire == (iconv_t) -1) {
+	char_conv->to.cd = tds_sys_iconv_open(iconv_names[server_canonical], iconv_names[client_canonical]);
+	if (char_conv->to.cd == (iconv_t) -1) {
 		tdsdump_log(TDS_DBG_FUNC, "tds_iconv_info_init: cannot convert \"%s\"->\"%s\"\n", client->name, server->name);
 	}
 
-	char_conv->from_wire = tds_sys_iconv_open(iconv_names[client_canonical], iconv_names[server_canonical]);
-	if (char_conv->from_wire == (iconv_t) -1) {
+	char_conv->from.cd = tds_sys_iconv_open(iconv_names[client_canonical], iconv_names[server_canonical]);
+	if (char_conv->from.cd == (iconv_t) -1) {
 		tdsdump_log(TDS_DBG_FUNC, "tds_iconv_info_init: cannot convert \"%s\"->\"%s\"\n", server->name, client->name);
 	}
 
-	/* try indirect conversions */
-	if (char_conv->to_wire == (iconv_t) -1 || char_conv->from_wire == (iconv_t) -1) {
-		tds_iconv_info_close(char_conv);
-
-		/* TODO reuse some conversion, client charset is usually constant in all connection (or ISO8859-1) */
-		char_conv->to_wire = tds_sys_iconv_open(iconv_names[POS_UTF8], iconv_names[client_canonical]);
-		char_conv->to_wire2 = tds_sys_iconv_open(iconv_names[server_canonical], iconv_names[POS_UTF8]);
-		char_conv->from_wire = tds_sys_iconv_open(iconv_names[POS_UTF8], iconv_names[server_canonical]);
-		char_conv->from_wire2 = tds_sys_iconv_open(iconv_names[client_canonical], iconv_names[POS_UTF8]);
-
-		if (char_conv->to_wire == (iconv_t) -1 || char_conv->to_wire2 == (iconv_t) -1
-		    || char_conv->from_wire == (iconv_t) -1 || char_conv->from_wire2 == (iconv_t) -1) {
-
-			tds_iconv_info_close(char_conv);
-			tdsdump_log(TDS_DBG_FUNC, "tds_iconv_info_init: cannot convert \"%s\"->\"%s\" indirectly\n",
-				    server->name, client->name);
-			return 0;
-		}
-
-		char_conv->flags |= TDS_ENCODING_INDIRECT;
-	}
-	
 	/* TODO, do some optimizations like UCS2 -> UTF8 min,max = 2,2 (UCS2) and 1,4 (UTF8) */
 
 	/* tdsdump_log(TDS_DBG_FUNC, "tds_iconv_info_init: converting \"%s\"->\"%s\"\n", client->name, server->name); */
@@ -518,38 +473,42 @@ _iconv_close(iconv_t * cd)
 static void
 tds_iconv_info_close(TDSICONV * char_conv)
 {
-	_iconv_close(&char_conv->to_wire);
-	_iconv_close(&char_conv->to_wire2);
-	_iconv_close(&char_conv->from_wire);
-	_iconv_close(&char_conv->from_wire2);
+	_iconv_close(&char_conv->to.cd);
+	_iconv_close(&char_conv->from.cd);
 }
 
 void
-tds_iconv_close(TDSSOCKET * tds)
+tds_iconv_close(TDSCONNECTION * conn)
 {
 	int i;
 
-	for (i = 0; i < tds->char_conv_count; ++i) {
-		tds_iconv_info_close(tds->char_convs[i]);
-	}
+	for (i = 0; i < conn->char_conv_count; ++i)
+		tds_iconv_info_close(conn->char_convs[i]);
 }
 
 #define CHUNK_ALLOC 4
 
 void
-tds_iconv_free(TDSSOCKET * tds)
+tds_iconv_free(TDSCONNECTION * conn)
 {
 	int i;
 
-	if (!tds->char_convs)
+	if (!conn->char_convs)
 		return;
-	tds_iconv_close(tds);
+	tds_iconv_close(conn);
 
-	free(tds->char_convs[0]);
-	for (i = initial_char_conv_count + 1; i < tds->char_conv_count; i += CHUNK_ALLOC)
-		free(tds->char_convs[i]);
-	TDS_ZERO_FREE(tds->char_convs);
-	tds->char_conv_count = 0;
+	free(conn->char_convs[0]);
+	for (i = initial_char_conv_count + 1; i < conn->char_conv_count; i += CHUNK_ALLOC)
+		free(conn->char_convs[i]);
+	TDS_ZERO_FREE(conn->char_convs);
+	conn->char_conv_count = 0;
+}
+
+static void
+tds_iconv_err(TDSSOCKET *tds, int err)
+{
+	if (tds)
+		tdserror(tds_get_ctx(tds), tds, err, 0);
 }
 
 /** 
@@ -577,21 +536,20 @@ tds_iconv_free(TDSSOCKET * tds)
  * 	On a write error we emit Msg 2402, Severity 16 (EX_USER):
  *		"Error converting client characters into server's character set. Some character(s) could not be converted."
  *  	  and return an error code.  Client libraries relying on this routine should reflect an error back to the application.  
- * 	
+ *
  * \todo Check for variable multibyte non-UTF-8 input character set.  
  * \todo Use more robust error message generation.  
  * \todo For reads, cope with \a outbuf encodings that don't have the equivalent of an ASCII '?'.  
  * \todo Support alternative to '?' for the replacement character.  
  */
 size_t
-tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
+tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	  const char **inbuf, size_t * inbytesleft, char **outbuf, size_t * outbytesleft)
 {
 	static const iconv_t invalid = (iconv_t) -1;
-	const TDS_ENCODING *input_charset = NULL;
-	const char *output_charset_name = NULL;
+	TDSICONVDIR *from = NULL;
+	TDSICONVDIR *to = NULL;
 
-	iconv_t cd = invalid, cd2 = invalid;
 	iconv_t error_cd = invalid;
 
 	char quest_mark[] = "?";	/* best to leave non-const; implementations vary */
@@ -599,25 +557,29 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	size_t lquest_mark;
 	size_t irreversible;
 	size_t one_character;
-	char *p;
 	int eilseq_raised = 0;
+	int conv_errno;
 	/* cast away const-ness */
 	TDS_ERRNO_MESSAGE_FLAGS *suppress = (TDS_ERRNO_MESSAGE_FLAGS*) &conv->suppress;
 
 	assert(inbuf && inbytesleft && outbuf && outbytesleft);
 
+	/* if empty there's nothing to return.
+	 * This fix case with some iconv implementation that does
+	 * not handle *inbuf == NULL and *inbytesleft == 0 as
+	 * empty strings
+	 */
+	if (*inbytesleft == 0)
+		return 0;
+
 	switch (io) {
 	case to_server:
-		cd = conv->to_wire;
-		cd2 = conv->to_wire2;
-		input_charset = &conv->client_charset;
-		output_charset_name = conv->server_charset.name;
+		from = &conv->from;
+		to = &conv->to;
 		break;
 	case to_client:
-		cd = conv->from_wire;
-		cd2 = conv->from_wire2;
-		input_charset = &conv->server_charset;
-		output_charset_name = conv->client_charset.name;
+		from = &conv->to;
+		to = &conv->from;
 		break;
 	default:
 		tdsdump_log(TDS_DBG_FUNC, "tds_iconv: unable to determine if %d means in or out.  \n", io);
@@ -626,7 +588,7 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	}
 
 	/* silly case, memcpy */
-	if (conv->flags & TDS_ENCODING_MEMCPY || cd == invalid) {
+	if (conv->flags & TDS_ENCODING_MEMCPY || to->cd == invalid) {
 		size_t len = *inbytesleft < *outbytesleft ? *inbytesleft : *outbytesleft;
 
 		memcpy(*outbuf, *inbuf, len);
@@ -641,84 +603,10 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	/*
 	 * Call iconv() as many times as necessary, until we reach the end of input or exhaust output.  
 	 */
-	errno = 0;
-	p = *outbuf;
 	for (;;) {
-		if (conv->flags & TDS_ENCODING_INDIRECT) {
-#if ENABLE_EXTRA_CHECKS
-			char tmp[8];
-#else
-			char tmp[128];
-#endif
-			char *pb = tmp;
-			size_t l = sizeof(tmp);
-			int temp_errno;
-			size_t temp_irreversible;
+		conv_errno = 0;
+		irreversible = tds_sys_iconv(to->cd, (ICONV_CONST char **) inbuf, inbytesleft, outbuf, outbytesleft);
 
-			temp_irreversible = tds_sys_iconv(cd, (ICONV_CONST char **) inbuf, inbytesleft, &pb, &l);
-			temp_errno = errno;
-
-			/* convert partial */
-			pb = tmp;
-			l = sizeof(tmp) - l;
-			for (;;) {
-				errno = 0;
-				irreversible = tds_sys_iconv(cd2, (ICONV_CONST char **) &pb, &l, outbuf, outbytesleft);
-				if (irreversible != (size_t) - 1) {
-					if (inbytesleft && *inbytesleft)
-						break;
-					goto end_loop;
-				}
-				/* EINVAL should be impossible, all characters came from previous iconv... */
-				if (errno == E2BIG || errno == EINVAL)
-					goto end_loop;
-
-				/*
-				 * error should be EILSEQ, not convertible sequence 
-				 * skip UTF-8 sequence 
-				 */
-				/* avoid infinite recursion */
-				eilseq_raised = 1;
-				if (*pb == '?')
-					goto end_loop;
-				*pb = (char) 0x80;
-				while(l && (*pb & 0xC0) == 0x80)
-					++pb, --l;
-				--pb;
-				++l;
-				*pb = '?';
-			}
-			if (temp_errno == E2BIG) {
-				errno = 0;
-				continue;
-			}
-			errno = temp_errno;
-			irreversible = temp_irreversible;
-			break;
-		} else if (io == to_client && conv->flags & TDS_ENCODING_SWAPBYTE && inbuf) {
-			/* swap bytes if necessary */
-#if ENABLE_EXTRA_CHECKS
-			char tmp[8];
-#else
-			char tmp[128];
-#endif
-			char *pib = tmp;
-			size_t il = *inbytesleft > sizeof(tmp) ? sizeof(tmp) : *inbytesleft;
-			size_t n;
-
-			for (n = 0; n < il; n += 2) {
-				tmp[n] = (*inbuf)[n + 1];
-				tmp[n + 1] = (*inbuf)[n];
-			}
-			irreversible = tds_sys_iconv(cd, (ICONV_CONST char **) &pib, &il, outbuf, outbytesleft);
-			il = pib - tmp;
-			*inbuf += il;
-			*inbytesleft -= il;
-			if (irreversible != (size_t) - 1 && *inbytesleft)
-				continue;
-		} else {
-			irreversible = tds_sys_iconv(cd, (ICONV_CONST char **) inbuf, inbytesleft, outbuf, outbytesleft);
-		}
 		/* iconv success, return */
 		if (irreversible != (size_t) - 1) {
 			/* here we detect end of conversion and try to reset shift state */
@@ -733,16 +621,19 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 			break;
 		}
 
-		if (errno == EILSEQ)
+		/* save errno, other function could change its value */
+		conv_errno = errno;
+
+		if (conv_errno == EILSEQ)
 			eilseq_raised = 1;
 
-		if (errno != EILSEQ || io != to_client || !inbuf)
+		if (conv_errno != EILSEQ || io != to_client || !inbuf)
 			break;
 		/* 
 		 * Invalid input sequence encountered reading from server. 
 		 * Skip one input sequence, adjusting pointers. 
 		 */
-		one_character = skip_one_input_sequence(cd, input_charset, inbuf, inbytesleft);
+		one_character = skip_one_input_sequence(to->cd, &from->charset, inbuf, inbytesleft);
 
 		if (!one_character)
 			break;
@@ -754,7 +645,7 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		 * do not convert singlebyte <-> singlebyte.
 		 */
 		if (error_cd == invalid) {
-			error_cd = tds_sys_iconv_open(output_charset_name, iconv_names[POS_UTF8]);
+			error_cd = tds_sys_iconv_open(to->charset.name, iconv_names[POS_UTF8]);
 			if (error_cd == invalid) {
 				break;	/* what to do? */
 			}
@@ -763,7 +654,6 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		lquest_mark = 1;
 		pquest_mark = quest_mark;
 
-		p = *outbuf;
 		irreversible = tds_sys_iconv(error_cd, &pquest_mark, &lquest_mark, outbuf, outbytesleft);
 
 		if (irreversible == (size_t) - 1)
@@ -772,46 +662,34 @@ tds_iconv(TDSSOCKET * tds, const TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		if (!*inbytesleft)
 			break;
 	}
-end_loop:
-	
-	/* swap bytes if necessary */
-	if (io == to_server && conv->flags & TDS_ENCODING_SWAPBYTE) {
-		assert((*outbuf - p) % 2 == 0);
-		for (; p < *outbuf; p += 2) {
-			char tmp = p[0];
-
-			p[0] = p[1];
-			p[1] = tmp;
-		}
-	}
 
 	if (eilseq_raised && !suppress->eilseq) {
 		/* invalid multibyte input sequence encountered */
 		if (io == to_client) {
 			if (irreversible == (size_t) - 1) {
-				tdserror(tds->tds_ctx, tds, TDSEICONV2BIG, 0);
+				tds_iconv_err(tds, TDSEICONV2BIG);
 			} else {
-				tdserror(tds->tds_ctx, tds, TDSEICONVI, 0);
-				errno = 0;
+				tds_iconv_err(tds, TDSEICONVI);
+				conv_errno = 0;
 			}
 		} else {
-			tdserror(tds->tds_ctx, tds, TDSEICONVO, 0);
+			tds_iconv_err(tds, TDSEICONVO);
 		}
 		suppress->eilseq = 1;
 	}
 
-	switch (errno) {
+	switch (conv_errno) {
 	case EINVAL:		/* incomplete multibyte sequence is encountered */
 		if (suppress->einval)
 			break;
 		/* in chunk conversion this can mean we end a chunk inside a character */
-		tdserror(tds->tds_ctx, tds, TDSEICONVAVAIL, 0);
+		tds_iconv_err(tds, TDSEICONVAVAIL);
 		suppress->einval = 1;
 		break;
 	case E2BIG:		/* output buffer has no more room */
 		if (suppress->e2big)
 			break;
-		tdserror(tds->tds_ctx, tds, TDSEICONVIU, 0);
+		tds_iconv_err(tds, TDSEICONVIU);
 		suppress->e2big = 1;
 		break;
 	default:
@@ -822,144 +700,58 @@ end_loop:
 		tds_sys_iconv_close(error_cd);
 	}
 
+	errno = conv_errno;
 	return irreversible;
 }
 
-/**
- * Read a data file, passing the data through iconv().
- * \return Count of bytes either not read, or read but not converted.  Returns zero on success.  
- */
-size_t
-tds_iconv_fread(iconv_t cd, FILE * stream, size_t field_len, size_t term_len, char *outbuf, size_t * outbytesleft)
-{
-#ifdef ENABLE_EXTRA_CHECKS
-	char buffer[16];
-#else
-	char buffer[16000];
-#endif
-	char *ib;
-	size_t isize = 0, nonreversible_conversions = 0;
-
-	/*
-	 * If cd isn't valid, it's just an indication that this column needs no conversion.  
-	 */
-	if (cd == (iconv_t) -1) {
-		assert(field_len <= *outbytesleft);
-		if (field_len > 0) {
-			if (1 != fread(outbuf, field_len, 1, stream)) {
-				return field_len + term_len;	/* unable to read */
-			}
-		}
-
-		/* prepare to read the terminator and return */
-		*outbytesleft -= field_len;	/* as iconv would have done */
-		isize = 0;			/* as iconv would have done */
-		field_len = 0;			/* as the loop would have done */
-
-		goto READ_TERMINATOR;
-	}
-	
-	/*
-	 * Read in chunks.  
-	 * 	field_len  is the total size to read
-	 * 	isize	   is the size of the current chunk (which might be the whole thing).
-	 * They are decremented as they are successfully processed.  
-	 * On success, we exit the loop with both equal to zero, indicating nothing we
-	 * were asked to read remains unread.
-	 */
-	isize = (sizeof(buffer) < field_len) ? sizeof(buffer) : field_len;
-
-	for (ib = buffer; isize && (isize = fread(ib, 1, isize, stream)) > 0;) {
-
-		tdsdump_log(TDS_DBG_FUNC, "tds_iconv_fread: read %u of %u bytes; outbuf has %u left.\n", (unsigned int) isize,
-			    (unsigned int) field_len, (unsigned int) *outbytesleft);
-		field_len -= isize;
-
-		isize += ib - buffer;
-		ib = buffer;
-		nonreversible_conversions += tds_sys_iconv(cd, (ICONV_CONST char **) &ib, &isize, &outbuf, outbytesleft);
-
-		if (isize != 0) {
-			memmove(buffer, ib, isize);
-			switch (errno) {
-			case EINVAL:	/* incomplete multibyte sequence encountered in input */
-				break;
-			case E2BIG:	/* insufficient room in output buffer */
-			case EILSEQ:	/* invalid multibyte sequence encountered in input */
-			default:
-				/* FIXME: emit message */
-				tdsdump_log(TDS_DBG_FUNC, "tds_iconv_fread: error %d: %s.\n", errno, strerror(errno));
-				break;
-			}
-		}
-		ib = buffer + isize;
-		isize = sizeof(buffer) - isize;
-		if (isize > field_len)
-			isize = field_len;
-	}
-	
-	READ_TERMINATOR:
-
-	if (term_len > 0 && !feof(stream)) {
-		isize += term_len;
-		if (term_len && 1 == fread(buffer, term_len, 1, stream)) {
-			isize -= term_len;
-		} else {
-			tdsdump_log(TDS_DBG_FUNC, "tds_iconv_fread: cannot read %u-byte terminator\n", (unsigned int) term_len);
-		}
-	}
-
-	return field_len + isize;
-}
-
 /**
  * Get a iconv info structure, allocate and initialize if needed
  */
 static TDSICONV *
-tds_iconv_get_info(TDSSOCKET * tds, int canonic_client, int canonic_server)
+tds_iconv_get_info(TDSCONNECTION * conn, int canonic_client, int canonic_server)
 {
 	TDSICONV *info;
 	int i;
 
 	/* search a charset from already allocated charsets */
-	for (i = tds->char_conv_count; --i >= initial_char_conv_count;)
-		if (canonic_client == tds->char_convs[i]->client_charset.canonic
-		    && canonic_server == tds->char_convs[i]->server_charset.canonic)
-			return tds->char_convs[i];
+	for (i = conn->char_conv_count; --i >= initial_char_conv_count;)
+		if (canonic_client == conn->char_convs[i]->from.charset.canonic
+		    && canonic_server == conn->char_convs[i]->to.charset.canonic)
+			return conn->char_convs[i];
 
 	/* allocate a new iconv structure */
-	if (tds->char_conv_count % CHUNK_ALLOC == ((initial_char_conv_count + 1) % CHUNK_ALLOC)) {
+	if (conn->char_conv_count % CHUNK_ALLOC == ((initial_char_conv_count + 1) % CHUNK_ALLOC)) {
 		TDSICONV **p;
 		TDSICONV *infos;
 
-		infos = (TDSICONV *) malloc(sizeof(TDSICONV) * CHUNK_ALLOC);
+		infos = tds_new(TDSICONV, CHUNK_ALLOC);
 		if (!infos)
 			return NULL;
-		p = (TDSICONV **) realloc(tds->char_convs, sizeof(TDSICONV *) * (tds->char_conv_count + CHUNK_ALLOC));
+		p = (TDSICONV **) realloc(conn->char_convs, sizeof(TDSICONV *) * (conn->char_conv_count + CHUNK_ALLOC));
 		if (!p) {
 			free(infos);
 			return NULL;
 		}
-		tds->char_convs = p;
+		conn->char_convs = p;
 		memset(infos, 0, sizeof(TDSICONV) * CHUNK_ALLOC);
 		for (i = 0; i < CHUNK_ALLOC; ++i) {
-			tds->char_convs[i + tds->char_conv_count] = &infos[i];
+			conn->char_convs[i + conn->char_conv_count] = &infos[i];
 			tds_iconv_reset(&infos[i]);
 		}
 	}
-	info = tds->char_convs[tds->char_conv_count++];
+	info = conn->char_convs[conn->char_conv_count++];
 
 	/* init */
 	if (tds_iconv_info_init(info, canonic_client, canonic_server))
 		return info;
 
 	tds_iconv_info_close(info);
-	--tds->char_conv_count;
+	--conn->char_conv_count;
 	return NULL;
 }
 
 TDSICONV *
-tds_iconv_get(TDSSOCKET * tds, const char *client_charset, const char *server_charset)
+tds_iconv_get(TDSCONNECTION * conn, const char *client_charset, const char *server_charset)
 {
 	int canonic_client_charset_num = tds_canonical_charset(client_charset);
 	int canonic_server_charset_num = tds_canonical_charset(server_charset);
@@ -973,41 +765,31 @@ tds_iconv_get(TDSSOCKET * tds, const char *client_charset, const char *server_ch
 		return NULL;
 	}
 
-	return tds_iconv_get_info(tds, canonic_client_charset_num, canonic_server_charset_num);
+	return tds_iconv_get_info(conn, canonic_client_charset_num, canonic_server_charset_num);
 }
 
 /* change singlebyte conversions according to server */
 static void
-tds_srv_charset_changed_num(TDSSOCKET * tds, int canonic_charset_num)
+tds_srv_charset_changed_num(TDSCONNECTION * conn, int canonic_charset_num)
 {
-	TDSICONV *char_conv = tds->char_convs[client2server_chardata];
+	TDSICONV *char_conv = conn->char_convs[client2server_chardata];
 
-	if (IS_TDS7_PLUS(tds) && canonic_charset_num == TDS_CHARSET_ISO_8859_1)
+	if (IS_TDS7_PLUS(conn) && canonic_charset_num == TDS_CHARSET_ISO_8859_1)
 		canonic_charset_num = TDS_CHARSET_CP1252;
 
 	tdsdump_log(TDS_DBG_FUNC, "setting server single-byte charset to \"%s\"\n", canonic_charsets[canonic_charset_num].name);
 
-	if (canonic_charset_num == char_conv->server_charset.canonic)
+	if (canonic_charset_num == char_conv->to.charset.canonic)
 		return;
 
 	/* find and set conversion */
-	char_conv = tds_iconv_get_info(tds, tds->char_convs[client2ucs2]->client_charset.canonic, canonic_charset_num);
+	char_conv = tds_iconv_get_info(conn, conn->char_convs[client2ucs2]->from.charset.canonic, canonic_charset_num);
 	if (char_conv)
-		tds->char_convs[client2server_chardata] = char_conv;
-
-	/* if sybase change also server conversions */
-	if (IS_TDS7_PLUS(tds))
-		return;
-
-	char_conv = tds->char_convs[iso2server_metadata];
-
-	tds_iconv_info_close(char_conv);
-
-	tds_iconv_info_init(char_conv, TDS_CHARSET_ISO_8859_1, canonic_charset_num);
+		conn->char_convs[client2server_chardata] = char_conv;
 }
 
 void
-tds_srv_charset_changed(TDSSOCKET * tds, const char *charset)
+tds_srv_charset_changed(TDSCONNECTION * conn, const char *charset)
 {
 	int n = tds_canonical_charset(charset);
 
@@ -1017,14 +799,14 @@ tds_srv_charset_changed(TDSSOCKET * tds, const char *charset)
 		return;
 	}
 
-	tds_srv_charset_changed_num(tds, n);
+	tds_srv_charset_changed_num(conn, n);
 }
 
 /* change singlebyte conversions according to server */
 void
-tds7_srv_charset_changed(TDSSOCKET * tds, int sql_collate, int lcid)
+tds7_srv_charset_changed(TDSCONNECTION * conn, int sql_collate, int lcid)
 {
-	tds_srv_charset_changed_num(tds, collate2charset(sql_collate, lcid));
+	tds_srv_charset_changed_num(conn, collate2charset(sql_collate, lcid));
 }
 
 /**
@@ -1036,7 +818,7 @@ tds7_srv_charset_changed(TDSSOCKET * tds, int sql_collate, int lcid)
 static size_t
 skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **input, size_t * input_size)
 {
-	int charsize = CHARSIZE(charset);
+	unsigned charsize = CHARSIZE(charset);
 	char ib[16];
 	char ob[16];
 	ICONV_CONST char *pib;
@@ -1176,32 +958,6 @@ tds_canonical_charset_name(const char *charset_name)
 	return charset_name;	/* hope for the best */
 }
 
-/**
- * Determine the name Sybase uses for a character set, given a canonical iconv name.  
- * \returns Sybase name, or NULL if lookup failed.
- * \remarks Returned name can be sent to Sybase a server.
- */
-const char *
-tds_sybase_charset_name(const char *charset_name)
-{
-	int res, i;
-
-	/* search in sybase */
-	res = lookup_canonic(iconv_aliases, charset_name);
-	if (res < 0)
-		return NULL;
-
-	/* special case, ignore ascii_8, take iso_1 instead, note index start from 1 */
-	assert(strcmp(sybase_aliases[0].alias, "ascii_8") == 0);
-
-	for (i = 1; sybase_aliases[i].alias; ++i) {
-		if (sybase_aliases[i].canonic == res)
-			return sybase_aliases[i].alias;
-	}
-
-	return NULL;
-}
-
 static int
 collate2charset(int sql_collate, int lcid)
 {
@@ -1267,8 +1023,10 @@ collate2charset(int sql_collate, int lcid)
 	case 0x41b:
 	case 0x41c:
 	case 0x424:
+	case 0x442:
 		/* case 0x81a: seem wrong in XP table TODO check */
 	case 0x104e:		/* ?? */
+	case 0x141a:
 		cp = TDS_CHARSET_CP1250;
 		break;
 	case 0x402:
@@ -1299,6 +1057,7 @@ collate2charset(int sql_collate, int lcid)
 	case 0x180c:
 	case 0x1c09:
 	case 0x1c0a:
+	case 0x201a:
 	case 0x2009:
 	case 0x200a:
 	case 0x2409:
@@ -1316,6 +1075,15 @@ collate2charset(int sql_collate, int lcid)
 	case 0x400a:
 	case 0x403:
 	case 0x406:
+	case 0x417:
+	case 0x42e:
+	case 0x43b:
+	case 0x452:
+	case 0x462:
+	case 0x47a:
+	case 0x47c:
+	case 0x47e:
+	case 0x483:
 	case 0x407:		/* 0x10407 */
 	case 0x409:
 	case 0x40a:
@@ -1337,6 +1105,8 @@ collate2charset(int sql_collate, int lcid)
 	case 0x440a:
 	case 0x441:
 	case 0x456:
+	case 0x46d:
+	case 0x485:
 	case 0x480a:
 	case 0x4c0a:
 	case 0x500a:
@@ -1349,7 +1119,9 @@ collate2charset(int sql_collate, int lcid)
 	case 0x814:
 	case 0x816:
 	case 0x81d:
+	case 0x83b:
 	case 0x83e:
+	case 0x85f:
 	case 0xc07:
 	case 0xc09:
 	case 0xc0a:
@@ -1381,8 +1153,10 @@ collate2charset(int sql_collate, int lcid)
 	case 0x3c01:
 	case 0x4001:
 	case 0x401:
+	case 0x480:
 	case 0x420:
 	case 0x429:
+	case 0x48c:
 	case 0x801:
 	case 0xc01:
 		cp = TDS_CHARSET_CP1256;
@@ -1425,17 +1199,17 @@ collate2charset(int sql_collate, int lcid)
  * Get iconv information from a LCID (to support different column encoding under MSSQL2K)
  */
 TDSICONV *
-tds_iconv_from_collate(TDSSOCKET * tds, TDS_UCHAR collate[5])
+tds_iconv_from_collate(TDSCONNECTION * conn, TDS_UCHAR collate[5])
 {
 	const int sql_collate = collate[4];
 	const int lcid = collate[1] * 256 + collate[0];
 	int canonic_charset = collate2charset(sql_collate, lcid);
 
 	/* same as client (usually this is true, so this improve performance) ? */
-	if (tds->char_convs[client2server_chardata]->server_charset.canonic == canonic_charset)
-		return tds->char_convs[client2server_chardata];
+	if (conn->char_convs[client2server_chardata]->to.charset.canonic == canonic_charset)
+		return conn->char_convs[client2server_chardata];
 
-	return tds_iconv_get_info(tds, tds->char_convs[client2ucs2]->client_charset.canonic, canonic_charset);
+	return tds_iconv_get_info(conn, conn->char_convs[client2ucs2]->from.charset.canonic, canonic_charset);
 }
 
 /** @} */
