@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_LOCALE_H
 #include <locale.h>
@@ -38,15 +36,9 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include "tds.h"
-#include "tds_configs.h"
+#include <freetds/tds.h>
+#include <freetds/configs.h>
 #include "replacements.h"
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: locale.c,v 1.30 2010/07/21 20:12:18 freddy77 Exp $");
-
 
 static void tds_parse_locale(const char *option, const char *value, void *param);
 
@@ -83,7 +75,7 @@ tds_get_locale(void)
 			const char *strip = "@._";
 
 			/* do not change environment !!! */
-			tds_strlcpy(buf, s, sizeof(buf));
+			strlcpy(buf, s, sizeof(buf));
 
 			/* search full name */
 			rewind(in);
