@@ -474,6 +474,7 @@ tds_set_current_results(TDSSOCKET *tds, TDSRESULTINFO *info)
 		tds->current_results->attached_to = NULL;
 	if (info)
 		info->attached_to = tds;
+	tds->in_row = (info != NULL);
 	tds->current_results = info;
 }
 
@@ -485,6 +486,7 @@ tds_detach_results(TDSRESULTINFO *info)
 {
 	if (info && info->attached_to) {
 		info->attached_to->current_results = NULL;
+		info->attached_to->in_row = false;
 		info->attached_to = NULL;
 	}
 }
@@ -657,6 +659,7 @@ tds_free_all_results(TDSSOCKET * tds)
 	tds->param_info = NULL;
 	tds_free_compute_results(tds);
 	tds->has_status = 0;
+	tds->in_row = false;
 	tds->ret_status = 0;
 	if (tds->cur_dyn)
 		tds_detach_results(tds->cur_dyn->res_info);
@@ -993,6 +996,7 @@ tds_alloc_login(int use_environment)
 	tds_dstr_init(&login->crlfile);
 	tds_dstr_init(&login->db_filename);
 	tds_dstr_init(&login->openssl_ciphers);
+	tds_dstr_init(&login->routing_address);
 
 	if (use_environment) {
 		const char *s;
@@ -1047,6 +1051,7 @@ tds_free_login(TDSLOGIN * login)
 	tds_dstr_free(&login->crlfile);
 	tds_dstr_free(&login->db_filename);
 	tds_dstr_free(&login->openssl_ciphers);
+	tds_dstr_free(&login->routing_address);
 	free(login);
 }
 
