@@ -81,6 +81,12 @@
 #include <sys/eventfd.h>
 #endif /* HAVE_SYS_EVENTFD_H */
 
+#ifdef _WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#include <mstcpip.h>
+#endif
+
 #include <freetds/tds.h>
 #include <freetds/string.h>
 #include <freetds/tls.h>
@@ -289,9 +295,21 @@ tds_connect_socket(TDSSOCKET *tds, struct addrinfo *addr, unsigned int port, int
 TDSERRNO
 tds_open_socket(TDSSOCKET *tds, struct addrinfo *addr, unsigned int port, int timeout, int *p_oserr)
 {
+	enum {
+		TDS_SOCKET_KEEPALIVE_IDLE = 40,
+		TDS_SOCKET_KEEPALIVE_INTERVAL = 2
+	};
 	TDSCONNECTION *conn = tds->conn;
 	int len;
 	TDSERRNO tds_error;
+#if defined(_WIN32)
+	struct tcp_keepalive keepalive = {
+		TRUE,
+		TDS_SOCKET_KEEPALIVE_IDLE * 1000,
+		TDS_SOCKET_KEEPALIVE_INTERVAL * 1000
+	};
+	DWORD written;
+#endif
 
 	*p_oserr = 0;
 
@@ -309,10 +327,17 @@ tds_open_socket(TDSSOCKET *tds, struct addrinfo *addr, unsigned int port, int ti
 	setsockopt(conn->s, SOL_SOCKET, SO_KEEPALIVE, (const void *) &len, sizeof(len));
 #endif
 
-#if defined(TCP_KEEPIDLE) && defined(TCP_KEEPINTVL)
-	len = 40;
+#if defined(_WIN32)
+	if (WSAIoctl(conn->s, SIO_KEEPALIVE_VALS, &keepalive, sizeof(keepalive),
+		     NULL, 0, &written, NULL, NULL) != 0) {
+		char *errstr = sock_strerror(*p_oserr = sock_errno);
+		tdsdump_log(TDS_DBG_ERROR, "error setting keepalive: %s\n", errstr);
+		sock_strerror_free(errstr);
+	}
+#elif defined(TCP_KEEPIDLE) && defined(TCP_KEEPINTVL)
+	len = TDS_SOCKET_KEEPALIVE_IDLE;
 	setsockopt(conn->s, SOL_TCP, TCP_KEEPIDLE, (const void *) &len, sizeof(len));
-	len = 2;
+	len = TDS_SOCKET_KEEPALIVE_INTERVAL;
 	setsockopt(conn->s, SOL_TCP, TCP_KEEPINTVL, (const void *) &len, sizeof(len));
 #endif
 
