@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 2004, 2005 Frediano Ziglio
+ * Copyright (C) 2004-2011  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,9 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #undef NDEBUG
 
@@ -35,34 +33,37 @@
 
 #include <assert.h>
 
-#include "tds.h"
-#include "tdsconvert.h"
-#include "tdsstring.h"
-#include "tds_checks.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: tds_checks.c,v 1.29 2009/11/26 09:07:28 freddy77 Exp $");
+#include <freetds/tds.h>
+#include <freetds/convert.h>
+#include <freetds/string.h>
+#include <freetds/checks.h>
 
 #if ENABLE_EXTRA_CHECKS
+static void
+tds_check_packet_extra(const TDSPACKET * packet)
+{
+	assert(packet);
+	for (; packet; packet = packet->next) {
+		assert(packet->len <= packet->capacity);
+		assert(packet->sid >= -1);
+	}
+}
 
 void
 tds_check_tds_extra(const TDSSOCKET * tds)
 {
 	const int invalid_state = 0;
-	int found, i;
-	int result_found = 0;
+	int i;
 	TDSDYNAMIC *cur_dyn = NULL;
 	TDSCURSOR *cur_cursor = NULL;
 
 	assert(tds);
 
-	/* teset state and connection */
+	/* test state and connection */
 	switch (tds->state) {
 	case TDS_DEAD:
-	case TDS_QUERYING:
+	case TDS_WRITING:
+	case TDS_SENDING:
 	case TDS_PENDING:
 	case TDS_IDLE:
 	case TDS_READING:
@@ -71,81 +72,72 @@ tds_check_tds_extra(const TDSSOCKET * tds)
 		assert(invalid_state);
 	}
 
-	assert(tds->state == TDS_DEAD || !TDS_IS_SOCKET_INVALID(tds->s));
-	assert(tds->state != TDS_DEAD || TDS_IS_SOCKET_INVALID(tds->s));
+	assert(tds->conn);
+
+#if ENABLE_ODBC_MARS
+	if (tds->state != TDS_DEAD)
+		assert(!TDS_IS_SOCKET_INVALID(tds_get_s(tds)));
+#else
+	assert(tds->state == TDS_DEAD || !TDS_IS_SOCKET_INVALID(tds_get_s(tds)));
+	assert(tds->state != TDS_DEAD || TDS_IS_SOCKET_INVALID(tds_get_s(tds)));
+#endif
 
 	/* test env */
-	tds_check_env_extra(&tds->env);
+	tds_check_env_extra(&tds->conn->env);
 
 	/* test buffers and positions */
-	assert(tds->in_pos <= tds->in_len && tds->in_len <= tds->in_buf_max);
+	tds_check_packet_extra(tds->send_packet);
+	tds_check_packet_extra(tds->recv_packet);
+
+#if ENABLE_ODBC_MARS
+	if (tds->conn->send_packets)
+		assert(tds->conn->send_pos <= tds->conn->send_packets->len);
+	if (tds->conn->recv_packet)
+		assert(tds->conn->recv_pos <= tds->conn->recv_packet->len);
+#endif
+
+	assert(tds->in_pos <= tds->in_len);
+	assert(tds->in_len <= tds->recv_packet->capacity);
 	/* TODO remove blocksize from env and use out_len ?? */
 /*	assert(tds->out_pos <= tds->out_len); */
 /* 	assert(tds->out_len == 0 || tds->out_buf != NULL); */
-	assert(tds->out_pos <= tds->env.block_size);
-	assert(tds->env.block_size == 0 || tds->out_buf != NULL);
-	assert(tds->in_buf_max == 0 || tds->in_buf != NULL);
+	assert(tds->send_packet->capacity >= tds->out_buf_max + TDS_ADDITIONAL_SPACE);
+	assert(tds->out_buf >= tds->send_packet->buf);
+	assert(tds->out_buf + tds->out_buf_max + TDS_ADDITIONAL_SPACE <=
+		tds->send_packet->buf + tds->send_packet->capacity);
+	assert(tds->out_pos <= tds->out_buf_max + TDS_ADDITIONAL_SPACE);
+
+	assert(tds->in_buf == tds->recv_packet->buf || tds->in_buf == tds->recv_packet->buf + 16);
+	assert(tds->recv_packet->capacity > 0);
 
 	/* test res_info */
-	if (tds->res_info) {
+	if (tds->res_info)
 		tds_check_resultinfo_extra(tds->res_info);
-		if (tds->current_results == tds->res_info)
-			result_found = 1;
-	}
 
 	/* test num_comp_info, comp_info */
 	assert(tds->num_comp_info >= 0);
 	for (i = 0; i < tds->num_comp_info; ++i) {
 		assert(tds->comp_info);
 		tds_check_resultinfo_extra(tds->comp_info[i]);
-		if (tds->current_results == tds->comp_info[i])
-			result_found = 1;
 	}
 
 	/* param_info */
-	if (tds->param_info) {
+	if (tds->param_info)
 		tds_check_resultinfo_extra(tds->param_info);
-		if (tds->current_results == tds->param_info)
-			result_found = 1;
-	}
 
 	/* test cursors */
-	found = 0;
-	for (cur_cursor = tds->cursors; cur_cursor != NULL; cur_cursor = cur_cursor->next) {
+	for (cur_cursor = tds->conn->cursors; cur_cursor != NULL; cur_cursor = cur_cursor->next)
 		tds_check_cursor_extra(cur_cursor);
-		if (tds->current_results == cur_cursor->res_info)
-			result_found = 1;
-		if (cur_cursor == tds->cur_cursor)
-			found = 1;
-	}
-	assert(found || tds->cur_cursor == NULL);
 
-	/* test num_dyms, cur_dyn, dyns */
-	found = 0;
-	for (cur_dyn = tds->dyns; cur_dyn != NULL; cur_dyn = cur_dyn->next) {
-		if (cur_dyn == tds->cur_dyn)
-			found = 1;
+	/* test dynamics */
+	for (cur_dyn = tds->conn->dyns; cur_dyn != NULL; cur_dyn = cur_dyn->next)
 		tds_check_dynamic_extra(cur_dyn);
-		if (tds->current_results == cur_dyn->res_info)
-			result_found = 1;
-	}
-	assert(found || tds->cur_dyn == NULL);
 
 	/* test tds_ctx */
-	tds_check_context_extra(tds->tds_ctx);
+	tds_check_context_extra(tds_get_ctx(tds));
 
 	/* TODO test char_conv_count, char_convs */
 
-	/* current_results should be one of res_info, comp_info, param_info or dynamic */
-	/*
-	 * TODO this test was here to check that current_results was an alias
-	 * but with cursor and reference counting current_results can point
-	 * to a cursor result available on upper layer
-	 * Perhaps we should free results on deallocate and enable
-	 * assert again?
-	 */
-	/* assert(result_found || tds->current_results == NULL); */
-
 	/* we can't have compute and no results */
 	assert(tds->num_comp_info == 0 || tds->res_info != NULL);
 
@@ -172,65 +164,64 @@ void
 tds_check_column_extra(const TDSCOLUMN * column)
 {
 	int size;
-	TDSSOCKET tds;
+	TDSCONNECTION conn;
 	int varint_ok;
+	int column_varint_size;
 
 	assert(column);
+	column_varint_size = column->column_varint_size;
 
 	/* 8 is for varchar(max) or similar */
-	assert(column->column_varint_size == 8 || (column->column_varint_size <= 5 && column->column_varint_size != 3));
+	assert(column_varint_size == 8 || (column_varint_size <= 5 && column_varint_size != 3));
 
 	assert(column->column_scale <= column->column_prec);
-	assert(column->column_prec <= 77);
+	assert(column->column_prec <= MAXPRECISION);
 
 	/* I don't like this that much... freddy77 */
 	if (column->column_type == 0)
 		return;
+	assert(column->funcs);
 	assert(column->column_type > 0);
 
-	assert(strlen(column->table_name) < sizeof(column->table_name));
-	assert(strlen(column->column_name) < sizeof(column->column_name));
+	/* specific checks, if true fully checked */
+	if (column->funcs->check(column))
+		return;
 
 	/* check type and server type same or SQLNCHAR -> SQLCHAR */
-#define SPECIAL(type, server_type, varint) \
-	if (column->column_type == type && column->on_server.column_type == server_type && column->column_varint_size == varint) {} else
+#define SPECIAL(ttype, server_type, varint) \
+	if (column->column_type == ttype && column->on_server.column_type == server_type && column_varint_size == varint) {} else
 	SPECIAL(SYBTEXT, XSYBVARCHAR, 8)
 	SPECIAL(SYBTEXT, XSYBNVARCHAR, 8)
 	SPECIAL(SYBIMAGE, XSYBVARBINARY, 8)
 	assert(tds_get_cardinal_type(column->on_server.column_type, column->column_usertype) == column->column_type
-		|| (tds_get_null_type(column->column_type) == column->on_server.column_type
-		&& column->column_varint_size == 1 && is_fixed_type(column->column_type)));
+		|| (tds_get_conversion_type(column->on_server.column_type, column->column_size) == column->column_type
+		&& column_varint_size == 1 && is_fixed_type(column->column_type)));
 
 	varint_ok = 0;
-	if (column->column_varint_size == 8) {
-		assert(column->on_server.column_type == XSYBVARCHAR || column->on_server.column_type == XSYBVARBINARY || column->on_server.column_type == XSYBNVARCHAR || column->on_server.column_type == SYBMSXML);
-		varint_ok = 1;
+	if (column_varint_size == 8) {
+		if (column->on_server.column_type == XSYBVARCHAR
+		    || column->on_server.column_type == XSYBVARBINARY
+		    || column->on_server.column_type == XSYBNVARCHAR)
+			varint_ok = 1;
 	} else if (is_blob_type(column->column_type)) {
-		assert(column->column_varint_size >= 4);
+		assert(column_varint_size >= 4);
 	} else if (column->column_type == SYBVARIANT) {
-		assert(column->column_varint_size == 4);
+		assert(column_varint_size == 4);
 	}
-	tds.tds_version = 0x500;
-	varint_ok = varint_ok || tds_get_varint_size(&tds, column->on_server.column_type) == column->column_varint_size;
-	tds.tds_version = 0x700;
-	varint_ok = varint_ok || tds_get_varint_size(&tds, column->on_server.column_type) == column->column_varint_size;
+	conn.tds_version = 0x500;
+	varint_ok = varint_ok || tds_get_varint_size(&conn, column->on_server.column_type) == column_varint_size;
+	conn.tds_version = 0x700;
+	varint_ok = varint_ok || tds_get_varint_size(&conn, column->on_server.column_type) == column_varint_size;
 	assert(varint_ok);
 
-	/* check current size <= size */
-	if (is_numeric_type(column->column_type)) {
-		/* I don't like that much this difference between numeric and not numeric - freddy77 */
-		/* TODO what should be the size ?? */
-		assert(column->column_prec >= 1 && column->column_prec <= 77);
-		assert(column->column_scale <= column->column_prec);
-/*		assert(column->column_cur_size == tds_numeric_bytes_per_prec[column->column_prec] + 2 || column->column_cur_size == -1); */
-	} else {
-		assert(column->column_cur_size <= column->column_size);
-	}
+	assert(!is_numeric_type(column->column_type));
+	assert(column->column_cur_size <= column->column_size);
 
 	/* check size of fixed type correct */
 	size = tds_get_size_by_type(column->column_type);
-	assert(size != 0 || column->column_type == SYBVOID);
-	if (size >= 0 && column->column_type != SYBBITN) {
+	/* these peculiar types are variable but have only a possible size */
+	if ((size > 0 && (column->column_type != SYBBITN && column->column_type != SYBDATEN && column->column_type != SYBTIMEN))
+	    || column->column_type == SYBVOID) {
 		/* check macro */
 		assert(is_fixed_type(column->column_type));
 		/* check current size */
@@ -238,18 +229,19 @@ tds_check_column_extra(const TDSCOLUMN * column)
 		/* check cases where server need nullable types */
 		if (column->column_type != column->on_server.column_type && (column->column_type != SYBINT8 || column->on_server.column_type != SYB5INT8)) {
 			assert(!is_fixed_type(column->on_server.column_type));
-			assert(column->column_varint_size == 1);
+			assert(column_varint_size == 1);
 			assert(column->column_size == column->column_cur_size || column->column_cur_size == -1);
 		} else {
-			assert(column->column_varint_size == 0 || (column->column_type == SYBUNIQUE && column->column_varint_size == 1));
-			assert(column->column_size == column->column_cur_size || 
-				(column->column_type == SYBUNIQUE && column->column_cur_size == -1));
+			assert(column_varint_size == 0
+				|| (column->column_type == SYBUNIQUE && column_varint_size == 1));
+			assert(column->column_size == column->column_cur_size
+				|| (column->column_type == SYBUNIQUE && column->column_cur_size == -1));
 		}
 		assert(column->column_size == column->on_server.column_size);
 	} else {
 		assert(!is_fixed_type(column->column_type));
 		assert(is_char_type(column->column_type) || (column->on_server.column_size == column->column_size || column->on_server.column_size == 0));
-		assert(column->column_varint_size != 0);
+		assert(column_varint_size != 0);
 	}
 
 	/* check size of nullable types (ie intN) it's supported */
@@ -291,6 +283,12 @@ tds_check_resultinfo_extra(const TDSRESULTINFO * res_info)
 void
 tds_check_cursor_extra(const TDSCURSOR * cursor)
 {
+	assert(cursor);
+
+	assert(cursor->ref_count > 0);
+
+	if (cursor->res_info)
+		tds_check_resultinfo_extra(cursor->res_info);
 }
 
 void
@@ -298,6 +296,8 @@ tds_check_dynamic_extra(const TDSDYNAMIC * dyn)
 {
 	assert(dyn);
 
+	assert(dyn->ref_count > 0);
+
 	if (dyn->res_info)
 		tds_check_resultinfo_extra(dyn->res_info);
 	if (dyn->params)
