@@ -71,10 +71,34 @@
 #define SSL_PUSH_ARGS gnutls_transport_ptr_t ptr, const void *data, size_t len
 #define SSL_PTR ptr
 #else
+
+/* some compatibility layer */
+#if OPENSSL_VERSION_NUMBER < 0x1010000FL
+static inline void
+BIO_set_init(BIO *b, int init)
+{
+	b->init = init;
+}
+
+static inline void
+BIO_set_data(BIO *b, void *ptr)
+{
+	b->ptr = ptr;
+}
+
+static inline void *
+BIO_get_data(const BIO *b)
+{
+	return b->ptr;
+}
+#define TLS_client_method SSLv23_client_method
+#define TLS_ST_OK SSL_ST_OK
+#endif
+
 #define SSL_RET int
 #define SSL_PULL_ARGS BIO *bio, char *data, int len
 #define SSL_PUSH_ARGS BIO *bio, const char *data, int len
-#define SSL_PTR bio->ptr
+#define SSL_PTR BIO_get_data(bio)
 #endif
 
 static SSL_RET
@@ -548,7 +572,7 @@ tds_ssl_deinit(TDSCONNECTION *conn)
 static long
 tds_ssl_ctrl_login(BIO *b, int cmd, long num, void *ptr)
 {
-	TDSSOCKET *tds = (TDSSOCKET *) b->ptr;
+	TDSSOCKET *tds = (TDSSOCKET *) BIO_get_data(b);
 
 	switch (cmd) {
 	case BIO_CTRL_FLUSH:
@@ -566,7 +590,8 @@ tds_ssl_free(BIO *a)
 	return 1;
 }
 
-static BIO_METHOD tds_method_login =
+#if OPENSSL_VERSION_NUMBER < 0x1010000FL
+static BIO_METHOD tds_method_login[1] = {
 {
 	BIO_TYPE_MEM,
 	"tds",
@@ -578,9 +603,9 @@ static BIO_METHOD tds_method_login =
 	NULL,
 	tds_ssl_free,
 	NULL,
-};
+}};
 
-static BIO_METHOD tds_method =
+static BIO_METHOD tds_method[1] = {
 {
 	BIO_TYPE_MEM,
 	"tds",
@@ -592,7 +617,42 @@ static BIO_METHOD tds_method =
 	NULL,
 	tds_ssl_free,
 	NULL,
-};
+}};
+
+static inline void
+tds_init_ssl_methods(void)
+{
+}
+#else
+static BIO_METHOD *tds_method_login;
+static BIO_METHOD *tds_method;
+
+static void
+tds_init_ssl_methods(void)
+{
+	BIO_METHOD *meth;
+
+	tds_method_login = meth = BIO_meth_new(BIO_TYPE_MEM, "tds");
+	BIO_meth_set_write(meth, tds_push_func_login);
+	BIO_meth_set_read(meth, tds_pull_func_login);
+	BIO_meth_set_ctrl(meth, tds_ssl_ctrl_login);
+	BIO_meth_set_destroy(meth, tds_ssl_free);
+
+	tds_method = meth = BIO_meth_new(BIO_TYPE_MEM, "tds");
+	BIO_meth_set_write(meth, tds_push_func);
+	BIO_meth_set_read(meth, tds_pull_func);
+	BIO_meth_set_destroy(meth, tds_ssl_free);
+}
+
+#  ifdef TDS_ATTRIBUTE_DESTRUCTOR
+static void __attribute__((destructor))
+tds_deinit_openssl_methods(void)
+{
+	BIO_meth_free(tds_method_login);
+	BIO_meth_free(tds_method);
+}
+#  endif
+#endif
 
 
 static SSL_CTX *
@@ -604,11 +664,12 @@ tds_init_openssl(void)
 		tds_mutex_lock(&tls_mutex);
 		if (!tls_initialized) {
 			SSL_library_init();
+			tds_init_ssl_methods();
 			tls_initialized = 1;
 		}
 		tds_mutex_unlock(&tls_mutex);
 	}
-	meth = TLSv1_client_method ();
+	meth = TLS_client_method();
 	if (meth == NULL)
 		return NULL;
 	return SSL_CTX_new (meth);
@@ -699,7 +760,7 @@ check_name_match(ASN1_STRING *name, const char *hostname)
 
 	tdsdump_log(TDS_DBG_INFO1, "Got name %s\n", name_utf8);
 	ret = 0;
-	if (strlen(name_utf8) == name_len && check_wildcard(name_utf8, hostname) == 0)
+	if (strlen(name_utf8) == name_len && check_wildcard(name_utf8, hostname))
 		ret = 1;
 	OPENSSL_free(name_utf8);
 	return ret;
@@ -799,12 +860,8 @@ check_hostname(X509 *cert, const char *hostname)
 int
 tds_ssl_init(TDSSOCKET *tds)
 {
-#define OPENSSL_CIPHERS \
-	"DHE-RSA-AES256-SHA DHE-DSS-AES256-SHA " \
-	"AES256-SHA EDH-RSA-DES-CBC3-SHA " \
-	"EDH-DSS-DES-CBC3-SHA DES-CBC3-SHA " \
-	"DES-CBC3-MD5 DHE-RSA-AES128-SHA " \
-	"DHE-DSS-AES128-SHA AES128-SHA RC2-CBC-MD5 RC4-SHA RC4-MD5"
+#define DEFAULT_OPENSSL_CTX_OPTIONS (SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3)
+#define DEFAULT_OPENSSL_CIPHERS "HIGH:!SSLv2:!aNULL:-DH"
 
 	SSL *con;
 	SSL_CTX *ctx;
@@ -827,6 +884,8 @@ tds_ssl_init(TDSSOCKET *tds)
 	if (!ctx)
 		goto cleanup;
 
+	SSL_CTX_set_options(ctx, DEFAULT_OPENSSL_CTX_OPTIONS);
+
 	if (!tds_dstr_isempty(&tds->login->cafile)) {
 		tls_msg = "loading CA file";
 		if (strcasecmp(tds_dstr_cstr(&tds->login->cafile), "system") == 0)
@@ -856,24 +915,28 @@ tds_ssl_init(TDSSOCKET *tds)
 		goto cleanup;
 
 	tls_msg = "creating bio";
-	b = BIO_new(&tds_method_login);
+	b = BIO_new(tds_method_login);
 	if (!b)
 		goto cleanup;
 
-	b2 = BIO_new(&tds_method);
+	b2 = BIO_new(tds_method);
 	if (!b2)
 		goto cleanup;
 
-	b->shutdown=1;
-	b->init=1;
-	b->num= -1;
-	b->ptr = tds;
+	BIO_set_init(b, 1);
+	BIO_set_data(b, tds);
 	BIO_set_conn_hostname(b, tds_dstr_cstr(&tds->login->server_host_name));
 	SSL_set_bio(con, b, b);
 	b = NULL;
 
-	/* use priorities... */
-	SSL_set_cipher_list(con, OPENSSL_CIPHERS);
+	/* use default priorities unless overridden by openssl ciphers setting in freetds.conf file... */
+	if (!tds_dstr_isempty(&tds->login->openssl_ciphers)) {
+		tdsdump_log(TDS_DBG_INFO1, "setting custom openssl cipher to:%s\n", tds_dstr_cstr(&tds->login->openssl_ciphers));
+		SSL_set_cipher_list(con, tds_dstr_cstr(&tds->login->openssl_ciphers) );
+	} else {
+		tdsdump_log(TDS_DBG_INFO1, "setting default openssl cipher to:%s\n", DEFAULT_OPENSSL_CIPHERS );
+		SSL_set_cipher_list(con, DEFAULT_OPENSSL_CIPHERS);
+	}
 
 #ifdef SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS
 	/* this disable a security improvement but allow connection... */
@@ -883,7 +946,7 @@ tds_ssl_init(TDSSOCKET *tds)
 	/* Perform the TLS handshake */
 	tls_msg = "handshake";
 	SSL_set_connect_state(con);
-	ret = SSL_connect(con) != 1 || con->state != SSL_ST_OK;
+	ret = SSL_connect(con) != 1 || SSL_get_state(con) != TLS_ST_OK;
 	if (ret != 0)
 		goto cleanup;
 
@@ -899,10 +962,8 @@ tds_ssl_init(TDSSOCKET *tds)
 
 	tdsdump_log(TDS_DBG_INFO1, "handshake succeeded!!\n");
 
-	b2->shutdown = 1;
-	b2->init = 1;
-	b2->num = -1;
-	b2->ptr = tds->conn;
+	BIO_set_init(b2, 1);
+	BIO_set_data(b2, tds->conn);
 	SSL_set_bio(con, b2, b2);
 
 	tds->conn->tls_session = con;
