@@ -56,6 +56,13 @@
 		return TDS_FAIL; \
 	tds_set_column_type(tds->conn, col, (TDS_SERVER_TYPE) _tds_type); \
 } while(0)
+
+#define TDS_GET_COLUMN_INFO(tds, col) do { \
+	TDSRET _tds_rc = col->funcs->get_info(tds, col); \
+	if (TDS_FAILED(_tds_rc)) \
+		return _tds_rc; \
+} while(0)
+
 /** \endcond */
 
 static TDSRET tds_process_info(TDSSOCKET * tds, int marker);
@@ -589,7 +596,9 @@ tds_process_tokens(TDSSOCKET *tds, TDS_INT *result_type, int *done_flags, unsign
 				tdsdump_log(TDS_DBG_FUNC, "processing parameters for op %d\n", tds->current_op);
 				while ((marker = tds_get_byte(tds)) == TDS_PARAM_TOKEN) {
 					tdsdump_log(TDS_DBG_INFO1, "calling tds_process_param_result\n");
-					tds_process_param_result(tds, &pinfo);
+					rc = tds_process_param_result(tds, &pinfo);
+					if (TDS_FAILED(rc))
+						goto set_return_exit;
 				}
 				tds_unget_byte(tds);
 				tdsdump_log(TDS_DBG_FUNC, "%d hidden return parameters\n", pinfo ? pinfo->num_cols : -1);
@@ -807,7 +816,7 @@ tds_process_tokens(TDSSOCKET *tds, TDS_INT *result_type, int *done_flags, unsign
 
 	set_return_exit:
 		if (TDS_FAILED(rc)) {
-			tds_set_state(tds, TDS_PENDING);
+			tds_set_state(tds, rc == TDS_CANCELLED ? TDS_PENDING : TDS_DEAD);
 			return rc;
 		}
 
@@ -1027,6 +1036,8 @@ tds_process_col_fmt(TDSSOCKET * tds)
 
 	/* TODO use current_results instead of res_info ?? */
 	info = tds->res_info;
+	if (!info || info->num_cols < 0)
+		return TDS_FAIL;
 	for (col = 0; col < info->num_cols; col++) {
 		curcol = info->columns[col];
 		/* In Sybase all 4 byte are used for usertype, while mssql place 2 byte as usertype and 2 byte as flags */
@@ -1045,7 +1056,7 @@ tds_process_col_fmt(TDSSOCKET * tds)
 		tdsdump_log(TDS_DBG_INFO1, "processing result. type = %d(%s), varint_size %d\n",
 			    curcol->column_type, tds_prtype(curcol->column_type), curcol->column_varint_size);
 
-		curcol->funcs->get_info(tds, curcol);
+		TDS_GET_COLUMN_INFO(tds, curcol);
 
 		/* Adjust column size according to client's encoding */
 		curcol->on_server.column_size = curcol->column_size;
@@ -1262,7 +1273,7 @@ tds_process_param_result(TDSSOCKET * tds, TDSPARAMINFO ** pinfo)
 {
 	TDSCOLUMN *curparam;
 	TDSPARAMINFO *info;
-	TDSRET token;
+	TDSRET token, rc;
 
 	tdsdump_log(TDS_DBG_FUNC, "tds_process_param_result(%p, %p)\n", tds, pinfo);
 
@@ -1284,7 +1295,9 @@ tds_process_param_result(TDSSOCKET * tds, TDSPARAMINFO ** pinfo)
 	 * FIXME check support for tds7+ (seem to use same format of tds5 for data...)
 	 * perhaps varint_size can be 2 or collation can be specified ??
 	 */
-	tds_get_data_info(tds, curparam, 1);
+	rc = tds_get_data_info(tds, curparam, 1);
+	if (TDS_FAILED(rc))
+		return rc;
 
 	curparam->column_cur_size = curparam->column_size;	/* needed ?? */
 
@@ -1318,6 +1331,7 @@ tds_process_param_result_tokens(TDSSOCKET * tds)
 {
 	int marker;
 	TDSPARAMINFO **pinfo;
+	TDSRET rc;
 
 	CHECK_TDS_EXTRA(tds);
 
@@ -1327,7 +1341,9 @@ tds_process_param_result_tokens(TDSSOCKET * tds)
 		pinfo = &(tds->param_info);
 
 	while ((marker = tds_get_byte(tds)) == TDS_PARAM_TOKEN) {
-		tds_process_param_result(tds, pinfo);
+		rc = tds_process_param_result(tds, pinfo);
+		if (TDS_FAILED(rc))
+			return rc;
 	}
 	if (!marker) {
 		tdsdump_log(TDS_DBG_FUNC, "error: tds_process_param_result() returned TDS_FAIL\n");
@@ -1433,7 +1449,7 @@ tds_process_compute_result(TDSSOCKET * tds)
 
 		TDS_GET_COLUMN_TYPE(curcol);
 
-		curcol->funcs->get_info(tds, curcol);
+		TDS_GET_COLUMN_INFO(tds, curcol);
 
 		tdsdump_log(TDS_DBG_INFO1, "compute column_size is %d\n", curcol->column_size);
 
@@ -1490,7 +1506,7 @@ tds7_get_data_info(TDSSOCKET * tds, TDSCOLUMN * curcol)
 
 	curcol->column_timestamp = (curcol->column_type == SYBBINARY && curcol->column_usertype == TDS_UT_TIMESTAMP);
 
-	curcol->funcs->get_info(tds, curcol);
+	TDS_GET_COLUMN_INFO(tds, curcol);
 
 	/* Adjust column size according to client's encoding */
 	curcol->on_server.column_size = curcol->column_size;
@@ -1571,7 +1587,9 @@ tds7_process_result(TDSSOCKET * tds)
 	for (col = 0; col < num_cols; col++) {
 		TDSCOLUMN *curcol = info->columns[col];
 		
-		tds7_get_data_info(tds, curcol);
+		result = tds7_get_data_info(tds, curcol);
+		if (TDS_FAILED(result))
+			return result;
 	}
 		
 	if (num_cols > 0) {
@@ -1673,7 +1691,7 @@ tds_get_data_info(TDSSOCKET * tds, TDSCOLUMN * curcol, int is_param)
 	tdsdump_log(TDS_DBG_INFO1, "processing result. type = %d(%s), varint_size %d\n",
 		    curcol->column_type, tds_prtype(curcol->column_type), curcol->column_varint_size);
 
-	curcol->funcs->get_info(tds, curcol);
+	TDS_GET_COLUMN_INFO(tds, curcol);
 
 	tdsdump_log(TDS_DBG_INFO1, "processing result. column_size %d\n", curcol->column_size);
 
@@ -1696,6 +1714,7 @@ tds5_process_result(TDSSOCKET * tds)
 	unsigned int col, num_cols;
 	TDSCOLUMN *curcol;
 	TDSRESULTINFO *info;
+	TDSRET rc;
 
 	CHECK_TDS_EXTRA(tds);
 
@@ -1722,7 +1741,9 @@ tds5_process_result(TDSSOCKET * tds)
 	for (col = 0; col < info->num_cols; col++) {
 		curcol = info->columns[col];
 
-		tds_get_data_info(tds, curcol, 0);
+		rc = tds_get_data_info(tds, curcol, 0);
+		if (TDS_FAILED(rc))
+			return rc;
 
 		/* skip locale information */
 		/* NOTE do not put into tds_get_data_info, param do not have locale information */
@@ -1825,7 +1846,7 @@ tds5_process_result2(TDSSOCKET * tds)
 
 		TDS_GET_COLUMN_TYPE(curcol);
 
-		curcol->funcs->get_info(tds, curcol);
+		TDS_GET_COLUMN_INFO(tds, curcol);
 
 		/* Adjust column size according to client's encoding */
 		curcol->on_server.column_size = curcol->column_size;
@@ -1907,11 +1928,9 @@ tds_process_row(TDSSOCKET * tds)
 	CHECK_TDS_EXTRA(tds);
 
 	info = tds->current_results;
-	if (!info)
+	if (!info || info->num_cols <= 0)
 		return TDS_FAIL;
 
-	assert(info->num_cols > 0);
-	
 	for (i = 0; i < info->num_cols; i++) {
 		tdsdump_log(TDS_DBG_INFO1, "tds_process_row(): reading column %d \n", i);
 		curcol = info->columns[i];
@@ -1935,11 +1954,9 @@ tds_process_nbcrow(TDSSOCKET * tds)
 	CHECK_TDS_EXTRA(tds);
 
 	info = tds->current_results;
-	if (!info)
+	if (!info || info->num_cols <= 0)
 		return TDS_FAIL;
 
-	assert(info->num_cols > 0);
-
 	nbcbuf = (char *) alloca((info->num_cols + 7) / 8);
 	tds_get_n(tds, nbcbuf, (info->num_cols + 7) / 8);
 	for (i = 0; i < info->num_cols; i++) {
@@ -2067,6 +2084,8 @@ tds_process_end(TDSSOCKET * tds, int marker, int *flags_parm)
 		    "\t\terror = %d\n"
 		    "\t\tdone_count_valid = %d\n", more_results, was_cancelled, error, done_count_valid);
 
+	tds->in_row = false;
+
 	if (tds->res_info) {
 		tds->res_info->more_results = more_results;
 		/* FIXME this should not happen !!! */
@@ -2115,6 +2134,37 @@ tds_process_end(TDSSOCKET * tds, int marker, int *flags_parm)
 	return was_cancelled ? TDS_CANCELLED : TDS_SUCCESS;
 }
 
+static TDSRET
+tds_process_env_routing(TDSSOCKET * tds)
+{
+	unsigned len = tds_get_usmallint(tds);
+	if (len) {
+		/* protocol (byte, 0 for ip)
+		 * port (short, not 0)
+		 * us_varchar
+		 */
+		TDS_TINYINT protocol;
+		TDS_USMALLINT port, address_len;
+		if (len < 5)
+			return TDS_FAIL;
+		protocol = tds_get_byte(tds);
+		port = tds_get_usmallint(tds);
+		address_len = tds_get_usmallint(tds);
+		len -= 5;
+		if (address_len * 2 < len)
+			return TDS_FAIL;
+		if (protocol == 0 && port != 0 && tds->login) {
+			tds->login->routing_port = port;
+			tds_dstr_get(tds, &tds->login->routing_address, address_len);
+			tds_get_n(tds, NULL, len - 2 * address_len);
+		} else {
+			tds_get_n(tds, NULL, len);
+		}
+	}
+	tds_get_n(tds, NULL, tds_get_usmallint(tds));
+	return TDS_SUCCESS;
+}
+
 /**
  * tds_process_env_chg() 
  * when ever certain things change on the server, such as database, character
@@ -2192,6 +2242,9 @@ tds_process_env_chg(TDSSOCKET * tds)
 		return TDS_SUCCESS;
 	}
 
+	if (IS_TDS71_PLUS(tds->conn) && type == TDS_ENV_ROUTING)
+		return tds_process_env_routing(tds);
+
 	/* discard byte values, not still supported */
 	/* TODO support them */
 	if (IS_TDS71_PLUS(tds->conn) && type > TDS_ENV_PACKSIZE) {
@@ -2269,6 +2322,9 @@ tds_process_info(TDSSOCKET * tds, int marker)
 
 	CHECK_TDS_EXTRA(tds);
 
+	if (!tds->in_row)
+		tds_free_all_results(tds);
+
 	/* make sure message has been freed */
 	memset(&msg, 0, sizeof(TDSMESSAGE));
 
@@ -2549,6 +2605,7 @@ tds_process_dyn_result(TDSSOCKET * tds)
 	TDSCOLUMN *curcol;
 	TDSPARAMINFO *info;
 	TDSDYNAMIC *dyn;
+	TDSRET rc;
 
 	CHECK_TDS_EXTRA(tds);
 
@@ -2571,7 +2628,9 @@ tds_process_dyn_result(TDSSOCKET * tds)
 	for (col = 0; col < info->num_cols; col++) {
 		curcol = info->columns[col];
 
-		tds_get_data_info(tds, curcol, 1);
+		rc = tds_get_data_info(tds, curcol, 1);
+		if (TDS_FAILED(rc))
+			return rc;
 
 		/* skip locale information */
 		tds_get_n(tds, NULL, tds_get_byte(tds));
@@ -2628,7 +2687,7 @@ tds5_process_dyn_result2(TDSSOCKET * tds)
 		/* column type */
 		TDS_GET_COLUMN_TYPE(curcol);
 
-		curcol->funcs->get_info(tds, curcol);
+		TDS_GET_COLUMN_INFO(tds, curcol);
 
 		/* Adjust column size according to client's encoding */
 		curcol->on_server.column_size = curcol->column_size;
@@ -2745,9 +2804,14 @@ tds7_process_compute_result(TDSSOCKET * tds)
 	TDS_USMALLINT compute_id;
 	TDSCOLUMN *curcol;
 	TDSCOMPUTEINFO *info;
+	TDSRET rc;
 
 	CHECK_TDS_EXTRA(tds);
 
+	/* compute without result should never happens */
+	if (!tds->res_info)
+		return TDS_FAIL;
+
 	/*
 	 * number of compute columns returned - so
 	 * COMPUTE SUM(x), AVG(x)... would return
@@ -2807,7 +2871,9 @@ tds7_process_compute_result(TDSSOCKET * tds)
 		curcol->column_operator = tds_get_byte(tds);
 		curcol->column_operand = tds_get_smallint(tds);
 
-		tds7_get_data_info(tds, curcol);
+		rc = tds7_get_data_info(tds, curcol);
+		if (TDS_FAILED(rc))
+			return rc;
 
 		if (tds_dstr_isempty(&curcol->column_name))
 			if (!tds_dstr_copy(&curcol->column_name, tds_pr_op(curcol->column_operator)))
@@ -2879,7 +2945,8 @@ tds5_process_optioncmd(TDSSOCKET * tds)
 
 	tdsdump_log(TDS_DBG_INFO1, "tds5_process_optioncmd()\n");
 
-	assert(IS_TDS50(tds->conn));
+	if (!IS_TDS50(tds->conn))
+		return TDS_FAIL;
 
 	tds_get_usmallint(tds);	/* length */
 	command = tds_get_byte(tds);
