@@ -1,8 +1,6 @@
+#define TDS_DONT_DEFINE_DEFAULT_FUNCTIONS
 #include "common.h"
 
-static char software_version[] = "$Id: common.c,v 1.30 2010/06/19 07:53:21 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 char USER[512];
 char SERVER[512];
 char PASSWORD[512];
@@ -46,7 +44,7 @@ read_login_info(void)
 		}
 	}
 	fclose(in);
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 TDSCONTEXT *test_context = NULL;
@@ -54,7 +52,7 @@ TDSCONTEXT *test_context = NULL;
 int
 try_tds_login(TDSLOGIN ** login, TDSSOCKET ** tds, const char *appname, int verbose)
 {
-	TDSCONNECTION *connection;
+	TDSLOGIN *connection;
 
 	if (verbose) {
 		fprintf(stdout, "Entered tds_try_login()\n");
@@ -76,19 +74,22 @@ try_tds_login(TDSLOGIN ** login, TDSSOCKET ** tds, const char *appname, int verb
 	if (verbose) {
 		fprintf(stdout, "Setting login parameters\n");
 	}
-	*login = tds_alloc_login();
+	*login = tds_alloc_login(1);
 	if (!*login) {
 		fprintf(stderr, "tds_alloc_login() failed.\n");
 		return TDS_FAIL;
 	}
-	tds_set_passwd(*login, PASSWORD);
-	tds_set_user(*login, USER);
-	tds_set_app(*login, appname);
-	tds_set_host(*login, "myhost");
-	tds_set_library(*login, "TDS-Library");
-	tds_set_server(*login, SERVER);
-	tds_set_client_charset(*login, CHARSET);
-	tds_set_language(*login, "us_english");
+	if (!tds_set_passwd(*login, PASSWORD)
+	    || !tds_set_user(*login, USER)
+	    || !tds_set_app(*login, appname)
+	    || !tds_set_host(*login, "myhost")
+	    || !tds_set_library(*login, "TDS-Library")
+	    || !tds_set_server(*login, SERVER)
+	    || !tds_set_client_charset(*login, CHARSET)
+	    || !tds_set_language(*login, "us_english")) {
+		fprintf(stderr, "tds_alloc_login() failed.\n");
+		return TDS_FAIL;
+	}
 
 	if (verbose) {
 		fprintf(stdout, "Connecting to database\n");
@@ -97,18 +98,18 @@ try_tds_login(TDSLOGIN ** login, TDSSOCKET ** tds, const char *appname, int verb
 	*tds = tds_alloc_socket(test_context, 512);
 	tds_set_parent(*tds, NULL);
 	connection = tds_read_config_info(*tds, *login, test_context->locale);
-	if (!connection || tds_connect_and_login(*tds, connection) != TDS_SUCCEED) {
+	if (!connection || tds_connect_and_login(*tds, connection) != TDS_SUCCESS) {
 		if (connection) {
 			tds_free_socket(*tds);
 			*tds = NULL;
-			tds_free_connection(connection);
+			tds_free_login(connection);
 		}
 		fprintf(stderr, "tds_connect_and_login() failed\n");
 		return TDS_FAIL;
 	}
-	tds_free_connection(connection);
+	tds_free_login(connection);
 
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 
@@ -119,11 +120,12 @@ try_tds_logout(TDSLOGIN * login, TDSSOCKET * tds, int verbose)
 	if (verbose) {
 		fprintf(stdout, "Entered tds_try_logout()\n");
 	}
+	tds_close_socket(tds);
 	tds_free_socket(tds);
 	tds_free_login(login);
 	tds_free_context(test_context);
 	test_context = NULL;
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
 
 /* Run query for which there should be no return results */
@@ -134,12 +136,12 @@ run_query(TDSSOCKET * tds, const char *query)
 	int result_type;
 
 	rc = tds_submit_query(tds, query);
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "tds_submit_query() failed for query '%s'\n", query);
 		return TDS_FAIL;
 	}
 
-	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 
 		switch (result_type) {
 		case TDS_DONE_RESULT:
@@ -161,5 +163,5 @@ run_query(TDSSOCKET * tds, const char *query)
 		return TDS_FAIL;
 	}
 
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
