@@ -25,17 +25,12 @@
  */
 #include "common.h"
 #include <assert.h>
-#include <tdsconvert.h>
+#include <freetds/convert.h>
 #include "replacements.h"
 
-#if HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
+#include <freetds/time.h>
 
-static char software_version[] = "$Id: convert.c,v 1.23 2007/12/23 21:12:02 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-int g_result = 0;
+static int g_result = 0;
 static TDSCONTEXT *ctx;
 
 static void
@@ -50,22 +45,11 @@ free_convert(int type, CONV_RESULT *cr)
 	}
 }
 
-extern const char STD_DATETIME_FMT[];
-
 int
 main(int argc, char **argv)
 {
-	/* the conversion pair matrix */
-	typedef struct
-	{
-		int srctype;
-		int desttype;
-		int yn;
-	}
-	ANSWER;
-	const static ANSWER answers[] = {
-#	include "tds_willconvert.h"
-	};
+	int srctype;
+	int desttype;
 
 	/* some default inputs */
 	static const int bit_input = 1;
@@ -85,20 +69,33 @@ main(int argc, char **argv)
 	TDS_MONEY4 money4;
 	TDS_DATETIME datetime;
 	TDS_DATETIME4 datetime4;
+	TDS_DATETIMEALL dta;
+	TDS_DATE date;
+	TDS_TIME time;
+	TDS_BIGDATETIME bigdatetime;
+	TDS_BIGTIME bigtime;
 
 	TDS_TINYINT tds_tinyint;
 	TDS_SMALLINT tds_smallint;
 	TDS_INT tds_int;
 	TDS_INT8 tds_int8;
+	TDS_USMALLINT tds_usmallint;
+	TDS_UINT tds_uint;
+	TDS_UINT8 tds_uint8;
 
 	TDS_REAL tds_real;
 	TDS_FLOAT tds_float;
 
+	TDS_UNIQUE tds_unique;
+
 	if (argc > 1) {
 		iterations = atoi(argv[1]);
 		printf("Computing %d iterations\n", iterations);
 	}
 
+	setbuf(stdout, NULL);
+	setbuf(stderr, NULL);
+
 	ctx = tds_alloc_context(NULL);
 	assert(ctx);
 	if (ctx->locale && !ctx->locale->date_fmt) {
@@ -110,24 +107,52 @@ main(int argc, char **argv)
 	/*
 	 * Test every possible conversion pair
 	 */
-	for (i = 0; i < sizeof(answers) / sizeof(ANSWER); i++) {
-		if (!answers[i].yn)
-			continue;	/* don't attempt nonconvertible types */
+	for (i = 0; i < 0x10000; i++) {
+		srctype  = i >> 8;
+		desttype = i & 0xff;
+		srctype = (srctype + SYBCHAR) & 0xff;
 
-		if (answers[i].srctype == answers[i].desttype)
+		if (!tds_willconvert(srctype, desttype)) {
+			/* pass a CONV_RESULT as input for make sure size and alignment is enough
+			 * for all types */
+			CONV_RESULT src;
+
+			memset(&src, 0, sizeof(src));
+			result = tds_convert(ctx, srctype, (const TDS_CHAR *) &src, 4, desttype, &cr);
+			if (result >= 0)
+				free_convert(desttype, &cr);
+			if (result != TDS_CONVERT_NOAVAIL) {
+				printf("NOT EXPECTED: converted %d (%s, %d bytes) : %d (%s, %d bytes).\n",
+				       srctype, tds_prtype(srctype), srclen,
+				       desttype, tds_prtype(desttype), result);
+				exit(1);
+			}
+			continue;	/* don't attempt nonconvertible types */
+		}
+
+		if (srctype == desttype)
 			continue;	/* don't attempt same types */
 
+		/* valid types should have a name ! */
+		assert(tds_prtype(srctype)[0] != 0);
+		assert(tds_prtype(desttype)[0] != 0);
+
 		cr.n.precision = 8;
 		cr.n.scale = 2;
 
-		switch (answers[i].srctype) {
+		switch (srctype) {
 		case SYBCHAR:
 		case SYBVARCHAR:
 		case SYBTEXT:
 		case SYBBINARY:
 		case SYBVARBINARY:
 		case SYBIMAGE:
-			switch (answers[i].desttype) {
+		case SYBLONGBINARY:
+		case XSYBBINARY:
+		case XSYBVARBINARY:
+		case XSYBCHAR:
+		case XSYBVARCHAR:
+			switch (desttype) {
 			case SYBCHAR:
 			case SYBVARCHAR:
 			case SYBTEXT:
@@ -135,6 +160,22 @@ main(int argc, char **argv)
 			case SYBDATETIME4:
 				src = "Jan  1, 1999";
 				break;
+			case SYBMSDATE:
+			case SYBMSDATETIME2:
+			case SYBMSDATETIMEOFFSET:
+			case SYBDATE:
+				src = "2012-11-27";
+				break;
+			case SYBMSTIME:
+			case SYBTIME:
+				src = "15:27:12";
+				break;
+			case SYB5BIGTIME:
+				src = "15:27:12.327862";
+				break;
+			case SYB5BIGDATETIME:
+				src = "2015-09-12 21:48:12.638161";
+				break;
 			case SYBBINARY:
 			case SYBIMAGE:
 				src = "0xbeef";
@@ -143,6 +184,10 @@ main(int argc, char **argv)
 			case SYBINT2:
 			case SYBINT4:
 			case SYBINT8:
+			case SYBUINT1:
+			case SYBUINT2:
+			case SYBUINT4:
+			case SYBUINT8:
 				src = "255";
 				break;
 			case SYBFLT8:
@@ -155,6 +200,9 @@ main(int argc, char **argv)
 				cr.n.precision = 8;
 				cr.n.scale = 2;
 				break;
+			case SYBUNIQUE:
+				src = "A8C60F70-5BD4-3E02-B769-7CCCCA585DCC";
+				break;
 			case SYBBIT:
 			default:
 				src = "1";
@@ -164,6 +212,7 @@ main(int argc, char **argv)
 			srclen = strlen(src);
 			break;
 		case SYBINT1:
+		case SYBUINT1:
 			src = (char *) &tds_tinyint;
 			srclen = sizeof(tds_tinyint);
 			break;
@@ -179,13 +228,25 @@ main(int argc, char **argv)
 			src = (char *) &tds_int8;
 			srclen = sizeof(tds_int8);
 			break;
+		case SYBUINT2:
+			src = (char *) &tds_usmallint;
+			srclen = sizeof(tds_usmallint);
+			break;
+		case SYBUINT4:
+			src = (char *) &tds_uint;
+			srclen = sizeof(tds_uint);
+			break;
+		case SYBUINT8:
+			src = (char *) &tds_uint8;
+			srclen = sizeof(tds_uint8);
+			break;
 		case SYBFLT8:
 			tds_float = 3.14159;
 			src = (char *) &tds_float;
 			srclen = sizeof(tds_float);
 			break;
 		case SYBREAL:
-			tds_real = 3.14159;
+			tds_real = (TDS_REAL) 3.14159;
 			src = (char *) &tds_real;
 			srclen = sizeof(tds_real);
 			break;
@@ -203,6 +264,7 @@ main(int argc, char **argv)
 			srclen = sizeof(money4);
 			break;
 		case SYBBIT:
+		case SYBBITN:
 			src = (char *) &bit_input;
 			srclen = sizeof(bit_input);
 			break;
@@ -214,15 +276,42 @@ main(int argc, char **argv)
 			src = (char *) &datetime4;
 			srclen = sizeof(datetime4);
 			break;
+		case SYBDATE:
+			src = (char *) &date;
+			srclen = sizeof(date);
+			break;
+		case SYBTIME:
+			src = (char *) &time;
+			srclen = sizeof(time);
+			break;
+		case SYB5BIGTIME:
+			src = (char *) &bigtime;
+			srclen = sizeof(bigtime);
+			break;
+		case SYB5BIGDATETIME:
+			src = (char *) &bigdatetime;
+			srclen = sizeof(bigdatetime);
+			break;
+		case SYBUNIQUE:
+			src = (char *) &tds_unique;
+			srclen = sizeof(tds_unique);
+			break;
+		case SYBMSTIME:
+		case SYBMSDATE:
+		case SYBMSDATETIME2:
+		case SYBMSDATETIMEOFFSET:
+			src = (char *) &dta;
+			srclen = sizeof(dta);
+			break;
 		/*****  not defined yet
 			case SYBBOUNDARY:
 			case SYBSENSITIVITY:
-				fprintf (stderr, "type %d not supported\n", answers[i].srctype );
+				fprintf (stderr, "type %d not supported\n", srctype );
 				continue;
 				break;
 		*****/
 		default:
-			fprintf(stderr, "no such type %d\n", answers[i].srctype);
+			fprintf(stderr, "no such type %d (%s)\n", srctype, tds_prtype(srctype));
 			return -1;
 		}
 
@@ -230,8 +319,9 @@ main(int argc, char **argv)
 		 * Now at last do the conversion
 		 */
 
-		result = tds_convert(ctx, answers[i].srctype, src, srclen, answers[i].desttype, &cr);
-		free_convert(answers[i].desttype, &cr);
+		result = tds_convert(ctx, srctype, src, srclen, desttype, &cr);
+		if (result >= 0)
+			free_convert(desttype, &cr);
 
 		if (result < 0) {
 			if (result == TDS_CONVERT_NOAVAIL)	/* tds_willconvert returned true, but it lied. */
@@ -239,23 +329,23 @@ main(int argc, char **argv)
 
 			fprintf(stderr, "failed (%d) to convert %d (%s, %d bytes) : %d (%s).\n",
 				result,
-				answers[i].srctype, tds_prtype(answers[i].srctype), srclen,
-				answers[i].desttype, tds_prtype(answers[i].desttype));
+				srctype, tds_prtype(srctype), srclen,
+				desttype, tds_prtype(desttype));
 
-			if (result != TDS_CONVERT_NOAVAIL)
-				return result;
+			if (result == TDS_CONVERT_NOAVAIL)
+				exit(1);
 		}
 
-		printf("converted %d (%s, %d bytes) : %d (%s, %d bytes).\n",
-		       answers[i].srctype, tds_prtype(answers[i].srctype), srclen,
-		       answers[i].desttype, tds_prtype(answers[i].desttype), result);
+		printf("converted %d (%s, %d bytes) -> %d (%s, %d bytes).\n",
+		       srctype, tds_prtype(srctype), srclen,
+		       desttype, tds_prtype(desttype), result);
 
 		/* 
 		 * In the first iteration, start with varchar -> others.  
 		 * By saving the output, we initialize subsequent inputs.
 		 */
 
-		switch (answers[i].desttype) {
+		switch (desttype) {
 		case SYBNUMERIC:
 		case SYBDECIMAL:
 			numeric = cr.n;
@@ -272,7 +362,23 @@ main(int argc, char **argv)
 		case SYBDATETIME4:
 			datetime4 = cr.dt4;
 			break;
+		case SYBDATE:
+			date = cr.date;
+			break;
+		case SYBTIME:
+			time = cr.time;
+			break;
+		case SYBMSDATETIME2:
+			dta = cr.dta;
+			break;
+		case SYB5BIGTIME:
+			bigtime = cr.bigtime;
+			break;
+		case SYB5BIGDATETIME:
+			bigdatetime = cr.bigdatetime;
+			break;
 		case SYBINT1:
+		case SYBUINT1:
 			tds_tinyint = cr.ti;
 			break;
 		case SYBINT2:
@@ -284,6 +390,18 @@ main(int argc, char **argv)
 		case SYBINT8:
 			tds_int8 = cr.bi;
 			break;
+		case SYBUINT2:
+			tds_usmallint = cr.usi;
+			break;
+		case SYBUINT4:
+			tds_uint = cr.ui;
+			break;
+		case SYBUINT8:
+			tds_uint8 = cr.ubi;
+			break;
+		case SYBUNIQUE:
+			tds_unique = cr.u;
+			break;
 		default:
 			break;
 		}
@@ -296,8 +414,9 @@ main(int argc, char **argv)
 		starttime = (double) start.tv_sec + (double) start.tv_usec * 0.000001;
 
 		for (j = 0; result >= 0 && j < iterations; j++) {
-			result = tds_convert(ctx, answers[i].srctype, src, srclen, answers[i].desttype, &cr);
-			free_convert(answers[i].desttype, &cr);
+			result = tds_convert(ctx, srctype, src, srclen, desttype, &cr);
+			if (result >= 0)
+				free_convert(desttype, &cr);
 		}
 		if (result < 0)
 			continue;
@@ -307,7 +426,7 @@ main(int argc, char **argv)
 
 		if (endtime != starttime) {
 			printf("%9.0f iterations/second converting %13s => %s.\n",
-				j / (endtime - starttime), tds_prtype(answers[i].srctype), tds_prtype(answers[i].desttype));
+				j / (endtime - starttime), tds_prtype(srctype), tds_prtype(desttype));
 		}
 
 	}
