@@ -17,13 +17,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/*
+ * Test data from server
+ * The test do a select for a given type, convert to char and compare result
+ */
 #include "common.h"
 
 #include <assert.h>
-#include <tdsconvert.h>
-
-static char software_version[] = "$Id: dataread.c,v 1.20 2009/05/28 16:23:32 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/convert.h>
 
 static int g_result = 0;
 static TDSLOGIN *login;
@@ -40,7 +41,7 @@ test(const char *type, const char *value, const char *result)
 static void
 exec_query(const char *query)
 {
-	if (tds_submit_query(tds, query) != TDS_SUCCEED || tds_process_simple_query(tds) != TDS_SUCCEED) {
+	if (tds_submit_query(tds, query) != TDS_SUCCESS || tds_process_simple_query(tds) != TDS_SUCCESS) {
                 fprintf(stderr, "executing query failed\n");
                 exit(1);
         }
@@ -85,12 +86,12 @@ test0(const char *type, ...)
 
 	/* execute it */
 	rc = tds_submit_query(tds, "SELECT * FROM #tmp");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "tds_submit_query() failed\n");
 		exit(1);
 	}
 
-	if (tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS) != TDS_SUCCEED) {
+	if (tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS) != TDS_SUCCESS) {
 		fprintf(stderr, "tds_process_tokens() failed\n");
 		exit(1);
 	}
@@ -100,7 +101,7 @@ test0(const char *type, ...)
 		exit(1);
 	}
 
-	if (tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS) != TDS_SUCCEED) {
+	if (tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS) != TDS_SUCCESS) {
 		fprintf(stderr, "tds_process_tokens() failed\n");
 		exit(1);
 	}
@@ -111,7 +112,7 @@ test0(const char *type, ...)
 	}
 
 	i_row = 0;
-	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_STOPAT_ROWFMT|TDS_RETURN_DONE|TDS_RETURN_ROW|TDS_RETURN_COMPUTE)) == TDS_SUCCEED && (result_type == TDS_ROW_RESULT || result_type == TDS_COMPUTE_RESULT)) {
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_STOPAT_ROWFMT|TDS_RETURN_DONE|TDS_RETURN_ROW|TDS_RETURN_COMPUTE)) == TDS_SUCCESS && (result_type == TDS_ROW_RESULT || result_type == TDS_COMPUTE_RESULT)) {
 
 		TDSCOLUMN *curcol = tds->current_results->columns[0];
 		TDS_CHAR *src = (TDS_CHAR *) curcol->column_data;
@@ -119,7 +120,7 @@ test0(const char *type, ...)
 
 		assert(i_row < num_data);
 
-		if (is_blob_col(curcol)) {
+		if (is_blob_col(curcol) && curcol->column_type != SYBVARIANT) {
 			TDSBLOB *blob = (TDSBLOB *) src;
 
 			src = blob->textvalue;
@@ -138,12 +139,12 @@ test0(const char *type, ...)
 		++i_row;
 	}
 
-	if (rc != TDS_NO_MORE_RESULTS && rc != TDS_SUCCEED) {
+	if (rc != TDS_NO_MORE_RESULTS && rc != TDS_SUCCESS) {
 		fprintf(stderr, "tds_process_tokens() unexpected return\n");
 		exit(1);
 	}
 
-	while ((rc = tds_process_tokens(tds, &result_type, &done_flags, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, &done_flags, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 		switch (result_type) {
 		case TDS_NO_MORE_RESULTS:
 			return;
@@ -168,7 +169,7 @@ int
 main(int argc, char **argv)
 {
 	fprintf(stdout, "%s: Testing conversion from server\n", __FILE__);
-	if (try_tds_login(&login, &tds, __FILE__, 0) != TDS_SUCCEED) {
+	if (try_tds_login(&login, &tds, __FILE__, 0) != TDS_SUCCESS) {
 		fprintf(stderr, "try_tds_login() failed\n");
 		return 1;
 	}
@@ -183,8 +184,8 @@ main(int argc, char **argv)
 	test("INT", "16909060", NULL);
 
 	/* floating point */
-	test("REAL", "1.23", NULL);
-	test("FLOAT", "-49586.345", NULL);
+	test("REAL", "1.25", NULL);
+	test("FLOAT", "-49586.345", "-49586.345000000001");
 
 	/* money */
 	test("MONEY", "-123.3400", "-123.34");
@@ -222,13 +223,17 @@ main(int argc, char **argv)
 	test("DATETIME", "2003-04-21 17:50:03", NULL);
 	test("SMALLDATETIME", "2003-04-21 17:50:03", "2003-04-21 17:50:00");
 
-	if (IS_TDS7_PLUS(tds)) {
+	if (IS_TDS7_PLUS(tds->conn)) {
 		test("UNIQUEIDENTIFIER", "12345678-1234-A234-9876-543298765432", NULL);
 		test("NVARCHAR(20)", "Excellent test", NULL);
 		test("NCHAR(20)", "Excellent test", "Excellent test      ");
 		test("NTEXT", "Excellent test", NULL);
 	}
 
+	if (IS_TDS71_PLUS(tds->conn)) {
+		test("SQL_VARIANT", "test123", NULL);
+	}
+
 	try_tds_logout(login, tds, 0);
 	return g_result;
 }
