@@ -18,24 +18,19 @@
  */
 #include "common.h"
 
-static char software_version[] = "$Id: t0002.c,v 1.12 2003/09/25 21:14:25 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/string.h>
+#include <replacements.h>
 
-char *value_as_string(TDSSOCKET * tds, int col_idx);
-
-char *
+static char *
 value_as_string(TDSSOCKET * tds, int col_idx)
 {
 	static char result[256];
 	const int type = tds->res_info->columns[col_idx]->column_type;
-	const unsigned char *row = tds->res_info->current_row;
-	const int offset = tds->res_info->columns[col_idx]->column_offset;
-	const void *value = (row + offset);
+	const void *value = tds->res_info->columns[col_idx]->column_data;
 
 	switch (type) {
 	case SYBVARCHAR:
-		strncpy(result, (const char *) value, sizeof(result) - 1);
-		result[sizeof(result) - 1] = '\0';
+		strlcpy(result, (const char *) value, sizeof(result));
 		break;
 	case SYBINT4:
 		sprintf(result, "%d", *(const int *) value);
@@ -56,25 +51,23 @@ main(int argc, char **argv)
 	int verbose = 0;
 	int num_cols = 2;
 	TDS_INT result_type;
-	TDS_INT row_type;
-	TDS_INT compute_id;
 	int rc;
 	int i, done_flags;
 
 	fprintf(stdout, "%s: Test basic submit query, results\n", __FILE__);
 	rc = try_tds_login(&login, &tds, __FILE__, verbose);
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "try_tds_login() failed\n");
 		return 1;
 	}
 
 	rc = tds_submit_query(tds, "select db_name() dbname, user_name() username");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "tds_submit_query() failed\n");
 		return 1;
 	}
 
-	while ((rc = tds_process_result_tokens(tds, &result_type, &done_flags)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, &done_flags, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 		switch (result_type) {
 		case TDS_ROWFMT_RESULT:
 			if (tds->res_info->num_cols != num_cols) {
@@ -86,8 +79,8 @@ main(int argc, char **argv)
 				fprintf(stderr, "Wrong column_type in %s\n", __FILE__);
 				return 1;
 			}
-			if (strcmp(tds->res_info->columns[0]->column_name, "dbname")
-			    || strcmp(tds->res_info->columns[1]->column_name, "username")) {
+			if (strcmp(tds_dstr_cstr(&tds->res_info->columns[0]->column_name), "dbname")
+			    || strcmp(tds_dstr_cstr(&tds->res_info->columns[1]->column_name), "username")) {
 				fprintf(stderr, "Wrong column_name in %s\n", __FILE__);
 				return 1;
 			}
@@ -95,15 +88,17 @@ main(int argc, char **argv)
 
 		case TDS_ROW_RESULT:
 
-			while ((rc = tds_process_row_tokens(tds, &row_type, &compute_id)) == TDS_SUCCEED) {
+			while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_STOPAT_ROWFMT|TDS_RETURN_DONE|TDS_RETURN_ROW|TDS_RETURN_COMPUTE)) == TDS_SUCCESS) {
+				if (result_type != TDS_ROW_RESULT || result_type != TDS_COMPUTE_RESULT)
+					break;
 				if (verbose) {
 					for (i = 0; i < num_cols; i++) {
 						printf("col %i is %s\n", i, value_as_string(tds, i));
 					}
 				}
 			}
-			if (rc != TDS_NO_MORE_ROWS) {
-				fprintf(stderr, "tds_process_row_tokens() unexpected return\n");
+			if (rc != TDS_SUCCESS) {
+				fprintf(stderr, "tds_process_tokens() unexpected return\n");
 			}
 			break;
 
@@ -114,12 +109,12 @@ main(int argc, char **argv)
 				break;
 
 		default:
-			fprintf(stderr, "tds_process_result_tokens() unexpected result_type\n");
+			fprintf(stderr, "tds_process_tokens() unexpected result_type\n");
 			break;
 		}
 	}
 	if (rc != TDS_NO_MORE_RESULTS) {
-		fprintf(stderr, "tds_process_result_tokens() unexpected return\n");
+		fprintf(stderr, "tds_process_tokens() unexpected return\n");
 	}
 
 	try_tds_logout(login, tds, verbose);
