@@ -17,22 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 #include "common.h"
+#include <replacements.h>
 
-static char software_version[] = "$Id: t0004.c,v 1.17 2003/09/25 21:14:25 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-char *varchar_as_string(TDSSOCKET * tds, int col_idx);
-
-char *
+static char *
 varchar_as_string(TDSSOCKET * tds, int col_idx)
 {
 	static char result[256];
-	const unsigned char *row = tds->res_info->current_row;
-	const int offset = tds->res_info->columns[col_idx]->column_offset;
-	const void *value = (row + offset);
+	const void *value = tds->res_info->columns[col_idx]->column_data;
 
-	strncpy(result, (const char *) value, sizeof(result) - 1);
-	result[sizeof(result) - 1] = '\0';
+	strlcpy(result, (const char *) value, sizeof(result));
 	return result;
 }
 
@@ -43,24 +36,23 @@ main(int argc, char **argv)
 	TDSLOGIN *login;
 	TDSSOCKET *tds;
 	int verbose = 0;
-	int rc;
+	int rc, i;
 
 	int result_type;
-	int row_type;
-	int compute_id;
 	int rows_returned = 0;
 
 	const char *len200 =
 		"01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789";
-	char long_query[1000];
+	char *long_query = (char*) malloc(5000);
 
-	sprintf(long_query,
-		"SELECT name FROM #longquerytest WHERE (name = 'A%s' OR name = 'B%s' OR name = 'C%s' OR name = 'correct')", len200,
-		len200, len200);
+	strcpy(long_query, "SELECT name FROM #longquerytest WHERE (");
+	for (i = 0; i < 20; ++i)
+		sprintf(strchr(long_query, 0), "name = '%c%s' OR ", 'A'+i, len200);
+	strcat(long_query, "name = 'correct')");
 
-	fprintf(stdout, "%s: Test large (>512 bytes) queries\n", __FILE__);
+	fprintf(stdout, "%s: Test large (>4096 bytes) queries\n", __FILE__);
 	rc = try_tds_login(&login, &tds, __FILE__, verbose);
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "try_tds_login() failed\n");
 		return 1;
 	}
@@ -68,15 +60,15 @@ main(int argc, char **argv)
 	/* do not check error here, if TABLE is not create this give error */
 	rc = run_query(tds, "DROP TABLE #longquerytest");
 	rc = run_query(tds, "CREATE TABLE #longquerytest (name varchar(255))");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		return 1;
 	}
 	rc = run_query(tds, "INSERT #longquerytest (name) VALUES ('incorrect')");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		return 1;
 	}
 	rc = run_query(tds, "INSERT #longquerytest (name) VALUES ('correct')");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		return 1;
 	}
 
@@ -84,10 +76,10 @@ main(int argc, char **argv)
 	 * The heart of the test
 	 */
 	if (verbose) {
-		fprintf(stdout, "block size %d\n", tds->env->block_size);
+		fprintf(stdout, "block size %d\n", tds->conn->env.block_size);
 	}
 	rc = tds_submit_query(tds, long_query);
-	while ((rc = tds_process_result_tokens(tds, &result_type, NULL)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_RETURN_ROWFMT|TDS_RETURN_ROW)) == TDS_SUCCESS) {
 		switch (result_type) {
 		case TDS_ROWFMT_RESULT:
 			if (tds->res_info->columns[0]->column_type != SYBVARCHAR) {
@@ -96,18 +88,9 @@ main(int argc, char **argv)
 			}
 			break;
 		case TDS_ROW_RESULT:
-			while ((rc = tds_process_row_tokens(tds, &row_type, &compute_id)) == TDS_SUCCEED) {
-				++rows_returned;
-				if (verbose) {
-					printf("col 0 is %s\n", varchar_as_string(tds, 0));
-				}
-			}
-			if (rc == TDS_FAIL) {
-				fprintf(stderr, "tds_process_row_tokens() returned TDS_FAIL\n");
-				return 1;
-			} else if (rc != TDS_NO_MORE_ROWS) {
-				fprintf(stderr, "tds_process_row_tokens() unexpected return\n");
-				return 1;
+			++rows_returned;
+			if (verbose) {
+				printf("col 0 is %s\n", varchar_as_string(tds, 0));
 			}
 			break;
 		default:
@@ -115,10 +98,10 @@ main(int argc, char **argv)
 		}
 	}
 	if (rc == TDS_FAIL) {
-		fprintf(stderr, "tds_process_result_tokens() returned TDS_FAIL for long query\n");
+		fprintf(stderr, "tds_process_tokens() returned TDS_FAIL for long query\n");
 		return 1;
 	} else if (rc != TDS_NO_MORE_RESULTS) {
-		fprintf(stderr, "tds_process_result_tokens() unexpected return\n");
+		fprintf(stderr, "tds_process_tokens() unexpected return\n");
 	}
 
 	if (rows_returned != 1) {
@@ -130,5 +113,6 @@ main(int argc, char **argv)
 	rc = run_query(tds, "DROP TABLE #longquerytest");
 
 	try_tds_logout(login, tds, verbose);
+	free(long_query);
 	return 0;
 }
