@@ -6,8 +6,12 @@
 
 #include <windows.h>
 
+#ifdef DLL_EXPORT
+
 HINSTANCE hinstFreeTDS;
 
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
+
 BOOL WINAPI
 DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
@@ -28,6 +32,8 @@ DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 
 		if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0)
 			return FALSE;
+
+		DisableThreadLibraryCalls(hinstDLL);
 		break;
 
 	case DLL_PROCESS_DETACH:
@@ -38,3 +44,6 @@ DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 	}
 	return TRUE;
 }
+
+#endif
+
